// Downloaded from https://repo.progsbase.com - Code Developed Using progsbase.

function Add(a, b){
  var m, n;
  var r, c;

  r = NumberOfRows(a);
  c = NumberOfColumns(a);
  for(m = 0; m < r; m = m + 1){
    for(n = 0; n < c; n = n + 1){
      a.r[m].c[n] = Element(a, m, n) + Element(b, m, n);
    }
  }
}
function Assign(A, B){
  var m, n;
  var r, c;

  r = NumberOfRows(A);
  c = NumberOfColumns(A);
  for(m = 0; m < r; m = m + 1){
    for(n = 0; n < c; n = n + 1){
      A.r[m].c[n] = Element(B, m, n);
    }
  }
}
function Resize(A, r, c){
  var m, n, ar, ac;
  var C;

  C = CreateMatrix(r, c);

  ar = NumberOfRows(A);
  ac = NumberOfColumns(A);

  for(m = 0; m < Math.min(r, ar); m = m + 1){
    for(n = 0; n < Math.min(c, ac); n = n + 1){
      C.r[m].c[n] = Element(A, m, n);
    }
  }

  FreeMatrixRows(A.r);
  A.r = C.r;
}
function Subtract(a, b){
  var m, n;
  var r, c;

  r = NumberOfRows(a);
  c = NumberOfColumns(a);
  for(m = 0; m < r; m = m + 1){
    for(n = 0; n < c; n = n + 1){
      a.r[m].c[n] = Element(a, m, n) - Element(b, m, n);
    }
  }
}
function SubtractToNew(a, b){
  var X;

  X = CreateCopyOfMatrix(a);
  Subtract(X, b);

  return X;
}
function ScalarMultiply(A, b){
  var m, n;
  var r, c;

  r = NumberOfRows(A);
  c = NumberOfColumns(A);
  for(m = 0; m < r; m = m + 1){
    for(n = 0; n < c; n = n + 1){
      A.r[m].c[n] = b*A.r[m].c[n];
    }
  }
}
function ScalarDivide(A, b){
  var m, n;
  var r, c;

  r = NumberOfRows(A);
  c = NumberOfColumns(A);
  for(m = 0; m < r; m = m + 1){
    for(n = 0; n < c; n = n + 1){
      A.r[m].c[n] = Element(A, m, n)/b;
    }
  }
}
function ElementWisePower(A, p){
  var m, n;
  var r, c;

  r = NumberOfRows(A);
  c = NumberOfColumns(A);

  for(m = 0; m < r; m = m + 1){
    for(n = 0; n < c; n = n + 1){
      A.r[m].c[n] = Math.pow(A.r[m].c[n], p);
    }
  }
}
function ScalarMultiplyToNew(A, b){
  var matrix;

  matrix = CreateCopyOfMatrix(A);
  ScalarMultiply(matrix, b);

  return matrix;
}
function MultiplyToNew(a, b){
  var rows, cols;
  var x;

  rows = NumberOfRows(a);
  cols = NumberOfColumns(b);
  x = CreateMatrix(rows, cols);
  Multiply(x, a, b);

  return x;
}
function Multiply(x, a, b){
  var m, n;
  var rows, cols, d;
  var i, s;

  rows = NumberOfRows(a);
  cols = NumberOfColumns(b);
  d = NumberOfColumns(a);

  for(m = 0; m < rows; m = m + 1){
    for(n = 0; n < cols; n = n + 1){
      s = 0;

      for(i = 0; i < d; i = i + 1){
        s = s + a.r[m].c[i]*b.r[i].c[n];
      }

      x.r[m].c[n] = s;
    }
  }
}
function CreateSquareMatrix(d){
  var m, n;
  var matrix;

  matrix = {};
  matrix.r = [];
  matrix.r.length = d;
  for(m = 0; m < d; m = m + 1){
    matrix.r[m] = {};
    matrix.r[m].c = [];
    matrix.r[m].c.length = d;
    for(n = 0; n < d; n = n + 1){
      matrix.r[m].c[n] = 0;
    }
  }

  return matrix;
}
function CreateMatrix(rows, cols){
  var m, n;
  var matrix;

  matrix = {};
  matrix.r = [];
  matrix.r.length = rows;
  for(m = 0; m < rows; m = m + 1){
    matrix.r[m] = {};
    matrix.r[m].c = [];
    matrix.r[m].c.length = cols;
    for(n = 0; n < cols; n = n + 1){
      matrix.r[m].c[n] = 0;
    }
  }

  return matrix;
}
function CreateIdentityMatrix(d){
  var m;
  var matrix;

  matrix = CreateSquareMatrix(d);
  Fill(matrix, 0);

  for(m = 0; m < d; m = m + 1){
    matrix.r[m].c[m] = 1;
  }

  return matrix;
}
function Transpose(a){
  var ap;

  ap = TransposeToNew(a);

  FreeMatrixRows(a.r);
  a.r = ap.r;
}
function TransposeAssign(t, a){
  var m, n;
  var rows, cols;

  cols = NumberOfRows(a);
  rows = NumberOfColumns(a);

  for(m = 0; m < cols; m = m + 1){
    for(n = 0; n < rows; n = n + 1){
      t.r[n].c[m] = a.r[m].c[n];
    }
  }
}
function TransposeToNew(a){
  var m, n;
  var rows, cols;
  var c;

  cols = NumberOfRows(a);
  rows = NumberOfColumns(a);

  c = CreateMatrix(rows, cols);

  for(m = 0; m < cols; m = m + 1){
    for(n = 0; n < rows; n = n + 1){
      c.r[n].c[m] = a.r[m].c[n];
    }
  }

  return c;
}
function CofactorOfMatrix(mat, temp, p, q, n){
  var i, j;
  var row, col;

  i = 0;
  j = 0;

  for(row = 0; row < n; row = row + 1){
    for(col = 0; col < n; col = col + 1){
      if(row != p && col != q){
        temp.r[i].c[j] = mat.r[row].c[col];
        j = j + 1;

        if(j == n - 1){
          j = 0;
          i = i + 1;
        }
      }
    }
  }
}
function DeterminantOfSubmatrix(mat, n){
  var D, f, sign;
  var temp;

  D = 0;

  if(n == 1){
    D = mat.r[0].c[0];
  }else{
    temp = CreateSquareMatrix(n);

    sign = 1;

    for(f = 0; f < n; f = f + 1){
      CofactorOfMatrix(mat, temp, 0, f, n);
      D = D + sign*mat.r[0].c[f]*DeterminantOfSubmatrix(temp, n - 1);
      sign =  -sign;
    }

    FreeMatrix(temp);
  }

  return D;
}
function Determinant(m){
  var D, n;

  n = NumberOfRows(m);
  D = DeterminantOfSubmatrix(m, n);

  return D;
}
function Adjoint(A, adj){
  var n, sign;
  var cofactors;
  var i, j;

  n = A.r.length;

  if(n == 1){
    adj.r[0].c[0] = 1;
  }else{
    cofactors = CreateSquareMatrix(n);

    for(i = 0; i < n; i = i + 1){
      for(j = 0; j < n; j = j + 1){
        CofactorOfMatrix(A, cofactors, i, j, n);

        if((i + j)%2 == 0){
          sign = 1;
        }else{
          sign =  -1;
        }

        adj.r[j].c[i] = sign*DeterminantOfSubmatrix(cofactors, n - 1);
      }
    }

    FreeMatrix(cofactors);
  }
}
function Inverse(A, inverseResult){
  return InverseUsingLUDecomposition(A, inverseResult);
}
function InverseUsingAdjoint(A, inverseResult){
  var success;
  var adj;
  var n, i, j;
  var det;

  if(NumberOfColumns(A) == NumberOfRows(A)){
    n = NumberOfColumns(A);

    det = Determinant(A);
    if(det != 0){
      adj = CreateSquareMatrix(n);
      Adjoint(A, adj);

      for(i = 0; i < n; i = i + 1){
        for(j = 0; j < n; j = j + 1){
          inverseResult.r[i].c[j] = adj.r[i].c[j]/det;
        }
      }

      success = true;
      FreeMatrix(adj);
    }else{
      success = false;
    }
  }else{
    success = false;
  }

  return success;
}
function InverseUsingLUDecomposition(A, inverseResult){
  var success;
  var l, u, li, ui;

  l = CreateCopyOfMatrix(A);
  u = CreateCopyOfMatrix(A);
  li = CreateCopyOfMatrix(A);
  ui = CreateCopyOfMatrix(A);
  inverseResult.r = CreateCopyOfMatrix(A).r;

  success = LUDecomposition(A, l, u);
  if(success){
    success = InvertLowerTriangularMatrix(l, li);
    if(success){
      success = InvertUpperTriangularMatrix(u, ui);
      if(success){
        Multiply(inverseResult, ui, li);
      }
    }
  }

  FreeMatrix(l);
  FreeMatrix(u);
  FreeMatrix(li);
  FreeMatrix(ui);

  return success;
}
function LUDecomposition(A, L, U){
  var n, i, j, k, sum;
  var success;

  n = NumberOfRows(A);

  L.r = CreateSquareMatrix(n).r;
  U.r = CreateSquareMatrix(n).r;

  if(IsSquare(A)){
    success = true;

    for(i = 0; i < n && success; i = i + 1){
      for(k = i; k < n; k = k + 1){
        sum = 0;
        for(j = 0; j < i; j = j + 1){
          sum = sum + (Element(L, i, j)*Element(U, j, k));
        }

        U.r[i].c[k] = Element(A, i, k) - sum;
      }

      for(k = i; k < n && success; k = k + 1){
        if(i == k){
          L.r[i].c[i] = 1;
        }else{
          sum = 0;
          for(j = 0; j < i; j = j + 1){
            sum = sum + (Element(L, k, j)*Element(U, j, i));
          }

          if(Element(U, i, i) == 0){
            success = false;
          }else{
            L.r[k].c[i] = (Element(A, k, i) - sum)/Element(U, i, i);
          }
        }
      }
    }
  }else{
    success = false;
  }

  return success;
}
function IsSymmetric(A){
  var N;
  var i, j;
  var is, done;

  N = NumberOfRows(A);

  done = false;
  is = true;
  for(i = 0; i < N &&  !done ; i = i + 1){
    for(j = 0; j < i &&  !done ; j = j + 1){
      if(A.r[i].c[j] != A.r[j].c[i]){
        is = false;
        done = true;
      }
    }
  }

  return is;
}
function IsSquare(A){
  var is;

  if(NumberOfRows(A) == NumberOfColumns(A)){
    is = true;
  }else{
    is = false;
  }

  return is;
}
function Cholesky(A, L){
  var success;
  var N;
  var i, j, k, s;

  Clear(L);

  if(IsSquare(A) && IsSymmetric(A)){
    success = true;

    N = NumberOfRows(A);

    for(i = 0; i < N && success; i = i + 1){
      for(j = 0; j <= i && success; j = j + 1){
        s = 0;
        for(k = 0; k < j; k = k + 1){
          s = s + L.r[i].c[k]*L.r[j].c[k];
        }
        if(i == j){
          L.r[i].c[i] = Math.sqrt(A.r[i].c[i] - s);
        }else{
          L.r[i].c[j] = 1/L.r[j].c[j]*(A.r[i].c[j] - s);
        }
      }
      if(L.r[i].c[i] <= 0){
        success = false;
      }
    }

    success = true;
  }else{
    success = false;
  }

  return success;
}
function Clear(a){
  Fill(a, 0);
}
function Fill(a, value){
  var m, n;

  for(m = 0; m < NumberOfRows(a); m = m + 1){
    for(n = 0; n < NumberOfColumns(a); n = n + 1){
      a.r[m].c[n] = value;
    }
  }
}
function Element(matrix, m, n){
  return matrix.r[m].c[n];
}
function Trace(a){
  var m;
  var d, tr;

  tr = 0;

  d = a.r.length;
  for(m = 0; m < d; m = m + 1){
    tr = tr + a.r[m].c[m];
  }

  return tr;
}
function ColumnCombineMatricesToNew(A, B){
  var X;
  var m, n;

  X = CreateMatrix(NumberOfRows(A), NumberOfColumns(A) + NumberOfColumns(B));

  for(m = 0; m < NumberOfRows(A); m = m + 1){
    for(n = 0; n < NumberOfColumns(A); n = n + 1){
      X.r[m].c[n] = A.r[m].c[n];
    }
  }

  for(m = 0; m < NumberOfRows(B); m = m + 1){
    for(n = 0; n < NumberOfColumns(B); n = n + 1){
      X.r[m].c[NumberOfColumns(A) + n] = B.r[m].c[n];
    }
  }

  return X;
}
function NumberOfRows(A){
  return A.r.length;
}
function NumberOfColumns(A){
  return A.r[0].c.length;
}
function CharacteristicPolynomial(A){
  var dummy;
  var coeffs;
  var determinant;

  dummy = CreateSquareMatrix(NumberOfRows(A));

  coeffs = {};
  determinant = {};
  CharacteristicPolynomialWithInverse(A, dummy, coeffs, determinant);

  FreeMatrix(dummy);

  return coeffs.numberArray;
}
function CharacteristicPolynomialWithInverse(A, AInverse, cp, determinant){
  FaddeevLeVerrierAlgorithm(A, AInverse, cp, determinant);
}
function FaddeevLeVerrierAlgorithm(A, AInverse, cp, determinant){
  var p;
  var Mk, Mkm1, t1, I;
  var n, k;

  n = NumberOfRows(A);
  p = [];
  p.length = n + 1;
  p[n] = 1;
  Mkm1 = CreateSquareMatrix(n);
  Fill(Mkm1, 0);
  I = CreateIdentityMatrix(n);
  Mk = CreateSquareMatrix(n);
  t1 = CreateSquareMatrix(n);

  for(k = 1; k <= n; k = k + 1){
    /* M_k = A * M_(k-1) + c_(n-k+1) * I */
    Multiply(Mk, A, Mkm1);
    Assign(t1, I);
    ScalarMultiply(t1, p[n - k + 1]);
    Add(Mk, t1);

    /* c_(n-k) = -1/k * trace(A * M_k) */
    Multiply(t1, A, Mk);
    p[n - k] =  -1/k*Trace(t1);

    /* done */
    Assign(Mkm1, Mk);

    if(k == n){
      Assign(AInverse, Mk);
      determinant.numberValue =  -p[0];
      if(p[0] == 0){
      }else{
        ScalarDivide(AInverse, determinant.numberValue);
      }
    }
  }

  FreeMatrix(Mkm1);
  FreeMatrix(I);
  FreeMatrix(Mk);
  FreeMatrix(t1);

  cp.numberArray = p;
}
function InverseUsingCharacteristicPolynomial(A){
  var inverse;
  var coeffs;
  var determinant;

  inverse = CreateSquareMatrix(NumberOfRows(A));
  coeffs = {};
  determinant = {};
  CharacteristicPolynomialWithInverse(A, inverse, coeffs, determinant);
  delete(coeffs.numberArray);
  delete(coeffs);

  return inverse;
}
function Eigenvalues(A, eigenValuesReference){
  var eigenVectorsReference;
  var success;
  var i;

  eigenVectorsReference = {};
  success = Eigenpairs(A, eigenValuesReference, eigenVectorsReference);
  if(success){
    for(i = 0; i < eigenVectorsReference.matrices.length; i = i + 1){
      FreeMatrix(eigenVectorsReference.matrices[i]);
    }
    delete(eigenVectorsReference.matrices);
    delete(eigenVectorsReference);
  }

  return success;
}
function EigenvaluesUsingQRAlgorithm(A, eigenValuesReference, precision, maxIterations){
  var x, q, r;
  var success;
  var i, n, v, v1, v2, ev, found;
  var cp;

  n = NumberOfRows(A);
  x = CreateSquareMatrix(n);
  q = CreateSquareMatrix(n);
  r = CreateSquareMatrix(n);
  eigenValuesReference.numberArray = [];
  eigenValuesReference.numberArray.length = n;
  success = QRAlgorithm(A, r, x, q, precision, maxIterations);
  found = 0;
  if(success){
    ExtractDiagonal(x, eigenValuesReference.numberArray);

    /* find the correct sign of the eigenvalue. */
    cp = CharacteristicPolynomial(A);
    for(i = 0; i < n; i = i + 1){
      ev = eigenValuesReference.numberArray[i];

      v1 = pEvaluate(cp, ev);
      v2 = pEvaluate(cp,  -ev);

      if(Math.abs(v2) < Math.abs(v1)){
        eigenValuesReference.numberArray[i] =  -ev;
        v = v2;
      }else{
        v = v1;
      }

      if(Math.abs(v) < precision*Math.pow(10, 4)){
        found = found + 1;
      }
    }

    FreeMatrix(x);
    FreeMatrix(q);
    FreeMatrix(r);
  }

  if(found != n){
    success = false;
  }

  return success;
}
function EigenvaluesUsingLaguerreIterations(A, eigenValuesReference){
  var p;
  var success;

  p = CharacteristicPolynomial(A);
  success = FindRoots(p, eigenValuesReference);

  return success;
}
function GaussianElimination(A){
  var h, k, m, n, maxElement, i, j, max, maxCandidate, f;

  m = NumberOfRows(A);
  n = NumberOfColumns(A);

  h = 0;
  k = 0;
  for(; h < m && k < n; ){
    maxElement = h;
    max = 0;
    for(i = h; i < m; i = i + 1){
      maxCandidate = Math.abs(Element(A, i, k));
      if(max < maxCandidate){
        maxElement = i;
        max = maxCandidate;
      }
    }
    if(A.r[maxElement].c[k] == 0){
      k = k + 1;
    }else{
      SwapRows(A, h, maxElement);
      for(i = h + 1; i < m; i = i + 1){
        f = Element(A, i, k)/Element(A, h, k);
        A.r[i].c[k] = 0;
        for(j = k + 1; j < n; j = j + 1){
          A.r[i].c[j] = Element(A, i, j) - Element(A, h, j)*f;
        }
      }
      h = h + 1;
      k = k + 1;
    }
  }
}
function GaussianEliminationToNew(A){
  var X;

  X = CreateCopyOfMatrix(A);
  GaussianElimination(X);

  return X;
}
function CreateCopyOfMatrix(A){
  var X;

  X = CreateMatrix(NumberOfRows(A), NumberOfColumns(A));
  Assign(X, A);

  return X;
}
function SwapRows(A, to, from){
  var n;
  var c, t;

  c = NumberOfRows(A);
  for(n = 0; n < c; n = n + 1){
    t = A.r[to].c[n];
    A.r[to].c[n] = A.r[from].c[n];
    A.r[from].c[n] = t;
  }
}
function UnnormalizeVector(numberArray){
  var i, m;
  var mSet;

  mSet = false;
  m = 0;

  for(i = 0; i < numberArray.length; i = i + 1){
    if(numberArray[i] - mathTruncate(numberArray[i]) < 0.001){
      if( !mSet ){
        m = Math.abs(numberArray[i]);
        mSet = true;
      }else{
        m = Math.min(m, Math.abs(numberArray[i]));
      }
    }
  }

  if(mSet){
    for(i = 0; i < numberArray.length; i = i + 1){
      numberArray[i] = numberArray[i]/m;
    }
  }
}
function InversePowerMethod(A, eigenvalue, maxIterations, eigenvector){
  var x, y, z, b, t;
  var n, i, c;
  var singular;

  n = NumberOfRows(A);

  x = CreateIdentityMatrix(n);
  ScalarMultiply(x, eigenvalue);
  y = SubtractToNew(A, x);
  z = CreateSquareMatrix(n);
  singular =  !Inverse(y, z) ;
  if(singular){
    /* Try again with more erroneous eigenvalue estimate. */
    x = CreateIdentityMatrix(n);
    ScalarMultiply(x, eigenvalue*1.01);
    y = SubtractToNew(A, x);
    z = CreateSquareMatrix(n);
    singular =  !Inverse(y, z) ;
  }

  if( !singular ){
    b = CreateMatrix(n, 1);

    for(i = 0; i < n; i = i + 1){
      b.r[i].c[0] = 1;
    }

    for(i = 0; i < maxIterations; i = i + 1){
      t = MultiplyToNew(z, b);
      c = Norm(t);
      ScalarDivide(t, c);
      Assign(b, t);
    }

    eigenvector.numberArray = [];
    eigenvector.numberArray.length = n;
    for(i = 0; i < n; i = i + 1){
      eigenvector.numberArray[i] = b.r[i].c[0];
    }
  }

  return  !singular ;
}
function Eigenvectors(A, eigenVectorsReference){
  var evsReference;
  var success;

  evsReference = {};
  success = Eigenpairs(A, evsReference, eigenVectorsReference);
  if(success){
    delete(evsReference.numberArray);
    delete(evsReference);
  }

  return success;
}
function Eigenpairs(A, eigenValuesReference, eigenVectorsReference){
  return EigenpairsUsingQRAlgorithmAndInversePowerMethod(A, eigenValuesReference, eigenVectorsReference, 0.00000000001, 100);
}
function EigenpairsUsingQRAlgorithmAndInversePowerMethod(M, eigenValuesReference, eigenVectorsReference, precision, maxIterations){
  var evecReference;
  var done, inverseSuccess;
  var i, j, k, N, v1, v2, eigenValue, withinPrecision;
  var A, Q, R, eigenVector;
  var cp;

  N = NumberOfRows(M);

  A = CreateCopyOfMatrix(M);
  Q = CreateCopyOfMatrix(M);
  R = CreateCopyOfMatrix(M);

  done = false;
  eigenVectorsReference.matrices = [];
  eigenVectorsReference.matrices.length = N;
  evecReference = {};
  eigenValuesReference.numberArray = [];
  eigenValuesReference.numberArray.length = N;
  cp = CharacteristicPolynomial(M);

  for(j = 0; j < N; j = j + 1){
    eigenVectorsReference.matrices[j] = CreateMatrix(N, 1);
  }

  for(i = 0; i < maxIterations &&  !done ; i = i + 1){
    QRDecomposition(A, Q, R);
    Multiply(A, R, Q);

    /* Check */
    withinPrecision = 0;
    ExtractDiagonal(R, eigenValuesReference.numberArray);

    for(j = 0; j < N; j = j + 1){
      /* Find the correct sign of the eigenvalue. */
      eigenValue = eigenValuesReference.numberArray[j];
      v1 = pEvaluate(cp, eigenValue);
      v2 = pEvaluate(cp,  -eigenValue);
      if(Math.abs(v2) < Math.abs(v1)){
        eigenValuesReference.numberArray[j] =  -eigenValue;
        eigenValue =  -eigenValue;
      }

      /* Calculate the eigenvector corresponding to the eigenvalue. */
      inverseSuccess = InversePowerMethod(M, eigenValue, i + 1, evecReference);
      if(inverseSuccess){
        for(k = 0; k < N; k = k + 1){
          eigenVectorsReference.matrices[j].r[k].c[0] = evecReference.numberArray[k];
        }

        /* Check eigenpair agains precision. */
        eigenVector = eigenVectorsReference.matrices[j];

        if(CheckEigenpairPrecision(M, eigenValue, eigenVector, precision)){
          withinPrecision = withinPrecision + 1;
        }
      }
    }

    if(withinPrecision == N){
      done = true;
    }
  }

  FreeMatrix(A);
  FreeMatrix(Q);
  FreeMatrix(R);
  delete(evecReference);
  delete(cp);

  return done;
}
function CheckEigenpairPrecision(a, lambda, e, precision){
  var vec1, vec2;
  var equal;

  vec1 = MultiplyToNew(a, e);
  vec2 = ScalarMultiplyToNew(e, lambda);

  equal = MatrixEqualsEpsilon(vec1, vec2, precision);

  return equal;
}
function EigenvectorsLaguerreIterationsAndGaussianEliminations(A, eigenVectorsReference){
  var Id, t1, B, v;
  var eigenVectorsResult;
  var success;
  var eigenValuesReference;
  var i, lambda, j, N, x, k;
  var ev;

  N = NumberOfRows(A);

  eigenValuesReference = {};
  success = Eigenvalues(A, eigenValuesReference);

  eigenVectorsResult = [];
  eigenVectorsResult.length = N;

  if(success){
    ev = eigenValuesReference.numberArray;

    Id = CreateIdentityMatrix(N);
    t1 = CreateSquareMatrix(N);
    B = CreateSquareMatrix(N);

    for(j = 0; j < ev.length && success; j = j + 1){
      lambda = ev[j];

      /* B = A - lambda * Id */
      Assign(t1, Id);
      ScalarMultiply(t1, lambda);

      Assign(B, A);
      Subtract(B, t1);

      GaussianElimination(B);

      v = CreateMatrix(N, 1);
      v.r[N - 1].c[0] = 1;
      for(i = N - 2; i >= 0 && success; i = i - 1){
        if( !RowIsZero(B, i) ){
          x = 0;

          for(k = N - 1; k > i; k = k - 1){
            x = x - Element(B, i, k)*Element(v, k, 0);
          }

          v.r[i].c[0] = x/Element(B, i, i);
        }else{
          success = false;
        }
      }

      eigenVectorsResult[j] = v;
    }

    FreeMatrix(t1);
    FreeMatrix(B);
    FreeMatrix(Id);

    eigenVectorsReference.matrices = eigenVectorsResult;
  }

  return success;
}
function RowIsZero(X, r){
  var isZero;
  var columns, i;

  isZero = true;

  columns = NumberOfColumns(X);
  for(i = 0; i < columns && isZero; i = i + 1){
    if(Element(X, r, i) != 0){
      isZero = false;
    }
  }

  return isZero;
}
function FreeMatrix(X){
  FreeMatrixRows(X.r);
  delete(X);
}
function FreeMatrixRows(r){
  var m, rows;

  rows = r.length;
  for(m = 0; m < rows; m = m + 1){
    delete(r[m].c);
    delete(r[m]);
  }

  delete(r);
}
function CreateDiagonalMatrixFromArray(array){
  var m;
  var matrix;

  matrix = CreateSquareMatrix(array.length);
  Fill(matrix, 0);

  for(m = 0; m < array.length; m = m + 1){
    matrix.r[m].c[m] = array[m];
  }

  return matrix;
}
function CreateMatrixFromRowCopies(row, times){
  var m, n;
  var matrix;

  matrix = CreateMatrix(times, row.length);

  for(m = 0; m < times; m = m + 1){
    for(n = 0; n < row.length; n = n + 1){
      matrix.r[m].c[n] = row[n];
    }
  }

  return matrix;
}
function ExtractDiagonal(X, diag){
  var n, i;

  n = NumberOfRows(X);

  for(i = 0; i < n; i = i + 1){
    diag[i] = X.r[i].c[i];
  }
}
function ExtractDiagonalToNew(X){
  var diag;
  var n, i;

  n = NumberOfRows(X);
  diag = [];
  diag.length = n;

  for(i = 0; i < n; i = i + 1){
    diag[i] = X.r[i].c[i];
  }

  return diag;
}
function MatrixEqualsEpsilon(a, b, epsilon){
  var x, y, columns, rows;
  var equals;

  equals = true;

  if(NumberOfRows(a) == NumberOfRows(b) && NumberOfColumns(a) == NumberOfColumns(b)){
    columns = NumberOfColumns(a);
    rows = NumberOfRows(a);

    for(x = 0; x < rows; x = x + 1){
      for(y = 0; y < columns; y = y + 1){
        equals = equals && mathEpsilonCompare(Element(a, x, y), Element(b, x, y), epsilon);
      }
    }
  }else{
    equals = false;
  }

  return equals;
}
function Minor(x, row, column){
  var theMinor;
  var cols, rows, i, j, m, n;

  rows = NumberOfRows(x) - 1;
  cols = NumberOfColumns(x) - 1;

  theMinor = CreateMatrix(rows, cols);

  for(i = 0; i < rows; i = i + 1){
    if(i < row){
      m = i;
    }else{
      m = i + 1;
    }

    for(j = 0; i != row && j < cols; j = j + 1){
      if(j != column){

        if(j < column){
          n = j;
        }else{
          n = j + 1;
        }

        theMinor.r[m].c[n] = x.r[i].c[j];
      }
    }
  }

  return theMinor;
}
function QRDecomposition(m, Q, R){
  HouseholderMethod(m, Q, R);
}
function HouseholderTriangularizationAlgorithm(m, Qout, Rout){
  var P, AA, A, PP, v, vt, t;
  var i, j, rows, cols, r, s;

  rows = NumberOfRows(m);
  cols = NumberOfColumns(m);

  P = CreateIdentityMatrix(rows);
  A = CreateCopyOfMatrix(m);
  AA = CreateMatrix(rows, cols);

  t = CreateMatrix(rows, cols);
  PP = CreateIdentityMatrix(rows);
  vt = CreateMatrix(1, rows);

  for(j = 0; j < cols; j = j + 1){
    v = ExtractSubMatrix(A, 0, rows - 1, j, j);
    if(j > 0){
      for(i = 0; i < j; i = i + 1){
        v.r[i].c[0] = 0;
      }
    }

    s = mathSign(v.r[j].c[0]);
    if(s == 0){
      s = 1;
    }
    v.r[j].c[0] = v.r[j].c[0] + Norm(v)*s;
    r =  -2/(Norm(v)*Norm(v));
    Assign(AA, A);

    TransposeAssign(vt, v);
    Multiply(t, vt, A);
    Assign(A, t);
    Multiply(t, v, A);
    Assign(A, t);
    ScalarMultiply(A, r);
    Assign(t, AA);
    Add(A, AA);

    Assign(PP, P);

    Multiply(t, vt, P);
    Assign(P, t);
    Multiply(t, v, P);
    Assign(P, t);
    ScalarMultiply(P, r);
    Assign(t, AA);
    Add(P, PP);
  }

  Assign(Rout, A);
  Assign(Qout, P);
  Transpose(Qout);
}
function HouseholderMethod(A, q, r){
  var QR, R, Q;
  var Rdiag;
  var m, n;
  var i, j, k;
  var s, nrm;
  var e;
  var N, ra, rq;

  /* Initialize. */
  QR = CreateCopyOfMatrix(A);
  m = NumberOfRows(A);
  n = NumberOfColumns(A);
  Rdiag = [];
  Rdiag.length = n;

  /* Main loop. */
  for(k = 0; k < n; k = k + 1){
    /* Compute 2-norm of k-th column without under/overflow. */
    nrm = 0;
    for(i = k; i < m; i = i + 1){
      nrm = Hypothenuse(nrm, Element(QR, i, k));
    }

    if(nrm != 0){
      /* Form k-th Householder vector. */
      if(Element(QR, k, k) < 0){
        nrm =  -nrm;
      }
      for(i = k; i < m; i = i + 1){
        QR.r[i].c[k] = Element(QR, i, k)/nrm;
      }
      QR.r[k].c[k] = Element(QR, k, k) + 1;

      /* Apply transformation to remaining columns. */
      for(j = k + 1; j < n; j = j + 1){
        s = 0;
        for(i = k; i < m; i = i + 1){
          s = s + Element(QR, i, k)*Element(QR, i, j);
        }
        s =  -s/Element(QR, k, k);
        for(i = k; i < m; i = i + 1){
          QR.r[i].c[j] = Element(QR, i, j) + s*Element(QR, i, k);
        }
      }
    }
    Rdiag[k] =  -nrm;
  }

  /* Compute R */
  R = CreateSquareMatrix(n);
  for(i = 0; i < n; i = i + 1){
    for(j = 0; j < n; j = j + 1){
      if(i < j){
        R.r[i].c[j] = Element(QR, i, j);
      }else if(i == j){
        R.r[i].c[j] = Rdiag[i];
      }else{
        R.r[i].c[j] = 0;
      }
    }
  }
  Assign(r, R);

  /* Compute Q */
  Q = CreateMatrix(m, n);
  for(k = n - 1; k >= 0; k = k - 1){
    for(i = 0; i < m; i = i + 1){
      Q.r[i].c[k] = 0;
    }
    Q.r[k].c[k] = 1;
    for(j = k; j < n; j = j + 1){
      if(Element(QR, k, k) != 0){
        s = 0;
        for(i = k; i < m; i = i + 1){
          s = s + Element(QR, i, k)*Element(Q, i, j);
        }
        s =  -s/Element(QR, k, k);
        for(i = k; i < m; i = i + 1){
          Q.r[i].c[j] = Element(Q, i, j) + s*Element(QR, i, k);
        }
      }
    }
  }
  Assign(q, Q);

  /* Adjust for positive R. */
  n = NumberOfRows(r);

  N = CreateIdentityMatrix(n);

  for(i = 0; i < n; i = i + 1){
    e = Element(r, i, i);

    if(e < 0){
      N.r[i].c[i] =  -1;
    }
  }

  ra = MultiplyToNew(N, r);
  Assign(r, ra);
  rq = MultiplyToNew(q, N);
  Assign(q, rq);

  FreeMatrix(ra);
  FreeMatrix(rq);
  FreeMatrix(Q);
  FreeMatrix(R);
  FreeMatrix(QR);
}
function Hypothenuse(a, b){
  return Math.sqrt(Math.pow(a, 2) + Math.pow(b, 2));
}
function Norm(a){
  var l, i, j, rows, cols;

  l = 0;

  rows = NumberOfRows(a);
  cols = NumberOfColumns(a);

  for(i = 0; i < rows; i = i + 1){
    for(j = 0; j < cols; j = j + 1){
      l = l + a.r[i].c[j]*a.r[i].c[j];
    }
  }
  l = Math.sqrt(l);

  return l;
}
function ExtractSubMatrix(M, r1, r2, c1, c2){
  var A;
  var i, j;

  A = CreateMatrix(r2 - r1 + 1, c2 - c1 + 1);

  for(i = r1; i <= r2; i = i + 1){
    for(j = c1; j <= c2; j = j + 1){
      A.r[i - r1].c[j - c1] = M.r[i].c[j];
    }
  }

  return A;
}
function QRAlgorithm(M, R, A, Q, precision, maxIterations){
  var n, i, j, v;
  var previous;
  var withinPrecision;
  var done, previousSet;

  Assign(A, M);
  n = NumberOfRows(M);
  previous = [];
  previous.length = n;
  previousSet = false;

  done = false;
  for(i = 0; i < maxIterations &&  !done ; i = i + 1){
    QRDecomposition(A, Q, R);
    Multiply(A, R, Q);

    /* Check precision. */
    if(previousSet){
      withinPrecision = 0;
      for(j = 0; j < n; j = j + 1){
        v = previous[j] - Element(A, j, j);
        if(Math.abs(v) < precision || v == 0){
          withinPrecision = withinPrecision + 1;
        }
      }
      if(withinPrecision == n){
        done = true;
      }
    }

    for(j = 0; j < n; j = j + 1){
      previous[j] = Element(A, j, j);
    }
    previousSet = true;
  }

  return done;
}
function InvertUpperTriangularMatrix(A, inverse){
  var sum, i, j, k, n;
  var success;

  inverse.r = CreateCopyOfMatrix(A).r;
  n = NumberOfRows(inverse);
  success = true;

  for(i = n - 1; i >= 0 && success; i = i - 1){
    if(Element(inverse, i, i) == 0){
      success = false;
    }else{
      inverse.r[i].c[i] = 1/Element(inverse, i, i);
      for(j = i - 1; j >= 0 && success; j = j - 1){
        sum = 0;
        for(k = i; k > j; k = k - 1){
          sum = sum - Element(inverse, j, k)*Element(inverse, k, i);
        }
        if(Element(inverse, j, j) == 0){
          success = false;
        }else{
          inverse.r[j].c[i] = sum/Element(inverse, j, j);
        }
      }
    }
  }

  return success;
}
function InvertLowerTriangularMatrix(A, inverse){
  var sum, i, j, k, n;
  var success;

  inverse.r = CreateCopyOfMatrix(A).r;
  n = NumberOfRows(inverse);
  success = true;

  for(i = 0; i < n && success; i = i + 1){
    if(Element(inverse, i, i) == 0){
      success = false;
    }else{
      inverse.r[i].c[i] = 1/Element(inverse, i, i);
      for(j = i + 1; j < n; j = j + 1){
        sum = 0;
        for(k = i; k < j && success; k = k + 1){
          sum = sum - Element(inverse, j, k)*Element(inverse, k, i);
        }
        if(Element(inverse, j, j) == 0){
          success = false;
        }else{
          inverse.r[j].c[i] = sum/Element(inverse, j, j);
        }
      }
    }
  }

  return success;
}
function ParseMatrixFromString(aref, matrixString, errorMessage){
  var success;
  var lines;
  var rows, cols, i;
  var row;
  var replaced, trimmed;

  replaced = ReplaceString(matrixString, "\r".split(''), "".split(''));
  trimmed = Trim(replaced);
  lines = SplitByCharacter(trimmed, '\n');

  delete(replaced);
  delete(trimmed);

  success = true;

  rows = lines.length;
  if(rows == 0){
    aref.matrix = CreateMatrix(0, 0);
  }else{
    row = nStringToNumberArray(lines[0].string);
    cols = row.length;
    delete(row);

    aref.matrix = CreateMatrix(rows, cols);

    for(i = 0; i < rows && success; i = i + 1){
      delete(aref.matrix.r[i].c);
      aref.matrix.r[i].c = nStringToNumberArray(lines[i].string);

      if(aref.matrix.r[i].c.length != cols){
        success = false;
        errorMessage.string = "All rows must have the same number of columns.".split('');
      }
    }
  }

  FreeStringReferenceArray(lines);

  return success;
}
function FreeStringReferenceArray(stringReferencesArray){
  var i;
  for(i = 0; i < stringReferencesArray.length; i = i + 1){
    delete(stringReferencesArray[i]);
  }
  delete(stringReferencesArray);
}
function MatrixToString(matrix, digitsAfterPoint){
  var s1, s2;
  var n, m, element;

  s1 = [];
  s1.length = 0;

  for(n = 0; n < NumberOfRows(matrix); n = n + 1){
    for(m = 0; m < NumberOfColumns(matrix); m = m + 1){
      element = Element(matrix, n, m);
      element = RoundToDigits(element, digitsAfterPoint);
      s2 = AppendString(s1, nCreateStringDecimalFromNumber(element));
      delete(s1);
      s1 = s2;
      if(m + 1 != NumberOfColumns(matrix)){
        s2 = AppendString(s1, ", ".split(''));
        delete(s1);
        s1 = s2;
      }
    }
    s2 = AppendString(s1, "\n".split(''));
    delete(s1);
    s1 = s2;
  }

  return s1;
}
function RoundToDigits(element, digitsAfterPoint){
  return mathRound(element*Math.pow(10, digitsAfterPoint))/Math.pow(10, digitsAfterPoint);
}
function MatrixArrayToString(matrices, digitsAfterPoint){
  var s1, s2;
  var i;

  s1 = [];
  s1.length = 0;

  for(i = 0; i < matrices.length; i = i + 1){
    s2 = AppendString(s1, MatrixToString(matrices[i], digitsAfterPoint));
    delete(s1);
    s1 = s2;

    s2 = AppendString(s1, "\n".split(''));
    delete(s1);
    s1 = s2;
  }

  return s1;
}
function RoundMatrixElementsToDigits(a, digits){
  var m, n;

  for(m = 0; m < NumberOfRows(a); m = m + 1){
    for(n = 0; n < NumberOfColumns(a); n = n + 1){
      a.r[m].c[n] = RoundToDigits(Element(a, m, n), digits);
      if(a.r[m].c[n] ==  -0){
        a.r[m].c[n] = 0;
      }
    }
  }
}
function SingularValueDecomposition(Ap, URef, SigmaRef, VRef){
  var A, U, V;
  var m, n, nu, nct, nrt, i, j, k, t, pp, iter, eps, tiny, kase, f, cs, sn, ks, size, orgm, orgn;
  var scale, sp, spm1, epm1, sk, ek, b, c, shift, g, p;
  var done;
  var s, e, work;

  /* Square matrix, adjust results correspondingly. */
  orgm = NumberOfRows(Ap);
  orgn = NumberOfColumns(Ap);
  size = Math.max(orgm, orgn);
  A = CreateCopyOfMatrix(Ap);
  Resize(A, size, size);

  /* Initialize. */
  m = size;
  n = size;

  /* Compute */
  nu = Math.min(m, n);
  s = [];
  s.length = Math.min(m + 1, n);
  U = CreateMatrix(m, nu);
  V = CreateSquareMatrix(n);
  e = [];
  e.length = n;
  work = [];
  work.length = m;

  /* Reduce A to bidiagonal form, storing the diagonal elements in s and the super-diagonal elements in e. */
  nct = Math.min(m - 1, n);
  nrt = Math.max(0, Math.min(n - 2, m));
  for(k = 0; k < Math.max(nct, nrt); k = k + 1){
    if(k < nct){

      /* Compute the transformation for the k-th column and place the k-th diagonal in s[k]. */
      /* Compute 2-norm of k-th column without under/overflow. */
      s[k] = 0;
      for(i = k; i < m; i = i + 1){
        s[k] = Hypothenuse(s[k], Element(A, i, k));
      }
      if(s[k] != 0){
        if(Element(A, k, k) < 0){
          s[k] =  -s[k];
        }
        for(i = k; i < m; i = i + 1){
          A.r[i].c[k] = Element(A, i, k)/s[k];
        }
        A.r[k].c[k] = Element(A, k, k) + 1;
      }
      s[k] =  -s[k];
    }
    for(j = k + 1; j < n; j = j + 1){
      if((k < nct) && (s[k] != 0)){

        /* Apply the transformation. */
        t = 0;
        for(i = k; i < m; i = i + 1){
          t = t + Element(A, i, k)*Element(A, i, j);
        }
        t =  -t/Element(A, k, k);
        for(i = k; i < m; i = i + 1){
          A.r[i].c[j] = Element(A, i, j) + t*Element(A, i, k);
        }
      }

      /* Place the k-th row of A into e for the subsequent calculation of the row transformation. */
      e[j] = Element(A, k, j);
    }
    if(k < nct){

      /* Place the transformation in U for subsequent back */
      /* multiplication. */
      for(i = k; i < m; i = i + 1){
        U.r[i].c[k] = Element(A, i, k);
      }
    }
    if(k < nrt){
      /* Compute the k-th row transformation and place the k-th super-diagonal in e[k]. */
      /* Compute 2-norm without under/overflow. */
      e[k] = 0;
      for(i = k + 1; i < n; i = i + 1){
        e[k] = Hypothenuse(e[k], e[i]);
      }
      if(e[k] != 0){
        if(e[k + 1] < 0){
          e[k] =  -e[k];
        }
        for(i = k + 1; i < n; i = i + 1){
          e[i] = e[i]/e[k];
        }
        e[k + 1] = e[k + 1] + 1;
      }
      e[k] =  -e[k];
      if((k + 1 < m) && (e[k] != 0)){

        /* Apply the transformation. */
        for(i = k + 1; i < m; i = i + 1){
          work[i] = 0;
        }
        for(j = k + 1; j < n; j = j + 1){
          for(i = k + 1; i < m; i = i + 1){
            work[i] = work[i] + e[j]*Element(A, i, j);
          }
        }
        for(j = k + 1; j < n; j = j + 1){
          t =  -e[j]/e[k + 1];
          for(i = k + 1; i < m; i = i + 1){
            A.r[i].c[j] = Element(A, i, j) + t*work[i];
          }
        }
      }

      /* Place the transformation in V for subsequent back multiplication. */
      for(i = k + 1; i < n; i = i + 1){
        V.r[i].c[k] = e[i];
      }
    }
  }

  /* Set up the final bidiagonal matrix or order p. */
  p = Math.min(n, m + 1);
  if(nct < n){
    s[nct] = Element(A, nct, nct);
  }
  if(m < p){
    s[p - 1] = 0;
  }
  if(nrt + 1 < p){
    e[nrt] = Element(A, nrt, p - 1);
  }
  e[p - 1] = 0;

  /* Generate U. */
  for(j = nct; j < nu; j = j + 1){
    for(i = 0; i < m; i = i + 1){
      U.r[i].c[j] = 0;
    }
    U.r[j].c[j] = 1;
  }
  for(k = nct - 1; k >= 0; k = k - 1){
    if(s[k] != 0){
      for(j = k + 1; j < nu; j = j + 1){
        t = 0;
        for(i = k; i < m; i = i + 1){
          t = t + Element(U, i, k)*Element(U, i, j);
        }
        t =  -t/Element(U, k, k);
        for(i = k; i < m; i = i + 1){
          U.r[i].c[j] = Element(U, i, j) + t*Element(U, i, k);
        }
      }
      for(i = k; i < m; i = i + 1){
        U.r[i].c[k] =  -Element(U, i, k);
      }
      U.r[k].c[k] = 1 + Element(U, k, k);
      for(i = 0; i < k - 1; i = i + 1){
        U.r[i].c[k] = 0;
      }
    }else{
      for(i = 0; i < m; i = i + 1){
        U.r[i].c[k] = 0;
      }
      U.r[k].c[k] = 1;
    }
  }

  /* Generate V. */
  for(k = n - 1; k >= 0; k = k - 1){
    if((k < nrt) && (e[k] != 0)){
      for(j = k + 1; j < nu; j = j + 1){
        t = 0;
        for(i = k + 1; i < n; i = i + 1){
          t = t + Element(V, i, k)*Element(V, i, j);
        }
        t =  -t/Element(V, k + 1, k);
        for(i = k + 1; i < n; i = i + 1){
          V.r[i].c[j] = Element(V, i, j) + t*Element(V, i, k);
        }
      }
    }
    for(i = 0; i < n; i = i + 1){
      V.r[i].c[k] = 0;
    }
    V.r[k].c[k] = 1;
  }

  /* Main iteration loop for the singular values. */
  pp = p - 1;
  iter = 0;
  eps = Math.pow(2,  -52);
  tiny = Math.pow(2,  -966);
  for(; p > 0; ){
    /* Here is where a test for too many iterations would go. */
    /* This section of the program inspects for negligible elements in the s and e arrays. */
    /* On completion the variables kase and k are set as follows. */
    /* kase = 1, if s(p) and e[k-1] are negligible and k<p */
    /* kase = 2, if s(k) is negligible and k<p */
    /* kase = 3, if e[k-1] is negligible, k<p, and s(k), ..., s(p) are not negligible (qr step). */
    /* kase = 4, if e(p-1) is negligible (convergence). */
    done = false;
    for(k = p - 2; k >  -1 &&  !done ; ){
      if(Math.abs(e[k]) <= tiny + eps*(Math.abs(s[k]) + Math.abs(s[k + 1]))){
        e[k] = 0;
        done = true;
      }else{
        k = k - 1;
      }
    }
    if(k == p - 2){
      kase = 4;
    }else{
      done = false;
      for(ks = p - 1; ks > k &&  !done ; ){
        if(ks != p){
          t = Math.abs(e[ks]);
        }else{
          t = 0;
        }

        if(ks != k + 1){
          t = t + Math.abs(e[ks - 1]);
        }

        if(Math.abs(s[ks]) <= tiny + eps*t){
          s[ks] = 0;
          done = true;
        }else{
          ks = ks - 1;
        }
      }
      if(ks == k){
        kase = 3;
      }else if(ks == p - 1){
        kase = 1;
      }else{
        kase = 2;
        k = ks;
      }
    }
    k = k + 1;

    /* Perform the task indicated by kase. */
    if(kase == 1){
      /* Deflate negligible s(p). */
      f = e[p - 2];
      e[p - 2] = 0;
      for(j = p - 2; j >= k; j = j - 1){
        t = Hypothenuse(s[j], f);
        cs = s[j]/t;
        sn = f/t;
        s[j] = t;
        if(j != k){
          f =  -sn*e[j - 1];
          e[j - 1] = cs*e[j - 1];
        }

        for(i = 0; i < n; i = i + 1){
          t = cs*Element(V, i, j) + sn*Element(V, i, p - 1);
          V.r[i].c[p - 1] =  -sn*Element(V, i, j) + cs*Element(V, i, p - 1);
          V.r[i].c[j] = t;
        }
      }
    }else if(kase == 2){
      /* Split at negligible s(k). */
      f = e[k - 1];
      e[k - 1] = 0;
      for(j = k; j < p; j = j + 1){
        t = Hypothenuse(s[j], f);
        cs = s[j]/t;
        sn = f/t;
        s[j] = t;
        f =  -sn*e[j];
        e[j] = cs*e[j];

        for(i = 0; i < m; i = i + 1){
          t = cs*Element(U, i, j) + sn*Element(U, i, k + 1);
          U.r[i].c[k - 1] =  -sn*Element(U, i, j) + cs*Element(U, i, k + 1);
          U.r[i].c[j] = t;
        }
      }
    }else if(kase == 3){
      /* Perform one qr step. */
      /* Calculate the shift. */
      scale = Math.max(Math.max(Math.max(Math.max(Math.abs(s[p - 1]), Math.abs(s[p - 2])), Math.abs(e[p - 2])), Math.abs(s[k])), Math.abs(e[k]));
      sp = s[p - 1]/scale;
      spm1 = s[p - 2]/scale;
      epm1 = e[p - 2]/scale;
      sk = s[k]/scale;
      ek = e[k]/scale;
      b = ((spm1 + sp)*(spm1 - sp) + epm1*epm1)/2;
      c = (sp*epm1)*(sp*epm1);
      shift = 0;
      if((b != 0) || (c != 0)){
        shift = Math.sqrt(b*b + c);
        if(b < 0){
          shift =  -shift;
        }
        shift = c/(b + shift);
      }
      f = (sk + sp)*(sk - sp) + shift;
      g = sk*ek;

      /* Chase zeros. */
      for(j = k; j < p - 1; j = j + 1){
        t = Hypothenuse(f, g);
        cs = f/t;
        sn = g/t;
        if(j != k){
          e[j - 1] = t;
        }
        f = cs*s[j] + sn*e[j];
        e[j] = cs*e[j] - sn*s[j];
        g = sn*s[j + 1];
        s[j + 1] = cs*s[j + 1];
        for(i = 0; i < n; i = i + 1){
          t = cs*Element(V, i, j) + sn*Element(V, i, j + 1);
          V.r[i].c[j + 1] =  -sn*Element(V, i, j) + cs*Element(V, i, j + 1);
          V.r[i].c[j] = t;
        }
        t = Hypothenuse(f, g);
        cs = f/t;
        sn = g/t;
        s[j] = t;
        f = cs*e[j] + sn*s[j + 1];
        s[j + 1] =  -sn*e[j] + cs*s[j + 1];
        g = sn*e[j + 1];
        e[j + 1] = cs*e[j + 1];
        if(j < m - 1){
          for(i = 0; i < m; i = i + 1){
            t = cs*Element(U, i, j) + sn*Element(U, i, j + 1);
            U.r[i].c[j + 1] =  -sn*Element(U, i, j) + cs*Element(U, i, j + 1);
            U.r[i].c[j] = t;
          }
        }
      }
      e[p - 2] = f;
      iter = iter + 1;
    }else if(kase == 4){
      /* Make the singular values positive. */
      if(s[k] <= 0){
        if(s[k] < 0){
          s[k] =  -s[k];
        }else{
          s[k] = 0;
        }

        for(i = 0; i <= pp; i = i + 1){
          V.r[i].c[k] =  -Element(V, i, k);
        }
      }

      /* Order the singular values. */
      for(; k < pp && s[k] < s[k + 1]; ){
        t = s[k];
        s[k] = s[k + 1];
        s[k + 1] = t;
        if(k < n - 1){
          for(i = 0; i < n; i = i + 1){
            t = Element(V, i, k + 1);
            V.r[i].c[k + 1] = Element(V, i, k);
            V.r[i].c[k] = t;
          }
        }
        if(k < m - 1){
          for(i = 0; i < m; i = i + 1){
            t = Element(U, i, k + 1);
            U.r[i].c[k + 1] = Element(U, i, k);
            U.r[i].c[k] = t;
          }
        }
        k = k + 1;
      }
      iter = 0;
      p = p - 1;
    }
  }

  Resize(U, orgm, orgm);
  Resize(V, orgn, orgn);

  URef.matrix = U;
  VRef.matrix = V;
  SigmaRef.matrix = CreateMatrix(orgm, orgn);
  for(i = 0; i < Math.min(orgm, orgn); i = i + 1){
    SigmaRef.matrix.r[i].c[i] = s[i];
  }

  return true;
}
function CreateComplexMatrix(rows, cols){
  var m, n;
  var matrix;

  matrix = {};
  matrix.r = [];
  matrix.r.length = rows;
  for(m = 0; m < rows; m = m + 1){
    matrix.r[m] = {};
    matrix.r[m].c = [];
    matrix.r[m].c.length = cols;
    for(n = 0; n < cols; n = n + 1){
      matrix.r[m].c[n] = cCreateComplexNumber(0, 0);
    }
  }

  return matrix;
}
function CreateComplexMatrixFromMatrix(a){
  var m, n, rows, cols;
  var matrix;

  rows = NumberOfRows(a);
  cols = NumberOfColumns(a);

  matrix = {};
  matrix.r = [];
  matrix.r.length = rows;
  for(m = 0; m < rows; m = m + 1){
    matrix.r[m] = {};
    matrix.r[m].c = [];
    matrix.r[m].c.length = cols;
    for(n = 0; n < cols; n = n + 1){
      matrix.r[m].c[n] = cCreateComplexNumber(a.r[m].c[n], 0);
    }
  }

  return matrix;
}
function CreateReMatrixFromComplexMatrix(a){
  var m, n, rows, cols;
  var matrix;

  rows = NumberOfRowsComplex(a);
  cols = NumberOfColumnsComplex(a);

  matrix = {};
  matrix.r = [];
  matrix.r.length = rows;
  for(m = 0; m < rows; m = m + 1){
    matrix.r[m] = {};
    matrix.r[m].c = [];
    matrix.r[m].c.length = cols;
    for(n = 0; n < cols; n = n + 1){
      matrix.r[m].c[n] = IndexComplex(a, m, n).re;
    }
  }

  return matrix;
}
function CreateImMatrixFromComplexMatrix(a){
  var m, n, rows, cols;
  var matrix;

  rows = NumberOfRowsComplex(a);
  cols = NumberOfColumnsComplex(a);

  matrix = {};
  matrix.r = [];
  matrix.r.length = rows;
  for(m = 0; m < rows; m = m + 1){
    matrix.r[m] = {};
    matrix.r[m].c = [];
    matrix.r[m].c.length = cols;
    for(n = 0; n < cols; n = n + 1){
      matrix.r[m].c[n] = IndexComplex(a, m, n).im;
    }
  }

  return matrix;
}
function NumberOfRowsComplex(A){
  return A.r.length;
}
function NumberOfColumnsComplex(A){
  return A.r[0].c.length;
}
function IndexComplex(a, m, n){
  return a.r[m].c[n];
}
function AddComplex(a, b){
  var m, n;
  var d;

  d = NumberOfRowsComplex(a);

  for(m = 0; m < d; m = m + 1){
    for(n = 0; n < d; n = n + 1){
      cAdd(IndexComplex(a, m, n), IndexComplex(b, m, n));
    }
  }
}
function SubtractComplex(a, b){
  var m, n;
  var r, c;

  r = NumberOfRowsComplex(a);
  c = NumberOfColumnsComplex(a);

  for(m = 0; m < r; m = m + 1){
    for(n = 0; n < c; n = n + 1){
      cSub(IndexComplex(a, m, n), IndexComplex(b, m, n));
    }
  }
}
function SubtractComplexToNew(a, b){
  var X;

  X = CreateCopyOfComplexMatrix(a);
  SubtractComplex(X, b);

  return X;
}
function MultiplyComplex(x, a, b){
  var m, n;
  var rows, cols, d;
  var i;
  var s, t;

  rows = NumberOfRowsComplex(a);
  cols = NumberOfColumnsComplex(b);
  d = NumberOfColumnsComplex(a);
  t = cCreateComplexNumber(0, 0);

  for(m = 0; m < rows; m = m + 1){
    for(n = 0; n < cols; n = n + 1){
      s = cCreateComplexNumber(0, 0);

      for(i = 0; i < d; i = i + 1){
        cAssignComplex(t, s);
        cAssignComplex(s, IndexComplex(a, m, i));
        cMul(s, IndexComplex(b, i, n));
        cAdd(s, t);
      }

      x.r[m].c[n] = s;
    }
  }
}
function MultiplyComplexToNew(a, b){
  var rows, cols;
  var x;

  rows = NumberOfRowsComplex(a);
  cols = NumberOfColumnsComplex(b);
  x = CreateComplexMatrix(rows, cols);
  MultiplyComplex(x, a, b);

  return x;
}
function Conjugate(a){
  var m, n;
  var rows, cols;

  rows = NumberOfRowsComplex(a);
  cols = NumberOfRowsComplex(a);

  for(m = 0; m < rows; m = m + 1){
    for(n = 0; n < cols; n = n + 1){
      cConjugate(IndexComplex(a, m, n));
    }
  }
}
function AssignComplexMatrix(A, B){
  var m, n;
  var r, c;

  r = NumberOfRowsComplex(A);
  c = NumberOfColumnsComplex(A);

  for(m = 0; m < r; m = m + 1){
    for(n = 0; n < c; n = n + 1){
      cAssignComplex(IndexComplex(A, m, n), IndexComplex(B, m, n));
    }
  }
}
function ScalarMultiplyComplex(A, b){
  var m, n;
  var r, c;

  r = NumberOfRowsComplex(A);
  c = NumberOfColumnsComplex(A);

  for(m = 0; m < r; m = m + 1){
    for(n = 0; n < c; n = n + 1){
      cMul(IndexComplex(A, m, n), b);
    }
  }
}
function ScalarMultiplyComplexToNew(A, b){
  var matrix;

  matrix = CreateCopyOfComplexMatrix(A);
  ScalarMultiplyComplex(matrix, b);

  return matrix;
}
function ScalarDivideComplex(A, b){
  var m, n;
  var r, c;

  r = NumberOfRowsComplex(A);
  c = NumberOfColumnsComplex(A);

  for(m = 0; m < r; m = m + 1){
    for(n = 0; n < c; n = n + 1){
      cDiv(IndexComplex(A, m, n), b);
    }
  }
}
function ElementWisePowerComplex(A, p){
  var m, n;
  var r, c;

  r = NumberOfRowsComplex(A);
  c = NumberOfColumnsComplex(A);

  for(m = 0; m < r; m = m + 1){
    for(n = 0; n < c; n = n + 1){
      cPower(IndexComplex(A, m, n), p);
    }
  }
}
function CreateComplexIdentityMatrix(d){
  var m;
  var matrix;

  matrix = CreateSquareComplexMatrix(d);
  FillComplex(matrix, 0, 0);

  for(m = 0; m < d; m = m + 1){
    IndexComplex(matrix, m, m).re = 1;
  }

  return matrix;
}
function CreateSquareComplexMatrix(d){
  var m, n;
  var matrix;

  matrix = {};
  matrix.r = [];
  matrix.r.length = d;
  for(m = 0; m < d; m = m + 1){
    matrix.r[m] = {};
    matrix.r[m].c = [];
    matrix.r[m].c.length = d;
    for(n = 0; n < d; n = n + 1){
      matrix.r[m].c[n] = cCreateComplexNumber(0, 0);
    }
  }

  return matrix;
}
function ClearComplex(a){
  FillComplex(a, 0, 0);
}
function FillComplex(a, re, im){
  var m, n;

  for(m = 0; m < NumberOfRowsComplex(a); m = m + 1){
    for(n = 0; n < NumberOfColumnsComplex(a); n = n + 1){
      IndexComplex(a, m, n).re = re;
      IndexComplex(a, m, n).im = im;
    }
  }
}
function TraceComplex(a){
  var m;
  var d;
  var tr;

  tr = cCreateComplexNumber(0, 0);

  d = a.r.length;
  for(m = 0; m < d; m = m + 1){
    cAdd(tr, IndexComplex(a, m, m));
  }

  return tr;
}
function CofactorOfComplexMatrix(mat, temp, p, q, n){
  var i, j;
  var row, col;

  i = 0;
  j = 0;

  for(row = 0; row < n; row = row + 1){
    for(col = 0; col < n; col = col + 1){
      if(row != p && col != q){
        cAssignComplex(IndexComplex(temp, i, j), IndexComplex(mat, row, col));
        j = j + 1;

        if(j == n - 1){
          j = 0;
          i = i + 1;
        }
      }
    }
  }
}
function DeterminantOfComplexSubmatrix(mat, n){
  var f, sign;
  var D, t;
  var temp;

  D = cCreateComplexNumber(0, 0);
  t = cCreateComplexNumber(0, 0);

  if(n == 1){
    D = mat.r[0].c[0];
  }else{
    temp = CreateSquareComplexMatrix(n);

    sign = 1;

    for(f = 0; f < n; f = f + 1){
      CofactorOfComplexMatrix(mat, temp, 0, f, n);
      cAssignComplexByValues(t, sign, 0);
      cMul(t, IndexComplex(mat, 0, f));
      cMul(t, DeterminantOfComplexSubmatrix(temp, n - 1));
      cAdd(D, t);
      sign =  -sign;
    }

    DeleteComplexMatrix(temp);
  }

  return D;
}
function DeleteComplexMatrix(X){
  var m, n, rows, cols;

  rows = NumberOfRowsComplex(X);
  cols = NumberOfColumnsComplex(X);
  for(m = 0; m < rows; m = m + 1){
    for(n = 0; n < cols; n = n + 1){
      delete(X.r[m].c[n]);
    }
    delete(X.r[m].c);
    delete(X.r[m]);
  }

  delete(X.r);
  delete(X);
}
function DeterminantComplex(m){
  var n;
  var D;

  n = NumberOfRowsComplex(m);
  D = DeterminantOfComplexSubmatrix(m, n);

  return D;
}
function AdjointComplex(A, adj){
  var n;
  var cofactors;
  var i, j;
  var t, sign;

  n = A.r.length;
  t = cCreateComplexNumber(0, 0);
  sign = cCreateComplexNumber(0, 0);

  if(n == 1){
    cAssignComplexByValues(IndexComplex(adj, 0, 0), 1, 0);
  }else{
    cofactors = CreateSquareComplexMatrix(n);

    for(i = 0; i < n; i = i + 1){
      for(j = 0; j < n; j = j + 1){
        CofactorOfComplexMatrix(A, cofactors, i, j, n);

        if((i + j)%2 == 0){
          cAssignComplexByValues(sign, 1, 0);
        }else{
          cAssignComplexByValues(sign,  -1, 0);
        }

        cAssignComplex(t, sign);
        cMul(t, DeterminantOfComplexSubmatrix(cofactors, n - 1));
        cAssignComplex(IndexComplex(adj, j, i), t);
      }
    }

    DeleteComplexMatrix(cofactors);
  }
}
function InverseComplex(A, inverseResult){
  var success;
  var adj;
  var n, i, j;
  var det, t;

  t = cCreateComplexNumber(0, 0);

  if(NumberOfColumnsComplex(A) == NumberOfRowsComplex(A)){
    n = NumberOfColumnsComplex(A);

    det = DeterminantComplex(A);
    if(det.re != 0 || det.im != 0){
      adj = CreateSquareComplexMatrix(n);
      AdjointComplex(A, adj);

      for(i = 0; i < n; i = i + 1){
        for(j = 0; j < n; j = j + 1){
          cAssignComplex(t, IndexComplex(adj, i, j));
          cDiv(t, det);
          cAssignComplex(IndexComplex(inverseResult, i, j), t);
        }
      }

      success = true;
      DeleteComplexMatrix(adj);
    }else{
      success = false;
    }
  }else{
    success = false;
  }

  return success;
}
function ComplexMatrixEqualsEpsilon(b, f, epsilon){
  var x, y, columns, rows;
  var equals;

  equals = true;

  if(NumberOfRowsComplex(b) == NumberOfRowsComplex(f) && NumberOfColumnsComplex(b) == NumberOfColumnsComplex(f)){
    columns = NumberOfColumnsComplex(b);
    rows = NumberOfRowsComplex(b);

    for(x = 0; x < rows; x = x + 1){
      for(y = 0; y < columns; y = y + 1){
        equals = equals && cEpsilonCompareComplex(IndexComplex(b, x, y), IndexComplex(f, x, y), epsilon);
      }
    }
  }else{
    equals = false;
  }

  return equals;
}
function MinorComplex(x, row, column){
  var minor;
  var cols, rows, i, j, m, n;

  rows = NumberOfRowsComplex(x) - 1;
  cols = NumberOfColumnsComplex(x) - 1;

  minor = CreateComplexMatrix(rows, cols);

  for(i = 0; i < rows; i = i + 1){
    if(i < row){
      m = i;
    }else{
      m = i + 1;
    }

    for(j = 0; i != row && j < cols; j = j + 1){
      if(j != column){

        if(j < column){
          n = j;
        }else{
          n = j + 1;
        }

        cAssignComplex(IndexComplex(minor, m, n), IndexComplex(x, i, j));
      }
    }
  }

  return minor;
}
function AssignComplex(A, B){
  var m, n;
  var r, c;

  r = NumberOfRowsComplex(A);
  c = NumberOfColumnsComplex(A);
  for(m = 0; m < r; m = m + 1){
    for(n = 0; n < c; n = n + 1){
      cAssignComplex(IndexComplex(A, m, n), IndexComplex(B, m, n));
    }
  }
}
function CreateCopyOfComplexMatrix(A){
  var X;

  X = CreateComplexMatrix(NumberOfRowsComplex(A), NumberOfColumnsComplex(A));
  AssignComplex(X, A);

  return X;
}
function TransposeComplex(a){
  var m, n;
  var rows;
  var square;
  var tmp;

  tmp = cCreateComplexNumber(0, 0);

  square = IsSquareComplexMatrix(a);
  if(square){
    rows = NumberOfColumnsComplex(a);

    for(m = 0; m < rows; m = m + 1){
      for(n = 0; n < m; n = n + 1){
        cAssignComplex(tmp, IndexComplex(a, n, m));
        cAssignComplex(IndexComplex(a, n, m), IndexComplex(a, m, n));
        cAssignComplex(IndexComplex(a, m, n), tmp);
      }
    }
  }

  return square;
}
function ConjugateTransposeComplex(a){
  var m, n;
  var rows;
  var square;
  var tmp;

  tmp = cCreateComplexNumber(0, 0);

  square = IsSquareComplexMatrix(a);
  if(square){
    rows = NumberOfColumnsComplex(a);

    for(m = 0; m < rows; m = m + 1){
      for(n = 0; n < m; n = n + 1){
        cAssignComplex(tmp, IndexComplex(a, n, m));
        cAssignComplex(IndexComplex(a, n, m), IndexComplex(a, m, n));
        cAssignComplex(IndexComplex(a, m, n), tmp);
        cConjugate(IndexComplex(a, m, n));
      }
    }
  }

  return square;
}
function IsSquareComplexMatrix(A){
  var is;

  if(NumberOfRowsComplex(A) == NumberOfColumnsComplex(A)){
    is = true;
  }else{
    is = false;
  }

  return is;
}
function TransposeComplexAssign(t, a){
  var m, n;
  var rows, cols;

  cols = NumberOfRowsComplex(a);
  rows = NumberOfColumnsComplex(a);

  for(m = 0; m < cols; m = m + 1){
    for(n = 0; n < rows; n = n + 1){
      cAssignComplex(IndexComplex(t, n, m), IndexComplex(a, m, n));
    }
  }
}
function TransposeComplexToNew(a){
  var m, n;
  var rows, cols;
  var c;

  cols = NumberOfRowsComplex(a);
  rows = NumberOfColumnsComplex(a);

  c = CreateComplexMatrix(rows, cols);

  for(m = 0; m < cols; m = m + 1){
    for(n = 0; n < rows; n = n + 1){
      cAssignComplex(IndexComplex(c, n, m), IndexComplex(a, m, n));
    }
  }

  return c;
}
function ExtractComplexSubMatrix(M, r1, r2, c1, c2){
  var A;
  var i, j;

  A = CreateComplexMatrix(r2 - r1 + 1, c2 - c1 + 1);

  for(i = r1; i <= r2; i = i + 1){
    for(j = c1; j <= c2; j = j + 1){
      cAssignComplex(IndexComplex(A, i - r1, j - c1), IndexComplex(M, i, j));
    }
  }

  return A;
}
function NormComplex(a){
  var l, i, j, rows, cols;
  var cComplexNumber;

  l = 0;

  rows = NumberOfRowsComplex(a);
  cols = NumberOfColumnsComplex(a);

  for(i = 0; i < rows; i = i + 1){
    for(j = 0; j < cols; j = j + 1){
      cComplexNumber = IndexComplex(a, i, j);
      l = l + Math.pow(cComplexNumber.re, 2) + Math.pow(cComplexNumber.im, 2);
    }
  }
  l = Math.sqrt(l);

  return l;
}
function ComplexCharacteristicPolynomial(A, p){
  var cp;
  var determinant;

  cp = CreateSquareComplexMatrix(NumberOfRowsComplex(A));
  determinant = {};

  ComplexCharacteristicPolynomialWithInverse(A, cp, p, determinant);

  DeleteComplexMatrix(cp);
}
function ComplexCharacteristicPolynomialWithInverse(A, AInverse, p, determinant){
  FaddeevLeVerrierAlgorithmComplex(A, AInverse, p, determinant);
}
function FaddeevLeVerrierAlgorithmComplex(A, AInverse, p, determinant){
  var Mk, Mkm1, t1, Id;
  var t, t2, n1;
  var i, n, k;

  n1 = cCreateComplexNumber( -1, 0);
  n = NumberOfRowsComplex(A);
  p.cs = [];
  p.cs.length = n + 1;
  for(i = 0; i < n + 1; i = i + 1){
    p.cs[i] = {};
  }
  cAssignComplexByValues(p.cs[n], 1, 0);
  Mkm1 = CreateSquareComplexMatrix(n);
  FillComplex(Mkm1, 0, 0);
  Id = CreateComplexIdentityMatrix(n);
  Mk = CreateSquareComplexMatrix(n);
  t1 = CreateSquareComplexMatrix(n);

  for(k = 1; k <= n; k = k + 1){
    MultiplyComplex(Mk, A, Mkm1);
    AssignComplex(t1, Id);
    ScalarMultiplyComplex(t1, p.cs[n - k + 1]);
    AddComplex(Mk, t1);

    MultiplyComplex(t1, A, Mk);
    t = TraceComplex(t1);
    t2 = cCreateComplexNumber( -1/k, 0);
    cMul(t, t2);
    cAssignComplex(p.cs[n - k], t);

    /* done */
    AssignComplex(Mkm1, Mk);

    if(k == n){
      AssignComplex(AInverse, Mk);
      cAssignComplex(t, p.cs[0]);
      cMul(t, n1);
      cAssignComplex(determinant, t);
      if(t.re == 0 && t.im == 0){
      }else{
        ScalarDivideComplex(AInverse, t);
      }
    }
  }

  DeleteComplexMatrix(Mkm1);
  DeleteComplexMatrix(Id);
  DeleteComplexMatrix(Mk);
  DeleteComplexMatrix(t1);
}
function EigenvaluesComplex(A, eigenValuesReference){
  var eigenVectorsReference;
  var success;
  var i;

  eigenVectorsReference = {};
  success = EigenpairsComplex(A, eigenValuesReference, eigenVectorsReference);
  if(success){
    for(i = 0; i < eigenVectorsReference.matrices.length; i = i + 1){
      DeleteComplexMatrix(eigenVectorsReference.matrices[i]);
    }
    delete(eigenVectorsReference.matrices);
    delete(eigenVectorsReference);
  }

  return success;
}
function EigenvectorsComplex(A, eigenVectorsReference){
  var evsReference;
  var success;
  var i;

  evsReference = {};
  success = EigenpairsComplex(A, evsReference, eigenVectorsReference);
  if(success){
    for(i = 0; i < evsReference.complexNumbers.length; i = i + 1){
      delete(evsReference.complexNumbers[i]);
    }
    delete(evsReference.complexNumbers);
    delete(evsReference);
  }

  return success;
}
function InversePowerMethodComplex(A, eigenvalue, maxIterations, eigenvector){
  var t1, t2, t3, t4, b;
  var n, i, c;
  var isSingular;
  var c101, k, cc;

  n = NumberOfRowsComplex(A);

  t2 = CreateComplexIdentityMatrix(n);
  ScalarMultiplyComplex(t2, eigenvalue);
  t3 = SubtractComplexToNew(A, t2);
  t4 = CreateSquareComplexMatrix(n);
  isSingular =  !InverseComplex(t3, t4) ;
  cc = cCreateComplexNumber(0, 0);
  t1 = CreateComplexMatrix(n, 1);

  if(isSingular){
    DeleteComplexMatrix(t2);
    DeleteComplexMatrix(t3);
    DeleteComplexMatrix(t4);

    c101 = cCreateComplexNumber(1.01, 0);
    /* Try again with more erroneous eigenvalue estimate. */
    t2 = CreateComplexIdentityMatrix(n);
    k = cMulToNew(eigenvalue, c101);
    ScalarMultiplyComplex(t2, k);
    t3 = SubtractComplexToNew(A, t2);
    t4 = CreateSquareComplexMatrix(n);
    isSingular =  !InverseComplex(t3, t4) ;
    delete(c101);
  }

  if( !isSingular ){
    b = CreateComplexMatrix(n, 1);

    for(i = 0; i < n; i = i + 1){
      cAssignComplexByValues(b.r[i].c[0], 1, 1);
    }

    for(i = 0; i < maxIterations; i = i + 1){
      MultiplyComplex(t1, t4, b);
      c = NormComplex(t1);
      cAssignComplexByValues(cc, c, 0);
      ScalarDivideComplex(t1, cc);
      AssignComplex(b, t1);
    }

    eigenvector.complexNumbers = [];
    eigenvector.complexNumbers.length = n;
    for(i = 0; i < n; i = i + 1){
      eigenvector.complexNumbers[i] = b.r[i].c[0];
    }
  }

  DeleteComplexMatrix(t1);
  DeleteComplexMatrix(t2);
  DeleteComplexMatrix(t3);
  DeleteComplexMatrix(t4);
  delete(cc);

  return  !isSingular ;
}
function EigenpairsComplex(M, eigenValuesReference, eigenVectorsReference){
  return ComplexEigenpairsUsingDurandKernerAndInversePowerMethod(M, eigenValuesReference, eigenVectorsReference, 0.000001, 100);
}
function ComplexEigenpairsUsingDurandKernerAndInversePowerMethod(M, eigenValuesReference, eigenVectorsReference, precision, maxIterations){
  var success, inverseSuccess;
  var n, i, j, k, withinPrecision;
  var t1, t2, t3, xn1, eigenValue;
  var rs, rsPrev;
  var p;
  var evecReference;
  var eigenVector;

  evecReference = {};

  p = {};
  ComplexCharacteristicPolynomial(M, p);

  n = p.cs.length - 1;
  rs = [];
  rs.length = n;
  for(i = 0; i < n; i = i + 1){
    rs[i] = cCreateComplexNumber(0, 0);
  }
  rsPrev = [];
  rsPrev.length = n;
  for(i = 0; i < n; i = i + 1){
    rsPrev[i] = cCreateComplexNumber(0.4, 0.9);
    cPower(rsPrev[i], i);
  }
  t2 = cCreateComplexNumber(0, 0);
  t3 = cCreateComplexNumber(0, 0);

  success = false;

  eigenVectorsReference.matrices = [];
  eigenVectorsReference.matrices.length = n;
  for(i = 0; i < n; i = i + 1){
    eigenVectorsReference.matrices[i] = CreateComplexMatrix(n, 1);
  }

  for(i = 0; i < maxIterations &&  !success ; i = i + 1){
    for(j = 0; j < n; j = j + 1){
      xn1 = rsPrev[j];

      t1 = pEvaluateComplex(p, xn1);
      cAssignComplexByValues(t2, 1, 0);
      for(k = 0; k < n; k = k + 1){
        if(k < j){
          cAssignComplex(t3, xn1);
          cSub(t3, rs[k]);
          cMul(t2, t3);
        }
        if(k > j){
          cAssignComplex(t3, xn1);
          cSub(t3, rsPrev[k]);
          cMul(t2, t3);
        }
      }
      cDiv(t1, t2);
      cAssignComplex(rs[j], xn1);
      cSub(rs[j], t1);

      delete(t1);
    }
    withinPrecision = 0;
    for(j = 0; j < n; j = j + 1){
      eigenValue = rs[j];

      /* Calculate the eigenvector corresponding to the eigenvalue. */
      inverseSuccess = InversePowerMethodComplex(M, eigenValue, i + 1, evecReference);
      if(inverseSuccess){
        for(k = 0; k < n; k = k + 1){
          eigenVectorsReference.matrices[j].r[k].c[0] = evecReference.complexNumbers[k];
        }

        /* Check eigenpair agains precision. */
        eigenVector = eigenVectorsReference.matrices[j];

        if(CheckComplexEigenpairPrecision(M, eigenValue, eigenVector, precision)){
          withinPrecision = withinPrecision + 1;
        }
        cAssignComplex(rsPrev[j], rs[j]);
      }
    }
    if(withinPrecision == n){
      success = true;
    }
  }

  eigenValuesReference.complexNumbers = rs;

  return success;
}
function CheckComplexEigenpairPrecision(a, lambda, e, precision){
  var vec1, vec2;
  var equal;

  vec1 = MultiplyComplexToNew(a, e);
  vec2 = ScalarMultiplyComplexToNew(e, lambda);

  equal = ComplexMatrixEqualsEpsilon(vec1, vec2, precision);

  return equal;
}
function testDeterminant(failures){
  var a;
  var d;

  a = createExample4x4_1();

  d = Determinant(a);

  AssertTrue(mathEpsilonCompare(d, 88, 0.00000000001), failures);
}
function testInverse(failures){
  var a, b, f;

  a = createExample4x4_1();

  b = CreateSquareMatrix(4);
  Inverse(a, b);

  f = createAnswerInverse4x4_1();

  AssertTrue(MatrixEqualsEpsilon(b, f, 0.00000000001), failures);
}
function createAnswerInverse4x4_1(){
  var a;
  var aref;
  var s;

  s = [];
  s.length = 0;
  s = AppendString(s, "-0.13636363636363635,  0.8636363636363636,   -0.6818181818181818,   -0.4090909090909091 \n".split(''));
  s = AppendString(s, "-0.6363636363636364,   2.3636363636363638,   -0.9318181818181818,   -0.6590909090909091 \n".split(''));
  s = AppendString(s, " 0.045454545454545456, 0.045454545454545456, -0.022727272727272728, -0.11363636363636363 \n".split(''));
  s = AppendString(s, " 0.045454545454545456, 0.045454545454545456,  0.22727272727272727,   0.13636363636363635 \n".split(''));

  aref = {};
  ParseMatrixFromString(aref, s, {});
  a = aref.matrix;

  delete(aref);

  return a;
}
function createExample4x4_1(){
  var a;
  var aref;
  var s;

  s = [];
  s.length = 0;
  s = AppendString(s, " 5, -2,  2, 7 \n".split(''));
  s = AppendString(s, " 1,  0,  0, 3 \n".split(''));
  s = AppendString(s, "-3,  1,  5, 0 \n".split(''));
  s = AppendString(s, " 3, -1, -9, 4 \n".split(''));

  aref = {};
  ParseMatrixFromString(aref, s, {});
  a = aref.matrix;

  delete(aref);

  return a;
}
function testInverseAlt2(failures){
  var a, b, f;

  a = createExample4x4_1();

  b = InverseUsingCharacteristicPolynomial(a);

  f = createAnswerInverse4x4_1();

  AssertTrue(MatrixEqualsEpsilon(b, f, 0.00000000001), failures);
}
function TestCholesky(failures){
  var a, b, f;

  a = CreateSquareMatrix(3);

  a.r[0].c = nStringToNumberArray("4, 1,  1".split(''));
  a.r[1].c = nStringToNumberArray("1, 5,  3".split(''));
  a.r[2].c = nStringToNumberArray("1, 3, 15".split(''));

  b = CreateSquareMatrix(3);
  Cholesky(a, b);

  f = CreateSquareMatrix(3);

  f.r[0].c = nStringToNumberArray("2,    0,       0".split(''));
  f.r[1].c = nStringToNumberArray("0.5,  2.17945, 0".split(''));
  f.r[2].c = nStringToNumberArray("0.5,  1.26179, 3.62738".split(''));

  AssertTrue(MatrixEqualsEpsilon(b, f, 0.001), failures);
}
function testCharacteristicPolynomial(failures){
  var a;
  var coeffs, answerCoeffs;
  var m;

  a = CreateSquareMatrix(3);

  a.r[0].c = nStringToNumberArray(" 0, -4, -6".split(''));
  a.r[1].c = nStringToNumberArray("-1,  0, -3".split(''));
  a.r[2].c = nStringToNumberArray(" 1,  2,  5".split(''));

  coeffs = CharacteristicPolynomial(a);

  answerCoeffs = nStringToNumberArray("-4, 8, -5, 1".split(''));

  for(m = 0; m < 4; m = m + 1){
    AssertTrue(mathEpsilonCompare(answerCoeffs[m], coeffs[m], 0.000001), failures);
  }
}
function testCharacteristicPolynomial2(failures){
  var a;
  var coeffs, answerCoeffs;
  var m;

  a = CreateSquareMatrix(3);

  a.r[0].c = nStringToNumberArray(" 3,  1,  5".split(''));
  a.r[1].c = nStringToNumberArray(" 3,  3,  1".split(''));
  a.r[2].c = nStringToNumberArray(" 4,  6,  4".split(''));

  coeffs = CharacteristicPolynomial(a);

  answerCoeffs = [];
  answerCoeffs.length = NumberOfRows(a) + 1;
  answerCoeffs[0] = -40;
  answerCoeffs[1] = 4;
  answerCoeffs[2] = -10;
  answerCoeffs[3] = 1;

  for(m = 0; m < 4; m = m + 1){
    AssertTrue(mathEpsilonCompare(answerCoeffs[m], coeffs[m], 0.000001), failures);
  }
}
function testCharacteristicPolynomial3(failures){
  var a;
  var coeffs, answerCoeffs;
  var m;

  failures = CreateNumberReference(0);

  a = CreateSquareMatrix(3);

  a.r[0].c = nStringToNumberArray(" 1, -3,  3".split(''));
  a.r[1].c = nStringToNumberArray(" 3, -5,  3".split(''));
  a.r[2].c = nStringToNumberArray(" 6, -6,  4".split(''));

  coeffs = CharacteristicPolynomial(a);

  answerCoeffs = nStringToNumberArray("-16, -12, 0, 1".split(''));

  for(m = 0; m < 4; m = m + 1){
    AssertTrue(mathEpsilonCompare(answerCoeffs[m], coeffs[m], 0.000001), failures);
  }
}
function testEigenValues(failures){
  var a;
  var evsReference;
  var evs, answerEvs;
  var i, j, m;
  var found, success;

  a = CreateSquareMatrix(2);

  a.r[0].c = nStringToNumberArray(" 0,  1".split(''));
  a.r[1].c = nStringToNumberArray("-2, -3".split(''));

  evsReference = {};
  success = Eigenvalues(a, evsReference);
  AssertTrue(success, failures);

  if(success){
    evs = evsReference.numberArray;

    answerEvs = [];
    answerEvs.length = NumberOfRows(a);
    answerEvs[0] = -1;
    answerEvs[1] = -2;

    for(i = 0; i < 2; i = i + 1){
      found = false;
      for(j = 0; j < 2; j = j + 1){
        if(mathEpsilonCompare(evs[i], answerEvs[j], 0.00001)){
          found = true;
        }
      }
      AssertTrue(found, failures);
    }
  }
}
function testEigenValues2(failures){
  var a;
  var evsReference;
  var evs, answerEvs;
  var success;

  a = CreateSquareMatrix(3);

  a.r[0].c = nStringToNumberArray(" 1, -3,  3".split(''));
  a.r[1].c = nStringToNumberArray(" 3, -5,  3".split(''));
  a.r[2].c = nStringToNumberArray(" 6, -6,  4".split(''));

  evsReference = {};
  success = Eigenvalues(a, evsReference);

  AssertTrue(success, failures);

  if(success){
    evs = evsReference.numberArray;

    answerEvs = [];
    answerEvs.length = NumberOfRows(a);
    answerEvs[0] = -2;
    answerEvs[1] = -2;
    answerEvs[2] = 4;

    SetsEqualEpsilon(evs, answerEvs, 0.001, failures);
  }
}
function testGaussianEliminationInto(failures){
  var a, f;

  a = CreateSquareMatrix(3);

  a.r[0].c = nStringToNumberArray(" 1,  3,  1".split(''));
  a.r[1].c = nStringToNumberArray(" 1,  1, -1".split(''));
  a.r[2].c = nStringToNumberArray(" 3, 11,  5".split(''));

  GaussianElimination(a);

  f = CreateSquareMatrix(3);

  f.r[0].c = nStringToNumberArray(" 3,  11,  5".split(''));
  f.r[1].c = nStringToNumberArray(" 0,  -2.6666666666666, -2.6666666666666".split(''));
  f.r[2].c = nStringToNumberArray(" 0, 0,  0".split(''));

  AssertTrue(MatrixEqualsEpsilon(a, f, 0.001), failures);
}
function testEigenpairs(failures){
  var a;
  var i, eval;
  var eigenvaluesReference;
  var eigenvectorsReference, eigenvectorsReference2;
  var evec;
  var success, equal;

  a = CreateSquareMatrix(3);

  a.r[0].c = nStringToNumberArray(" 1,  2,  1".split(''));
  a.r[1].c = nStringToNumberArray(" 6, -1,  0".split(''));
  a.r[2].c = nStringToNumberArray("-1, -2, -1".split(''));

  eigenvectorsReference = {};
  eigenvectorsReference2 = {};
  eigenvaluesReference = {};
  success = Eigenpairs(a, eigenvaluesReference, eigenvectorsReference);

  AssertTrue(success, failures);

  if(success){
    /* Check */
    for(i = 0; i < 3; i = i + 1){
      evec = eigenvectorsReference.matrices[i];
      eval = eigenvaluesReference.numberArray[i];

      equal = CheckEigenpairPrecision(a, eval, evec, 0.001);

      AssertTrue(equal, failures);
    }
  }

  success = Eigenvectors(a, eigenvectorsReference2);

  /*System.out.println(new String(MatrixArrayToString(eigenvectorsReference2.matrices, 3d))); */
  AssertTrue(success, failures);

  AssertTrue(MatrixEqualsEpsilon(eigenvectorsReference.matrices[0], eigenvectorsReference2.matrices[0], 0.001), failures);
  AssertTrue(MatrixEqualsEpsilon(eigenvectorsReference.matrices[1], eigenvectorsReference2.matrices[1], 0.001), failures);
  AssertTrue(MatrixEqualsEpsilon(eigenvectorsReference.matrices[2], eigenvectorsReference2.matrices[2], 0.001), failures);
}
function testQRDecomposition(failures){
  var a, fq, fr, r, q;

  r = CreateSquareMatrix(3);
  q = CreateSquareMatrix(3);

  /* Create example: */
  a = CreateSquareMatrix(3);

  a.r[0].c = nStringToNumberArray("12, -51,   4".split(''));
  a.r[1].c = nStringToNumberArray(" 6, 167, -68".split(''));
  a.r[2].c = nStringToNumberArray("-4,  24, -41".split(''));

  QRDecomposition(a, q, r);

  fq = CreateSquareMatrix(3);

  fq.r[0].c[0] = 6/7;
  fq.r[0].c[1] =  -69/175;
  fq.r[0].c[2] =  -58/175;

  fq.r[1].c[0] = 3/7;
  fq.r[1].c[1] = 158/175;
  fq.r[1].c[2] = 6/175;

  fq.r[2].c[0] =  -2/7;
  fq.r[2].c[1] = 6/35;
  fq.r[2].c[2] =  -33/35;

  fr = CreateSquareMatrix(3);

  fr.r[0].c = nStringToNumberArray("14,  21, -14".split(''));
  fr.r[1].c = nStringToNumberArray(" 0, 175, -70".split(''));
  fr.r[2].c = nStringToNumberArray(" 0,   0,  35".split(''));

  AssertTrue(MatrixEqualsEpsilon(r, fr, 0.000000000001), failures);
  AssertTrue(MatrixEqualsEpsilon(q, fq, 0.000000000001), failures);
}
function testQRDecomposition2(failures){
  var a, fq, fr, r, q;

  r = CreateSquareMatrix(2);
  q = CreateSquareMatrix(2);

  /* Create example: */
  a = CreateSquareMatrix(2);

  a.r[0].c = nStringToNumberArray(" 0,  1".split(''));
  a.r[1].c = nStringToNumberArray("-2, -3".split(''));

  QRDecomposition(a, q, r);

  fq = CreateSquareMatrix(2);

  fq.r[0].c[0] = 0;
  fq.r[0].c[1] = 1;

  fq.r[1].c[0] =  -1;
  fq.r[1].c[1] = 0;

  fr = CreateSquareMatrix(2);

  fr.r[0].c = nStringToNumberArray("2, 3".split(''));
  fr.r[1].c = nStringToNumberArray("0, 1".split(''));

  AssertTrue(MatrixEqualsEpsilon(r, fr, 0.0000000000001), failures);
  AssertTrue(MatrixEqualsEpsilon(q, fq, 0.0000000000001), failures);
}
function testQRDecomposition3(failures){
  var a, fq, fr, r, q;

  r = CreateSquareMatrix(3);
  q = CreateMatrix(4, 3);

  /* Create example: */
  a = CreateMatrix(4, 3);

  a.r[0].c = nStringToNumberArray(" -1, -1, 1".split(''));
  a.r[1].c = nStringToNumberArray("  1,  3, 3".split(''));
  a.r[2].c = nStringToNumberArray(" -1, -1, 5".split(''));
  a.r[3].c = nStringToNumberArray("  1,  3, 7".split(''));

  QRDecomposition(a, q, r);

  fq = CreateMatrix(4, 3);

  fq.r[0].c = nStringToNumberArray("-0.5, 0.5, -0.5".split(''));
  fq.r[1].c = nStringToNumberArray(" 0.5, 0.5, -0.5".split(''));
  fq.r[2].c = nStringToNumberArray("-0.5, 0.5,  0.5".split(''));
  fq.r[3].c = nStringToNumberArray(" 0.5, 0.5,  0.5".split(''));

  fr = CreateSquareMatrix(3);

  fr.r[0].c = nStringToNumberArray(" 2, 4, 2".split(''));
  fr.r[1].c = nStringToNumberArray(" 0, 2, 8".split(''));
  fr.r[2].c = nStringToNumberArray(" 0, 0, 4".split(''));

  AssertTrue(MatrixEqualsEpsilon(r, fr, 0.0000000000001), failures);
  AssertTrue(MatrixEqualsEpsilon(q, fq, 0.0000000000001), failures);
}
function testEigenValues3QR(failures){
  var a, x, q, r;
  var evs, answerEvs;
  var success;

  a = CreateSquareMatrix(2);

  a.r[0].c = nStringToNumberArray(" 0,  1".split(''));
  a.r[1].c = nStringToNumberArray("-2, -3".split(''));

  x = CreateMatrix(2, 2);
  q = CreateMatrix(2, 2);
  r = CreateMatrix(2, 2);
  success = QRAlgorithm(a, r, x, q, 0.00001, 100);
  AssertTrue(success, failures);

  if(success){
    evs = [];
    evs.length = 2;
    evs[0] = x.r[0].c[0];
    evs[1] = x.r[1].c[1];

    answerEvs = [];
    answerEvs.length = NumberOfRows(a);
    answerEvs[0] = -1;
    answerEvs[1] = -2;

    SetsEqualEpsilon(evs, answerEvs, 0.00001, failures);
  }
}
function SetsEqualEpsilon(a, b, epsilon, failures){
  var i, j;
  var found;

  for(i = 0; i < a.length; i = i + 1){
    found = false;
    for(j = 0; j < b.length; j = j + 1){
      if(mathEpsilonCompare(a[i], b[j], epsilon)){
        found = true;
      }
    }
    AssertTrue(found, failures);
  }
}
function testLUDecomposition(failures){
  var a, l, u, fl, fu, ui, li, i, fi;
  var success;

  a = CreateSquareMatrix(2);
  l = CreateSquareMatrix(2);
  u = CreateSquareMatrix(2);

  a.r[0].c = nStringToNumberArray(" 4,  3".split(''));
  a.r[1].c = nStringToNumberArray(" 6,  3".split(''));

  success = LUDecomposition(a, l, u);
  AssertTrue(success, failures);

  fl = CreateSquareMatrix(2);
  fl.r[0].c = nStringToNumberArray(" 1,  0".split(''));
  fl.r[1].c = nStringToNumberArray(" 1.5,  1".split(''));

  fu = CreateSquareMatrix(2);
  fu.r[0].c = nStringToNumberArray(" 4,  3".split(''));
  fu.r[1].c = nStringToNumberArray(" 0,  -1.5".split(''));

  AssertTrue(MatrixEqualsEpsilon(l, fl, 0.001), failures);
  AssertTrue(MatrixEqualsEpsilon(l, fl, 0.001), failures);

  li = CreateSquareMatrix(2);
  ui = CreateSquareMatrix(2);

  InvertLowerTriangularMatrix(l, li);
  InvertUpperTriangularMatrix(u, ui);

  i = CreateSquareMatrix(2);
  fi = CreateIdentityMatrix(2);

  Multiply(i, l, li);
  AssertTrue(MatrixEqualsEpsilon(i, fi, 0.001), failures);
  Multiply(i, u, ui);
  AssertTrue(MatrixEqualsEpsilon(i, fi, 0.001), failures);
}
function TestSingularValueDecomposition1(failures){
  var A;
  var URef, SRef, VRef;
  var success;

  A = CreateSquareMatrix(3);

  A.r[0].c = nStringToNumberArray("-1.000, 0.000, 0.000".split(''));
  A.r[1].c = nStringToNumberArray("-0.830, -0.557, 0.000".split(''));
  A.r[2].c = nStringToNumberArray("-0.691, -0.685, 0.232".split(''));

  URef = {};
  SRef = {};
  VRef = {};

  success = SingularValueDecomposition(A, URef, SRef, VRef);

  CheckSingularValueDecomposition(A, URef, SRef, VRef, success, failures);
}
function CheckSingularValueDecomposition(a, URef, SRef, VRef, success, failures){
  var U, S, V, Ac, Vt, I, Ut, Ic;
  var i;

  AssertTrue(success, failures);

  if(success){
    U = URef.matrix;
    S = SRef.matrix;
    V = VRef.matrix;

    Ac = MultiplyToNew(U, S);
    Vt = TransposeToNew(V);
    Ac = MultiplyToNew(Ac, Vt);

    AssertTrue(MatrixEqualsEpsilon(a, Ac, 0.0000001), failures);

    I = CreateSquareMatrix(NumberOfRows(U));
    for(i = 0; i < Math.min(NumberOfRows(U), NumberOfColumns(U)); i = i + 1){
      I.r[i].c[i] = 1;
    }
    Ut = TransposeToNew(U);
    Ic = MultiplyToNew(U, Ut);

    AssertTrue(MatrixEqualsEpsilon(I, Ic, 0.0000001), failures);

    I = CreateIdentityMatrix(NumberOfRows(V));
    Ic = MultiplyToNew(V, Vt);

    AssertTrue(MatrixEqualsEpsilon(I, Ic, 0.0000001), failures);
  }
}
function TestSingularValueDecomposition2(failures){
  var A;
  var URef, SRef, VRef;
  var success;

  A = CreateMatrix(2, 3);

  A.r[0].c = nStringToNumberArray("3, 2, 2".split(''));
  A.r[1].c = nStringToNumberArray("2, 3, -2".split(''));

  URef = {};
  SRef = {};
  VRef = {};

  success = SingularValueDecomposition(A, URef, SRef, VRef);

  CheckSingularValueDecomposition(A, URef, SRef, VRef, success, failures);
}
function TestSingularValueDecomposition3(failures){
  var A;
  var URef, SRef, VRef;
  var success;

  A = CreateSquareMatrix(3);

  A.r[0].c = nStringToNumberArray("2, 0, 0".split(''));
  A.r[1].c = nStringToNumberArray("2, 1, 0".split(''));
  A.r[2].c = nStringToNumberArray("0, -2, 0".split(''));

  URef = {};
  SRef = {};
  VRef = {};

  success = SingularValueDecomposition(A, URef, SRef, VRef);

  CheckSingularValueDecomposition(A, URef, SRef, VRef, success, failures);
}
function TestSingularValueDecomposition4(failures){
  var A;
  var URef, SRef, VRef;
  var success;

  A = CreateMatrix(2, 3);

  A.r[0].c = nStringToNumberArray("1, 1, 0".split(''));
  A.r[1].c = nStringToNumberArray("1, 0, 1".split(''));

  URef = {};
  SRef = {};
  VRef = {};

  success = SingularValueDecomposition(A, URef, SRef, VRef);

  CheckSingularValueDecomposition(A, URef, SRef, VRef, success, failures);
}
function TestSingularValueDecomposition5(failures){
  var A;
  var URef, SRef, VRef;
  var success;

  A = CreateMatrix(3, 2);

  A.r[0].c = nStringToNumberArray("1, 1".split(''));
  A.r[1].c = nStringToNumberArray("1, 0".split(''));
  A.r[2].c = nStringToNumberArray("0, 1".split(''));

  URef = {};
  SRef = {};
  VRef = {};

  success = SingularValueDecomposition(A, URef, SRef, VRef);

  CheckSingularValueDecomposition(A, URef, SRef, VRef, success, failures);
}
function CreateMatrixWithComplexEigenvalues(){
  var a;

  a = CreateSquareMatrix(3);

  a.r[0].c = nStringToNumberArray(" 0, 1, 0".split(''));
  a.r[1].c = nStringToNumberArray(" 0, 0, 1".split(''));
  a.r[2].c = nStringToNumberArray(" 1, 0, 0".split(''));

  return a;
}
function CreateMatrixWithRepeatedEigenvalues(){
  var a;

  a = CreateSquareMatrix(4);

  a.r[0].c = nStringToNumberArray(" 2, 0, 0, 0".split(''));
  a.r[1].c = nStringToNumberArray(" 1, 2, 0, 0".split(''));
  a.r[2].c = nStringToNumberArray(" 0, 1, 3, 0".split(''));
  a.r[3].c = nStringToNumberArray(" 0, 0, 1, 3".split(''));

  return a;
}
function test(){
  var failures;
  var m;
  var diag;

  failures = CreateNumberReference(0);

  testDeterminant(failures);
  testInverse(failures);
  testInverseAlt2(failures);
  TestCholesky(failures);
  testCharacteristicPolynomial(failures);
  testCharacteristicPolynomial2(failures);
  testCharacteristicPolynomial3(failures);
  testComplexCharacteristicPolynomial(failures);
  testComplexCharacteristicPolynomial2(failures);
  testEigenValues(failures);
  testEigenValues2(failures);
  testEigenValues3QR(failures);
  testComplexEigenValues(failures);
  testComplexEigenValues2(failures);
  testGaussianEliminationInto(failures);
  testEigenpairs(failures);
  testComplexMatrices(failures);

  testQRDecomposition(failures);
  testQRDecomposition2(failures);
  testQRDecomposition3(failures);
  testComplexInversePowerMethod(failures);
  testComplexEigenpairs(failures);
  testLUDecomposition(failures);

  TestSingularValueDecomposition1(failures);
  TestSingularValueDecomposition2(failures);
  TestSingularValueDecomposition3(failures);
  TestSingularValueDecomposition4(failures);
  TestSingularValueDecomposition5(failures);

  m = CreateSquareMatrix(2);
  m.r[0].c[0] = 1;
  m.r[0].c[1] = 2;
  m.r[1].c[0] =  -3;
  m.r[1].c[1] = 2;

  Transpose(m);
  ElementWisePower(m, 2);
  m = CreateDiagonalMatrixFromArray(m.r[0].c);
  diag = [];
  diag.length = 2;
  ExtractDiagonal(m, diag);

  return failures.numberValue;
}
function testComplexMatrices(failures){
  testComplexDeterminant(failures);
  testComplexInverse(failures);
}
function testComplexDeterminant(failures){
  var a;
  var ca;
  var d;

  a = createExample4x4_1();
  ca = CreateComplexMatrixFromMatrix(a);

  d = DeterminantComplex(ca);

  AssertTrue(mathEpsilonCompare(d.re, 88, 0.00000000001), failures);
  AssertTrue(mathEpsilonCompare(d.im, 0, 0.00000000001), failures);

  ca = CreateExampleComplexMatrix2x2();

  d = DeterminantComplex(ca);

  AssertTrue(mathEpsilonCompare(d.re, 0, 0.00000000001), failures);
  AssertTrue(mathEpsilonCompare(d.im,  -1, 0.00000000001), failures);
}
function CreateExampleComplexMatrix2x2(){
  var ca;
  ca = CreateComplexMatrix(2, 2);
  IndexComplex(ca, 0, 0).re = 0;
  IndexComplex(ca, 0, 0).im = 1;
  IndexComplex(ca, 0, 1).re = 1;
  IndexComplex(ca, 0, 1).im = 0;
  IndexComplex(ca, 1, 0).re = 0;
  IndexComplex(ca, 1, 0).im = 0;
  IndexComplex(ca, 1, 1).re =  -1;
  IndexComplex(ca, 1, 1).im = 0;
  return ca;
}
function CreateExampleComplexMatrix2x2Inverse(){
  var ca;
  ca = CreateComplexMatrix(2, 2);
  IndexComplex(ca, 0, 0).re = 0;
  IndexComplex(ca, 0, 0).im =  -1;
  IndexComplex(ca, 0, 1).re = 0;
  IndexComplex(ca, 0, 1).im =  -1;
  IndexComplex(ca, 1, 0).re = 0;
  IndexComplex(ca, 1, 0).im = 0;
  IndexComplex(ca, 1, 1).re =  -1;
  IndexComplex(ca, 1, 1).im = 0;
  return ca;
}
function testComplexInverse(failures){
  var a, f;
  var ca, b, cf;

  a = createExample4x4_1();
  ca = CreateComplexMatrixFromMatrix(a);

  b = CreateSquareComplexMatrix(4);
  InverseComplex(ca, b);

  f = createAnswerInverse4x4_1();
  cf = CreateComplexMatrixFromMatrix(f);

  AssertTrue(ComplexMatrixEqualsEpsilon(b, cf, 0.00000000001), failures);

  ca = CreateExampleComplexMatrix2x2();

  b = CreateSquareComplexMatrix(2);
  InverseComplex(ca, b);

  cf = CreateExampleComplexMatrix2x2Inverse();

  AssertTrue(ComplexMatrixEqualsEpsilon(b, cf, 0.00000000001), failures);
}
function testComplexCharacteristicPolynomial(failures){
  var a;
  var coeffs, answerCoeffs;
  var m;
  var success;

  a = CreateComplexMatrix(3, 3);
  IndexComplex(a, 0, 0).re = 0;
  IndexComplex(a, 0, 0).im = 0;
  IndexComplex(a, 0, 1).re =  -4;
  IndexComplex(a, 0, 1).im = 0;
  IndexComplex(a, 0, 2).re =  -6;
  IndexComplex(a, 0, 2).im = 0;
  IndexComplex(a, 1, 0).re =  -1;
  IndexComplex(a, 1, 0).im = 0;
  IndexComplex(a, 1, 1).re = 0;
  IndexComplex(a, 1, 1).im = 0;
  IndexComplex(a, 1, 2).re =  -3;
  IndexComplex(a, 1, 2).im = 0;
  IndexComplex(a, 2, 0).re = 1;
  IndexComplex(a, 2, 0).im = 0;
  IndexComplex(a, 2, 1).re = 2;
  IndexComplex(a, 2, 1).im = 0;
  IndexComplex(a, 2, 2).re = 5;
  IndexComplex(a, 2, 2).im = 0;

  coeffs = {};
  ComplexCharacteristicPolynomial(a, coeffs);

  answerCoeffs = pCreateComplexPolynomial(3);
  answerCoeffs.cs[0] = cCreateComplexNumber( -4, 0);
  answerCoeffs.cs[1] = cCreateComplexNumber(8, 0);
  answerCoeffs.cs[2] = cCreateComplexNumber( -5, 0);
  answerCoeffs.cs[3] = cCreateComplexNumber(1, 0);

  for(m = 0; m < 4; m = m + 1){
    AssertTrue(cEpsilonCompareComplex(answerCoeffs.cs[m], coeffs.cs[m], 0.000001), failures);
  }
}
function testComplexCharacteristicPolynomial2(failures){
  var a;
  var p, ap;
  var m;

  a = CreateComplexMatrix(2, 2);
  IndexComplex(a, 0, 0).re = 0;
  IndexComplex(a, 0, 0).im = 1;
  IndexComplex(a, 0, 1).re = 0;
  IndexComplex(a, 0, 1).im = 0;
  IndexComplex(a, 1, 0).re = 0;
  IndexComplex(a, 1, 0).im = 1;
  IndexComplex(a, 1, 1).re = 1;
  IndexComplex(a, 1, 1).im = 0;

  p = {};
  ComplexCharacteristicPolynomial(a, p);

  ap = pCreateComplexPolynomial(2);
  ap.cs[0] = cCreateComplexNumber(0, 1);
  ap.cs[1] = cCreateComplexNumber( -1,  -1);
  ap.cs[2] = cCreateComplexNumber(1, 0);

  for(m = 0; m < 3; m = m + 1){
    AssertTrue(cEpsilonCompareComplex(ap.cs[m], p.cs[m], 0.000001), failures);
  }
}
function testComplexEigenValues(failures){
  var a;
  var evsReference;
  var evs, answerEvs;
  var i, j;
  var found, success;

  a = CreateComplexMatrix(2, 2);
  IndexComplex(a, 0, 0).re = 0;
  IndexComplex(a, 0, 0).im = 0;
  IndexComplex(a, 0, 1).re = 1;
  IndexComplex(a, 0, 1).im = 0;
  IndexComplex(a, 1, 0).re =  -2;
  IndexComplex(a, 1, 0).im = 0;
  IndexComplex(a, 1, 1).re =  -3;
  IndexComplex(a, 1, 1).im = 0;

  evsReference = {};
  success = EigenvaluesComplex(a, evsReference);
  AssertTrue(success, failures);

  if(success){
    evs = evsReference.complexNumbers;

    answerEvs = [];
    answerEvs.length = NumberOfRowsComplex(a);
    answerEvs[0] = cCreateComplexNumber( -1, 0);
    answerEvs[1] = cCreateComplexNumber( -2, 0);

    for(i = 0; i < 2; i = i + 1){
      found = false;
      for(j = 0; j < 2; j = j + 1){
        if(cEpsilonCompareComplex(evs[i], answerEvs[j], 0.00001)){
          found = true;
        }
      }
      AssertTrue(found, failures);
    }
  }
}
function testComplexEigenValues2(failures){
  var a;
  var evsReference;
  var evs, answerEvs;
  var i, j;
  var found, success;

  a = CreateComplex3x3Matrix();

  evsReference = {};
  success = EigenvaluesComplex(a, evsReference);
  AssertTrue(success, failures);

  if(success){
    evs = evsReference.complexNumbers;

    answerEvs = [];
    answerEvs.length = NumberOfRowsComplex(a);
    answerEvs[0] = cCreateComplexNumber(1, 0);
    answerEvs[1] = cCreateComplexNumber( -0.5, 0.8660254037844386467637231);
    answerEvs[2] = cCreateComplexNumber( -0.5,  -0.8660254037844386467637231);

    for(i = 0; i < 3; i = i + 1){
      found = false;
      for(j = 0; j < 3; j = j + 1){
        if(cEpsilonCompareComplex(evs[i], answerEvs[j], 0.00001)){
          found = true;
        }
      }
      AssertTrue(found, failures);
    }
  }
}
function CreateComplex3x3Matrix(){
  var a;
  a = CreateComplexMatrix(3, 3);
  IndexComplex(a, 0, 0).re = 0;
  IndexComplex(a, 0, 0).im = 0;
  IndexComplex(a, 0, 1).re = 1;
  IndexComplex(a, 0, 1).im = 0;
  IndexComplex(a, 0, 2).re = 0;
  IndexComplex(a, 0, 2).im = 0;
  IndexComplex(a, 1, 0).re = 0;
  IndexComplex(a, 1, 0).im = 0;
  IndexComplex(a, 1, 1).re = 0;
  IndexComplex(a, 1, 1).im = 0;
  IndexComplex(a, 1, 2).re = 1;
  IndexComplex(a, 1, 2).im = 0;
  IndexComplex(a, 2, 0).re = 1;
  IndexComplex(a, 2, 0).im = 0;
  IndexComplex(a, 2, 1).re = 0;
  IndexComplex(a, 2, 1).im = 0;
  IndexComplex(a, 2, 2).re = 0;
  IndexComplex(a, 2, 2).im = 0;
  return a;
}
function testComplexInversePowerMethod(failures){
  var a, evecMatrix;
  var evsReference;
  var eval, evalApprox, answerEvs;
  var i, N;
  var found, success, equals;
  var eigenvector;

  a = CreateComplex3x3Matrix();

  eval = cCreateComplexNumber( -0.5, 0.8660254037844386467637231);
  evalApprox = cCreateComplexNumber( -0.5*1.1, 0.8660254037844386467637231*1.1);

  eigenvector = {};
  success = InversePowerMethodComplex(a, evalApprox, 100, eigenvector);

  AssertTrue(success, failures);

  N = NumberOfRowsComplex(a);
  evecMatrix = CreateComplexMatrix(N, 1);
  for(i = 0; i < N; i = i + 1){
    cAssignComplex(evecMatrix.r[i].c[0], eigenvector.complexNumbers[i]);
  }

  equals = CheckComplexEigenpairPrecision(a, eval, evecMatrix, 0.00001);

  AssertTrue(equals, failures);
}
function testComplexEigenpairs(failures){
  var a;
  var i;
  var eval;
  var eigenvaluesReference;
  var eigenvectorsReference, eigenvectorsReference2;
  var evec;
  var success, equal;

  a = CreateSquareComplexMatrix(3);

  IndexComplex(a, 0, 0).re = 1;
  IndexComplex(a, 0, 0).im = 0;
  IndexComplex(a, 0, 1).re = 2;
  IndexComplex(a, 0, 1).im = 0;
  IndexComplex(a, 0, 2).re = 1;
  IndexComplex(a, 0, 2).im = 0;
  IndexComplex(a, 1, 0).re = 6;
  IndexComplex(a, 1, 0).im = 0;
  IndexComplex(a, 1, 1).re =  -1;
  IndexComplex(a, 1, 1).im = 0;
  IndexComplex(a, 1, 2).re = 0;
  IndexComplex(a, 1, 2).im = 0;
  IndexComplex(a, 2, 0).re =  -1;
  IndexComplex(a, 2, 0).im = 0;
  IndexComplex(a, 2, 1).re =  -2;
  IndexComplex(a, 2, 1).im = 0;
  IndexComplex(a, 2, 2).re =  -1;
  IndexComplex(a, 2, 2).im = 0;

  eigenvectorsReference = {};
  eigenvaluesReference = {};
  eigenvectorsReference2 = {};
  success = EigenpairsComplex(a, eigenvaluesReference, eigenvectorsReference);

  AssertTrue(success, failures);

  if(success){
    /* Check */
    for(i = 0; i < 3; i = i + 1){
      evec = eigenvectorsReference.matrices[i];
      eval = eigenvaluesReference.complexNumbers[i];

      equal = CheckComplexEigenpairPrecision(a, eval, evec, 0.001);

      AssertTrue(equal, failures);
    }
  }

  success = EigenvectorsComplex(a, eigenvectorsReference2);

  AssertTrue(success, failures);

  AssertTrue(ComplexMatrixEqualsEpsilon(eigenvectorsReference.matrices[0], eigenvectorsReference2.matrices[0], 0.001), failures);
  AssertTrue(ComplexMatrixEqualsEpsilon(eigenvectorsReference.matrices[1], eigenvectorsReference2.matrices[1], 0.001), failures);
  AssertTrue(ComplexMatrixEqualsEpsilon(eigenvectorsReference.matrices[2], eigenvectorsReference2.matrices[2], 0.001), failures);
}
function FindRoots(p, rootsReference){
  return DurandKernerMethod(p, 0.000001, 100, rootsReference);
}
function LaguerresMethodWithRepeatedDivision(p, maxIterations, precision, guess, rootsReference){
  var n, nr, xk;
  var x;
  var q, r, d;
  var success;
  var xkReference;

  n = pDegree(p);

  x = [];
  x.length = n;

  q = pCreatePolynomial(n);
  r = pCreatePolynomial(n);
  d = pCreatePolynomial(n);

  success = true;
  xkReference = CreateNumberReference(0);

  for(nr = 0; nr < n && success; nr = nr + 1){
    success = LaguerresMethod(p, guess, maxIterations, precision, xkReference);

    if(success){
      xk = xkReference.numberValue;
      x[nr] = xk;

      pFill(d, 0);
      d[0] =  -xk;
      d[1] = 1;
      pDivide(q, r, p, d);
      pAssign(p, q);
    }
  }

  delete(q);
  delete(r);
  delete(d);
        
  rootsReference.numberArray = x;

  return success;
}
function LaguerresMethod(p, guess, maxIterations, precision, rootReference){
  var k, a, G, H, xk, denom1, denom2, denom, t1, n;
  var success;

  n = pDegree(p);
  success = true;

  xk = guess;

  for(k = 0; (k < maxIterations) && (Math.abs(pEvaluate(p, xk)) >= precision) && success; k = k + 1){
    G = pEvaluateDerivative(p, xk, 1)/pEvaluate(p, xk);
    H = Math.pow(G, 2) - pEvaluateDerivative(p, xk, 2)/pEvaluate(p, xk);
    t1 = (n - 1)*(n*H - Math.pow(G, 2));
    if(t1 >= 0){
      denom = Math.sqrt(t1);
      denom1 = G + denom;
      denom2 = G - denom;
      if(Math.abs(denom1) >= Math.abs(denom2)){
        denom = denom1;
      }else{
        denom = denom2;
      }
      a = n/denom;

      xk = xk - a;
    }else{
      success = false;
    }
  }

  if(k == maxIterations){
    success = false;
  }

  if(Math.abs(pEvaluate(p, xk)) >= precision){
    success = false;
  }

  rootReference.numberValue = xk;

  return success;
}
function DurandKernerMethod(p, precision, maxIterations, rootsReference){
  var success;
  var n, i, j, k, t1, t2, xn1, withinPrecision;
  var rs, rsPrev;

  n = p.length - 1;
  rs = [];
  rs.length = n;
  rsPrev = [];
  rsPrev.length = n;

  for(i = 0; i < n; i = i + 1){
    rsPrev[i] = i;
  }

  success = false;

  for(i = 0; i < maxIterations &&  !success ; i = i + 1){
    for(j = 0; j < n; j = j + 1){
      xn1 = rsPrev[j];

      t1 = pEvaluate(p, xn1);
      t2 = 1;
      for(k = 0; k < n; k = k + 1){
        if(k < j){
          t2 = t2*(xn1 - rs[k]);
        }
        if(k > j){
          t2 = t2*(xn1 - rsPrev[k]);
        }
      }
      t1 = t1/t2;
      rs[j] = xn1 - t1;
    }
    withinPrecision = 0;
    for(j = 0; j < n; j = j + 1){
      if(mathEpsilonCompare(rsPrev[j], rs[j], precision)){
        withinPrecision = withinPrecision + 1;
      }
      rsPrev[j] = rs[j];
    }
    if(withinPrecision == n){
      success = true;
    }
  }

  rootsReference.numberArray = rs;

  return success;
}
function FindRootsComplex(p, rootsReference){
  return DurandKernerMethodComplex(p, 0.000001, 100, rootsReference);
}
function DurandKernerMethodComplex(p, precision, maxIterations, rootsReference){
  var success;
  var n, i, j, k, withinPrecision;
  var t1, t2, t3, xn1;
  var rs, rsPrev;

  n = p.cs.length - 1;
  rs = [];
  rs.length = n;
  for(i = 0; i < n; i = i + 1){
    rs[i] = cCreateComplexNumber(0, 0);
  }
  rsPrev = [];
  rsPrev.length = n;
  for(i = 0; i < n; i = i + 1){
    rsPrev[i] = cCreateComplexNumber(0.4, 0.9);
    cPower(rsPrev[i], i);
  }
  t2 = cCreateComplexNumber(0, 0);
  t3 = cCreateComplexNumber(0, 0);

  success = false;

  for(i = 0; i < maxIterations &&  !success ; i = i + 1){
    for(j = 0; j < n; j = j + 1){
      xn1 = rsPrev[j];

      t1 = pEvaluateComplex(p, xn1);
      cAssignComplexByValues(t2, 1, 0);
      for(k = 0; k < n; k = k + 1){
        if(k < j){
          cAssignComplex(t3, xn1);
          cSub(t3, rs[k]);
          cMul(t2, t3);
        }
        if(k > j){
          cAssignComplex(t3, xn1);
          cSub(t3, rsPrev[k]);
          cMul(t2, t3);
        }
      }
      cDiv(t1, t2);
      cAssignComplex(rs[j], xn1);
      cSub(rs[j], t1);
    }
    withinPrecision = 0;
    for(j = 0; j < n; j = j + 1){
      if(cEpsilonCompareComplex(rsPrev[j], rs[j], precision)){
        withinPrecision = withinPrecision + 1;
      }
      cAssignComplex(rsPrev[j], rs[j]);
    }
    if(withinPrecision == n){
      success = true;
    }
  }

  rootsReference.complexNumbers = rs;

  return success;
}
function CreateBooleanReference(value){
  var ref;

  ref = {};
  ref.booleanValue = value;

  return ref;
}
function CreateBooleanArrayReference(value){
  var ref;

  ref = {};
  ref.booleanArray = value;

  return ref;
}
function CreateBooleanArrayReferenceLengthValue(lengthx, value){
  var ref;
  var i;

  ref = {};
  ref.booleanArray = [];
  ref.booleanArray.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    ref.booleanArray[i] = value;
  }

  return ref;
}
function FreeBooleanArrayReference(booleanArrayReference){
  delete(booleanArrayReference.booleanArray);
  delete(booleanArrayReference);
}
function CreateCharacterReference(value){
  var ref;

  ref = {};
  ref.characterValue = value;

  return ref;
}
function CreateNumberReference(value){
  var ref;

  ref = {};
  ref.numberValue = value;

  return ref;
}
function CreateNumberArrayReference(value){
  var ref;

  ref = {};
  ref.numberArray = value;

  return ref;
}
function CreateNumberArrayReferenceLengthValue(lengthx, value){
  var ref;
  var i;

  ref = {};
  ref.numberArray = [];
  ref.numberArray.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    ref.numberArray[i] = value;
  }

  return ref;
}
function FreeNumberArrayReference(numberArrayReference){
  delete(numberArrayReference.numberArray);
  delete(numberArrayReference);
}
function CreateStringReference(value){
  var ref;

  ref = {};
  ref.string = value;

  return ref;
}
function CreateStringReferenceLengthValue(lengthx, value){
  var ref;
  var i;

  ref = {};
  ref.string = [];
  ref.string.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    ref.string[i] = value;
  }

  return ref;
}
function FreeStringReference(stringReference){
  delete(stringReference.string);
  delete(stringReference);
}
function CreateStringArrayReference(strings){
  var ref;

  ref = {};
  ref.stringArray = strings;

  return ref;
}
function CreateStringArrayReferenceLengthValue(lengthx, value){
  var ref;
  var i;

  ref = {};
  ref.stringArray = [];
  ref.stringArray.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    ref.stringArray[i] = CreateStringReference(value);
  }

  return ref;
}
function FreeStringArrayReference(stringArrayReference){
  var i;

  for(i = 0; i < stringArrayReference.stringArray.length; i = i + 1){
    delete(stringArrayReference.stringArray[i]);
  }
  delete(stringArrayReference.stringArray);
  delete(stringArrayReference);
}
function mathNegate(x){
  return  -x;
}
function mathPositive(x){
  return  +x;
}
function mathFactorial(x){
  var i, f;

  f = 1;

  for(i = 2; i <= x; i = i + 1){
    f = f*i;
  }

  return f;
}
function mathRound(x){
  return Math.floor(x + 0.5);
}
function mathBankersRound(x){
  var r;

  if(mathAbsolute(x - mathTruncate(x)) == 0.5){
    if( !mathDivisibleBy(mathRound(x), 2) ){
      r = mathRound(x) - 1;
    }else{
      r = mathRound(x);
    }
  }else{
    r = mathRound(x);
  }

  return r;
}
function mathCeil(x){
  return Math.ceil(x);
}
function mathFloor(x){
  return Math.floor(x);
}
function mathTruncate(x){
  var t;

  if(x >= 0){
    t = Math.floor(x);
  }else{
    t = Math.ceil(x);
  }

  return t;
}
function mathAbsolute(x){
  return Math.abs(x);
}
function mathLogarithm(x){
  return Math.log10(x);
}
function mathNaturalLogarithm(x){
  return Math.log(x);
}
function mathSin(x){
  return Math.sin(x);
}
function mathCos(x){
  return Math.cos(x);
}
function mathTan(x){
  return Math.tan(x);
}
function mathAsin(x){
  return Math.asin(x);
}
function mathAcos(x){
  return Math.acos(x);
}
function mathAtan(x){
  return Math.atan(x);
}
function mathAtan2(y, x){
  var a;

  /* Atan2 is an invalid operation when x = 0 and y = 0, but this method does not return errors. */
  a = 0;

  if(x > 0){
    a = mathAtan(y/x);
  }else if(x < 0 && y >= 0){
    a = mathAtan(y/x) + Math.PI;
  }else if(x < 0 && y < 0){
    a = mathAtan(y/x) - Math.PI;
  }else if(x == 0 && y > 0){
    a = Math.PI/2;
  }else if(x == 0 && y < 0){
    a =  -Math.PI/2;
  }

  return a;
}
function mathSquareroot(x){
  return Math.sqrt(x);
}
function mathExp(x){
  return Math.exp(x);
}
function mathDivisibleBy(a, b){
  return ((a%b) == 0);
}
function mathCombinations(n, k){
  var i, j, c;

  c = 1;
  j = 1;
  i = n - k + 1;

  for(; i <= n; ){
    c = c*i;
    c = c/j;

    i = i + 1;
    j = j + 1;
  }

  return c;
}
function mathPermutations(n, k){
  var i, c;

  c = 1;

  for(i = n - k + 1; i <= n; i = i + 1){
    c = c*i;
  }

  return c;
}
function mathEpsilonCompare(a, b, epsilon){
  return Math.abs(a - b) < epsilon;
}
function mathGreatestCommonDivisor(a, b){
  var t;

  for(; b != 0; ){
    t = b;
    b = a%b;
    a = t;
  }

  return a;
}
function mathGCDWithSubtraction(a, b){
  var g;

  if(a == 0){
    g = b;
  }else{
    for(; b != 0; ){
      if(a > b){
        a = a - b;
      }else{
        b = b - a;
      }
    }

    g = a;
  }

  return g;
}
function mathIsInteger(a){
  return (a - Math.floor(a)) == 0;
}
function mathGreatestCommonDivisorWithCheck(a, b, gcdReference){
  var success;
  var gcd;

  if(mathIsInteger(a) && mathIsInteger(b)){
    gcd = mathGreatestCommonDivisor(a, b);
    gcdReference.numberValue = gcd;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function mathLeastCommonMultiple(a, b){
  var lcm;

  if(a > 0 && b > 0){
    lcm = Math.abs(a*b)/mathGreatestCommonDivisor(a, b);
  }else{
    lcm = 0;
  }

  return lcm;
}
function mathSign(a){
  var s;

  if(a > 0){
    s = 1;
  }else if(a < 0){
    s =  -1;
  }else{
    s = 0;
  }

  return s;
}
function mathMax(a, b){
  return Math.max(a, b);
}
function mathMin(a, b){
  return Math.min(a, b);
}
function mathPower(a, b){
  return Math.pow(a, b);
}
function mathGamma(x){
  return mathLanczosApproximation(x);
}
function mathLogGamma(x){
  return Math.log(mathGamma(x));
}
function mathLanczosApproximation(z){
  var p;
  var i, y, t, x;

  p = [];
  p.length = 8;
  p[0] = 676.5203681218851;
  p[1] =  -1259.1392167224028;
  p[2] = 771.32342877765313;
  p[3] =  -176.61502916214059;
  p[4] = 12.507343278686905;
  p[5] =  -0.13857109526572012;
  p[6] = 9.9843695780195716e-6;
  p[7] = 1.5056327351493116e-7;

  if(z < 0.5){
    y = Math.PI/(Math.sin(Math.PI*z)*mathLanczosApproximation(1 - z));
  }else{
    z = z - 1;
    x = 0.99999999999980993;
    for(i = 0; i < p.length; i = i + 1){
      x = x + p[i]/(z + i + 1);
    }
    t = z + p.length - 0.5;
    y = Math.sqrt(2*Math.PI)*Math.pow(t, z + 0.5)*Math.exp( -t)*x;
  }

  return y;
}
function mathBeta(x, y){
  return mathGamma(x)*mathGamma(y)/mathGamma(x + y);
}
function mathSinh(x){
  return (Math.exp(x) - Math.exp( -x))/2;
}
function mathCosh(x){
  return (Math.exp(x) + Math.exp( -x))/2;
}
function mathTanh(x){
  return mathSinh(x)/mathCosh(x);
}
function mathCot(x){
  return 1/Math.tan(x);
}
function mathSec(x){
  return 1/Math.cos(x);
}
function mathCsc(x){
  return 1/Math.sin(x);
}
function mathCoth(x){
  return mathCosh(x)/mathSinh(x);
}
function mathSech(x){
  return 1/mathCosh(x);
}
function mathCsch(x){
  return 1/mathSinh(x);
}
function mathError(x){
  var y, t, tau, c1, c2, c3, c4, c5, c6, c7, c8, c9, c10;

  if(x == 0){
    y = 0;
  }else if(x < 0){
    y =  -mathError( -x);
  }else{
    c1 =  -1.26551223;
    c2 =  +1.00002368;
    c3 =  +0.37409196;
    c4 =  +0.09678418;
    c5 =  -0.18628806;
    c6 =  +0.27886807;
    c7 =  -1.13520398;
    c8 =  +1.48851587;
    c9 =  -0.82215223;
    c10 =  +0.17087277;

    t = 1/(1 + 0.5*Math.abs(x));

    tau = t*Math.exp( -(Math.pow(x, 2)) + c1 + t*(c2 + t*(c3 + t*(c4 + t*(c5 + t*(c6 + t*(c7 + t*(c8 + t*(c9 + t*c10)))))))));

    y = 1 - tau;
  }

  return y;
}
function mathErrorInverse(x){
  var y, a, t;

  a = (8*(Math.PI - 3))/(3*Math.PI*(4 - Math.PI));

  t = 2/(Math.PI*a) + Math.log(1 - Math.pow(x, 2))/2;
  y = mathSign(x)*Math.sqrt(Math.sqrt(Math.pow(t, 2) - Math.log(1 - Math.pow(x, 2))/a) - t);

  return y;
}
function mathFallingFactorial(x, n){
  var k, y;

  y = 1;

  for(k = 0; k <= n - 1; k = k + 1){
    y = y*(x - k);
  }

  return y;
}
function mathRisingFactorial(x, n){
  var k, y;

  y = 1;

  for(k = 0; k <= n - 1; k = k + 1){
    y = y*(x + k);
  }

  return y;
}
function mathHypergeometric(a, b, c, z, maxIterations, precision){
  var y;

  if(Math.abs(z) >= 0.5){
    y = Math.pow(1 - z,  -a)*mathHypergeometricDirect(a, c - b, c, z/(z - 1), maxIterations, precision);
  }else{
    y = mathHypergeometricDirect(a, b, c, z, maxIterations, precision);
  }

  return y;
}
function mathHypergeometricDirect(a, b, c, z, maxIterations, precision){
  var y, yp, n;
  var done;

  y = 0;
  done = false;

  for(n = 0; n < maxIterations &&  !done ; n = n + 1){
    yp = mathRisingFactorial(a, n)*mathRisingFactorial(b, n)/mathRisingFactorial(c, n)*Math.pow(z, n)/mathFactorial(n);
    if(Math.abs(yp) < precision){
      done = true;
    }
    y = y + yp;
  }

  return y;
}
function mathBernouilliNumber(n){
  return mathAkiyamaTanigawaAlgorithm(n);
}
function mathAkiyamaTanigawaAlgorithm(n){
  var m, j, B;
  var A;

  A = [];
  A.length = n + 1;

  for(m = 0; m <= n; m = m + 1){
    A[m] = 1/(m + 1);
    for(j = m; j >= 1; j = j - 1){
      A[j - 1] = j*(A[j - 1] - A[j]);
    }
  }

  B = A[0];

  delete(A);

  return B;
}
function cCreateComplexNumber(re, im){
  var z;

  z = {};
  z.re = re;
  z.im = im;

  return z;
}
function cCreatePolarComplexNumber(r, phi){
  var p;

  p = {};
  p.r = r;
  p.phi = phi;

  return p;
}
function cAdd(z1, z2){
  var a, b, c, d;

  a = z1.re;
  b = z1.im;
  c = z2.re;
  d = z2.im;

  z1.re = a + c;
  z1.im = b + d;
}
function cAddToNew(z1, z2){
  var x;
  var a, b, c, d;

  a = z1.re;
  b = z1.im;
  c = z2.re;
  d = z2.im;

  x = {};

  x.re = a + c;
  x.im = b + d;

  return x;
}
function cSub(z1, z2){
  var a, b, c, d;

  a = z1.re;
  b = z1.im;
  c = z2.re;
  d = z2.im;

  z1.re = a - c;
  z1.im = b - d;
}
function cSubToNew(z1, z2){
  var x;
  var a, b, c, d;

  a = z1.re;
  b = z1.im;
  c = z2.re;
  d = z2.im;

  x = {};

  x.re = a - c;
  x.im = b - d;

  return x;
}
function cMul(z1, z2){
  var a, b, c, d;

  a = z1.re;
  b = z1.im;
  c = z2.re;
  d = z2.im;

  z1.re = a*c - b*d;
  z1.im = b*c + a*d;
}
function cMulToNew(z1, z2){
  var x;
  var a, b, c, d;

  a = z1.re;
  b = z1.im;
  c = z2.re;
  d = z2.im;

  x = {};

  x.re = a*c - b*d;
  x.im = b*c + a*d;

  return x;
}
function cDiv(z1, z2){
  var a, b, c, d;

  a = z1.re;
  b = z1.im;
  c = z2.re;
  d = z2.im;

  z1.re = (a*c + b*d)/(Math.pow(c, 2) + Math.pow(d, 2));
  z1.im = (b*c - a*d)/(Math.pow(c, 2) + Math.pow(d, 2));
}
function cDivToNew(z1, z2){
  var x;
  var a, b, c, d;

  a = z1.re;
  b = z1.im;
  c = z2.re;
  d = z2.im;

  x = {};

  x.re = (a*c + b*d)/(Math.pow(c, 2) + Math.pow(d, 2));
  x.im = (b*c - a*d)/(Math.pow(c, 2) + Math.pow(d, 2));

  return x;
}
function cConjugate(z){
  z.im =  -z.im;
}
function cConjugateToNew(z){
  var x;

  x = {};

  x.re = z.re;
  x.im =  -z.im;

  return x;
}
function cAbs(z){
  var x;

  x = Math.sqrt(Math.pow(z.re, 2) + Math.pow(z.im, 2));

  return x;
}
function cArg(z){
  var x;

  x = mathAtan2(z.im, z.re);

  return x;
}
function cCreatePolarFromComplexNumber(z){
  var x;

  x = {};

  x.r = cAbs(z);
  x.phi = cArg(z);

  return x;
}
function cCreateComplexFromPolar(p){
  var z;

  z = {};

  z.re = p.r*Math.cos(p.phi);
  z.im = p.r*Math.sin(p.phi);

  return z;
}
function cRe(z){
  return z.re;
}
function cIm(z){
  return z.im;
}
function cAddPolar(p1, p2){
  var x;
  var z1, z2;

  z1 = cCreateComplexFromPolar(p1);
  z2 = cCreateComplexFromPolar(p2);

  cAdd(z1, z2);

  x = cCreatePolarFromComplexNumber(z1);

  p1.r = x.r;
  p1.phi = x.phi;

  delete(z1);
  delete(z2);
  delete(x);
}
function cAddPolarToNew(p1, p2){
  var x;
  var z1, z2;

  z1 = cCreateComplexFromPolar(p1);
  z2 = cCreateComplexFromPolar(p2);

  cAdd(z1, z2);

  x = cCreatePolarFromComplexNumber(z1);

  delete(z1);
  delete(z2);

  return x;
}
function cSubPolar(p1, p2){
  var x;
  var z1, z2;

  z1 = cCreateComplexFromPolar(p1);
  z2 = cCreateComplexFromPolar(p2);

  cSub(z1, z2);

  x = cCreatePolarFromComplexNumber(z1);

  p1.r = x.r;
  p1.phi = x.phi;

  delete(z1);
  delete(z2);
  delete(x);
}
function cSubPolarToNew(p1, p2){
  var x;
  var z1, z2;

  z1 = cCreateComplexFromPolar(p1);
  z2 = cCreateComplexFromPolar(p2);

  cSub(z1, z2);

  x = cCreatePolarFromComplexNumber(z1);

  delete(z1);
  delete(z2);

  return x;
}
function cMulPolar(p1, p2){
  var r1, r2, phi1, phi2;

  r1 = p1.r;
  r2 = p2.r;
  phi1 = p1.phi;
  phi2 = p2.phi;

  p1.r = r1*r2;
  p1.phi = phi1 + phi2;
}
function cMulPolarToNew(p1, p2){
  var x;
  var r1, r2, phi1, phi2;

  r1 = p1.r;
  r2 = p2.r;
  phi1 = p1.phi;
  phi2 = p2.phi;

  x = {};

  x.r = r1*r2;
  x.phi = phi1 + phi2;

  return x;
}
function cDivPolar(p1, p2){
  var r1, r2, phi1, phi2;

  r1 = p1.r;
  r2 = p2.r;
  phi1 = p1.phi;
  phi2 = p2.phi;

  p1.r = r1/r2;
  p1.phi = phi1 - phi2;
}
function cDivPolarToNew(p1, p2){
  var x;
  var r1, r2, phi1, phi2;

  r1 = p1.r;
  r2 = p2.r;
  phi1 = p1.phi;
  phi2 = p2.phi;

  x = {};

  x.r = r1/r2;
  x.phi = phi1 - phi2;

  return x;
}
function cSquareRoot(z){
  var a, b, m;

  a = z.re;
  b = z.im;

  m = Math.sqrt(Math.pow(a, 2) + Math.pow(b, 2));

  z.re = Math.sqrt((m + a)/2);
  z.im = mathSign(b)*Math.sqrt((m - a)/2);
}
function cPowerPolar(p, n){
  p.r = Math.pow(p.r, n);
  p.phi = p.phi*n;
}
function cPowerToNew(z, n){
  var p;
  var zp;

  p = cCreatePolarFromComplexNumber(z);
  cPowerPolar(p, n);
  zp = cCreateComplexFromPolar(p);

  delete(p);

  return zp;
}
function cPower(z, n){
  var zp;

  zp = cPowerToNew(z, n);
  z.re = zp.re;
  z.im = zp.im;

  delete(zp);
}
function cNegate(z){
  z.re = mathNegate(z.re);
  z.im = mathNegate(z.im);
}
function cAssignComplexByValues(s, re, im){
  s.re = re;
  s.im = im;
}
function cAssignComplex(a, b){
  a.re = b.re;
  a.im = b.im;
}
function cEpsilonCompareComplex(a, b, epsilon){
  return mathEpsilonCompare(a.re, b.re, epsilon) && mathEpsilonCompare(a.im, b.im, epsilon);
}
function cExpComplex(x){
  var re, im;

  re = Math.exp(x.re)*Math.cos(x.im);
  im = Math.exp(x.re)*Math.sin(x.im);
  x.re = re;
  x.im = im;
}
function cSineComplex(x){
  var re, im;

  re = Math.sin(x.re)*mathCosh(x.im);
  im = Math.cos(x.re)*mathSinh(x.im);
  x.re = re;
  x.im = im;
}
function cCosineComplex(x){
  var re, im;

  re = Math.cos(x.re)*mathCosh(x.im);
  im = Math.sin(x.re)*mathSinh(x.im);
  x.re = re;
  x.im = im;
}
function cComplexToString(a){
  var str, number;
  var ll;
  var i;

  ll = CreateLinkedListCharacter();

  number = nCreateStringDecimalFromNumber(a.re);

  for(i = 0; i < number.length; i = i + 1){
    LinkedListAddCharacter(ll, number[i]);
  }

  delete(number);

  if(a.im < 0){
    LinkedListAddCharacter(ll, '-');
    number = nCreateStringDecimalFromNumber( -a.im);
  }else{
    LinkedListAddCharacter(ll, '+');
    number = nCreateStringDecimalFromNumber(a.im);
  }

  for(i = 0; i < number.length; i = i + 1){
    LinkedListAddCharacter(ll, number[i]);
  }

  delete(number);

  LinkedListAddCharacter(ll, 'i');

  str = LinkedListCharactersToArray(ll);
  FreeLinkedListCharacter(ll);

  return str;
}
function pPolynomialToTextDirect(p, x){
  var ll;
  var str;
  var i, c, j;
  var buffer;

  buffer = {};

  ll = CreateLinkedListCharacter();

  if(p.length == 0){
    LinkedListAddCharacter(ll, '0');
  }else{
    for(i = 0; i < p.length; i = i + 1){
      c = p[i];
      if(c < 0){
        LinkedListAddCharacter(ll, '-');
      }else{
        LinkedListAddCharacter(ll, '+');
      }

      nCreateStringFromNumberWithCheck(Math.abs(c), 10, buffer);
      pLinkedListCharactersAddString(ll, buffer.string);
      delete(buffer.string);

      pLinkedListCharactersAddString(ll, x);
      LinkedListAddCharacter(ll, '^');

      nCreateStringFromNumberWithCheck(i, 10, buffer);
      pLinkedListCharactersAddString(ll, buffer.string);
      delete(buffer.string);
    }
  }

  str = LinkedListCharactersToArray(ll);
  FreeLinkedListCharacter(ll);

  return str;
}
function pLinkedListCharactersAddString(ll, str){
  var j;

  for(j = 0; j < str.length; j = j + 1){
    LinkedListAddCharacter(ll, str[j]);
  }
}
function pGenerateCommonRenderSpecification(p, showCoefficient, sign, coefficient, showPower, showX){
  var i, zeros, c;
  var setZero;

  setZero = false;

  if(p.length == 0){
    setZero = true;
  }else{
    zeros = 0;
    for(i = 0; i < p.length; i = i + 1){
      if(p[i] == 0){
        zeros = zeros + 1;
      }
    }

    if(zeros == p.length){
      setZero = true;
    }else{
      showCoefficient.booleanArray = [];
      showCoefficient.booleanArray.length = p.length;
      sign.string = [];
      sign.string.length = p.length;
      coefficient.numberArray = [];
      coefficient.numberArray.length = p.length;
      showPower.booleanArray = [];
      showPower.booleanArray.length = p.length;
      showX.booleanArray = [];
      showX.booleanArray.length = p.length;

      for(i = 0; i < p.length; i = i + 1){
        c = p[i];

        if(c < 0){
          sign.string[i] = '-';
        }else{
          sign.string[i] = '+';
        }
        coefficient.numberArray[i] = Math.abs(p[i]);
        if(c == 0){
          showCoefficient.booleanArray[i] = false;
        }else{
if(Math.abs(c) == 1 && i > 0){
            showCoefficient.booleanArray[i] = false;
          }else{
            showCoefficient.booleanArray[i] = true;
          }

          if(i == 0){
            showX.booleanArray[i] = false;
            showPower.booleanArray[i] = false;
          }else{
            showX.booleanArray[i] = true;
            if(i == 1){
              showPower.booleanArray[i] = false;
            }else{
              showPower.booleanArray[i] = true;
            }
          }
        }
      }
    }
  }

  if(setZero){
    showCoefficient.booleanArray = [];
    showCoefficient.booleanArray.length = 1;
    sign.string = [];
    sign.string.length = 1;
    coefficient.numberArray = [];
    coefficient.numberArray.length = 1;
    showPower.booleanArray = [];
    showPower.booleanArray.length = 1;
    showX.booleanArray = [];
    showX.booleanArray.length = 1;

    showCoefficient.booleanArray[0] = true;
    sign.string[0] = '+';
    coefficient.numberArray[0] = 0;
    showPower.booleanArray[0] = true;
    showX.booleanArray[0] = false;
  }
}
function pPolynomialToText(p, x){
  var ll;
  var str;
  var i, c;
  var buffer;
  var showCoefficient, showPower, showX;
  var sign;
  var coefficient;
  var hasPrinted;

  showCoefficient = CreateBooleanArrayReferenceLengthValue(0, false);
  showPower = CreateBooleanArrayReferenceLengthValue(0, false);
  showX = CreateBooleanArrayReferenceLengthValue(0, false);
  sign = CreateStringReferenceLengthValue(0, ' ');
  coefficient = CreateNumberArrayReferenceLengthValue(0, 0);

  pGenerateCommonRenderSpecification(p, showCoefficient, sign, coefficient, showPower, showX);

  buffer = CreateStringReferenceLengthValue(0, ' ');

  ll = CreateLinkedListCharacter();

  hasPrinted = false;
  for(i = 0; i < showCoefficient.booleanArray.length; i = i + 1){
    c = p[i];

    if(showCoefficient.booleanArray[i] || showX.booleanArray[i]){
      if( !hasPrinted  && c >= 0){
      }else{
        LinkedListAddCharacter(ll, sign.string[i]);
      }

      if(showCoefficient.booleanArray[i]){
        nCreateStringFromNumberWithCheck(coefficient.numberArray[i], 10, buffer);
        pLinkedListCharactersAddString(ll, buffer.string);
        delete(buffer.string);
        hasPrinted = true;
      }

      if(showX.booleanArray[i]){
        pLinkedListCharactersAddString(ll, x);
        hasPrinted = true;
        if(showPower.booleanArray[i]){
          LinkedListAddCharacter(ll, '^');
          nCreateStringFromNumberWithCheck(i, 10, buffer);
          pLinkedListCharactersAddString(ll, buffer.string);
          delete(buffer.string);
        }
      }
    }
  }

  str = LinkedListCharactersToArray(ll);
  FreeLinkedListCharacter(ll);

  return str;
}
function pComplexPolynomialToTextDirect(p, x){
  var ll;
  var str, number;
  var i;
  var c;
  var buffer;

  buffer = {};

  ll = CreateLinkedListCharacter();

  if(p.cs.length == 0){
    LinkedListAddCharacter(ll, '0');
  }else{
    for(i = 0; i < p.cs.length; i = i + 1){
      c = p.cs[i];
      if(i > 0){
        LinkedListAddCharacter(ll, '+');
      }

      number = cComplexToString(c);
      LinkedListAddCharacter(ll, '(');
      pLinkedListCharactersAddString(ll, number);
      LinkedListAddCharacter(ll, ')');
      delete(number);

      pLinkedListCharactersAddString(ll, x);
      LinkedListAddCharacter(ll, '^');

      nCreateStringFromNumberWithCheck(i, 10, buffer);
      pLinkedListCharactersAddString(ll, buffer.string);
      delete(buffer.string);
    }
  }

  str = LinkedListCharactersToArray(ll);
  FreeLinkedListCharacter(ll);

  return str;
}
function pAdd(a, b){
  var i, nr;

  nr = b.length;

  for(i = 0; i < nr; i = i + 1){
    a[i] = a[i] + b[i];
  }
}
function pSubtract(a, b){
  var i, nr;

  nr = b.length;

  for(i = 0; i < nr; i = i + 1){
    a[i] = a[i] - b[i];
  }
}
function pMultiply(c, a, b){
  var k, n, m, i;
  var av, bv;

  n = pDegree(a);
  m = pDegree(b);

  pFill(c, 0);

  for(i = 0; i <= n + m; i = i + 1){
    c[i] = 0;
    for(k = 0; k <= i && k < a.length && i - k < b.length; k = k + 1){
      av = a[k];
      bv = b[i - k];
      c[i] = c[i] + av*bv;
    }
  }
}
function pDivide(q, r, n, d){
  var t, t1;
  var tcoff, tdegree, i;
  var deg;
  var rd, dd;

  pFill(q, 0);
  pAssign(r, n);
  deg = pDegree(n);
  t = pCreatePolynomial(deg);
  t1 = pCreatePolynomial(deg);

  rd = pDegree(r);
  dd = pDegree(d);
  for(i = 0; i < deg + 1 &&  !pIsZero(r)  && rd - i >= dd; i = i + 1){
    pFill(t, 0);
    tdegree = rd - i - dd;
    tcoff = r[rd - i]/d[dd];
    t[tdegree] = tcoff;
    pAdd(q, t);
    pFill(t1, 0);
    pMultiply(t1, t, d);
    pSubtract(r, t1);
  }

  delete(t);
  delete(t1);
}
function pIsZero(a){
  var i;
  var itIsZero;

  itIsZero = true;

  for(i = 0; i < a.length; i = i + 1){
    if(a[i] != 0){
      itIsZero = false;
    }
  }

  return itIsZero;
}
function pAssign(a, b){
  var i, nr;

  nr = b.length;

  for(i = 0; i < nr; i = i + 1){
    a[i] = b[i];
  }
}
function pCreatePolynomial(deg){
  var p;

  p = [];
  p.length = deg + 1;

  pFill(p, 0);

  return p;
}
function pFill(p, value){
  var i;

  for(i = 0; i < p.length; i = i + 1){
    p[i] = value;
  }
}
function pDegree(A){
  var i;
  var deg;
  var done;

  done = false;
  deg = 0;
  for(i = A.length - 1; i >= 0 &&  !done ; i = i - 1){
    if(A[i] != 0){
      deg = i;
      done = true;
    }
  }

  return deg;
}
function pLead(A){
  var deg;

  deg = pDegree(A);

  return A[deg];
}
function pEvaluate(A, x){
  return pEvaluateWithHornersMethod(A, x);
}
function pEvaluateWithHornersMethod(A, x){
  var r, i;

  r = 0;

  for(i = A.length - 1; i >= 0; i = i - 1){
    r = r*x;
    r = A[i] + r;
  }

  return r;
}
function pEvaluateWithPowers(A, x){
  var r, i;

  r = 0;

  for(i = 0; i < A.length; i = i + 1){
    r = r + A[i]*Math.pow(x, i);
  }

  return r;
}
function pEvaluateDerivative(A, x, n){
  var r, i, v;

  r = 0;

  for(i = 0; i < A.length; i = i + 1){
    if(i - n >= 0){
      /*v = A[(int) i] * mFactorial(n) * mCombinations(i, n) * pow(x, i - n); */
      v = A[i]*mathPermutations(i, n)*Math.pow(x, i - n);
      r = r + v;
    }
  }

  return r;
}
function pDerivative(A){
  var i, degree;

  degree = 0;
  for(i = 1; i < A.length; i = i + 1){
    degree = degree + 1;
    A[i - 1] = degree*A[i];
  }

  A[A.length - 1] = 0;
}
function pAddComplex(a, b){
  var i, nr;

  nr = a.cs.length;

  for(i = 0; i < nr; i = i + 1){
    cAdd(a.cs[i], b.cs[i]);
  }
}
function pSubtractComplex(a, b){
  var i, nr;

  nr = a.cs.length;

  for(i = 0; i < nr; i = i + 1){
    cSub(a.cs[i], b.cs[i]);
  }
}
function pIsZeroComplex(a){
  var i;
  var itIsZero;

  itIsZero = true;

  for(i = 0; i < a.cs.length; i = i + 1){
    if(a.cs[i].re != 0 && a.cs[i].im != 0){
      itIsZero = false;
    }
  }

  return itIsZero;
}
function pAssignComplex(a, b){
  var i, nr;

  nr = b.cs.length;

  for(i = 0; i < nr; i = i + 1){
    cAssignComplex(a.cs[i], b.cs[i]);
  }
}
function pCreateComplexPolynomial(deg){
  var p;
  var i;

  p = {};
  p.cs = [];
  p.cs.length = deg + 1;

  for(i = 0; i < deg + 1; i = i + 1){
    p.cs[i] = {};
  }

  pFillComplex(p, 0, 0);

  return p;
}
function pFillComplex(p, re, im){
  var i;
  var c;

  c = cCreateComplexNumber(re, im);

  for(i = 0; i < p.cs.length; i = i + 1){
    cAssignComplex(p.cs[i], c);
  }

  delete(c);
}
function pDegreeComplex(A){
  var i;
  var deg;
  var done;

  done = false;
  deg = 0;
  for(i = A.cs.length - 1; i >= 0 &&  !done ; i = i - 1){
    if(A.cs[i].re != 0 && A.cs[i].im != 0){
      deg = i;
      done = true;
    }
  }

  return deg;
}
function pLeadComplex(A){
  var deg;

  deg = pDegreeComplex(A);

  return A.cs[deg];
}
function pEvaluateComplex(A, x){
  var i;
  var r, t;

  r = cCreateComplexNumber(0, 0);
  t = cCreateComplexNumber(0, 0);

  for(i = 0; i < A.cs.length; i = i + 1){
    cAssignComplex(t, x);
    cPower(t, i);
    cMul(t, A.cs[i]);
    cAdd(r, t);
  }

  return r;
}
function pTotalNumberOfRoots(p){
  return pDegree(p);
}
function WriteStringToStingStream(stream, index, src){
  var i;

  for(i = 0; i < src.length; i = i + 1){
    stream[index.numberValue + i] = src[i];
  }
  index.numberValue = index.numberValue + src.length;
}
function WriteCharacterToStingStream(stream, index, src){
  stream[index.numberValue] = src;
  index.numberValue = index.numberValue + 1;
}
function WriteBooleanToStingStream(stream, index, src){
  if(src){
    WriteStringToStingStream(stream, index, "true".split(''));
  }else{
    WriteStringToStingStream(stream, index, "false".split(''));
  }
}
function SubstringWithCheck(string, from, to, stringReference){
  var success;

  if(from >= 0 && from <= string.length && to >= 0 && to <= string.length && from <= to){
    stringReference.string = Substring(string, from, to);
    success = true;
  }else{
    success = false;
  }

  return success;
}
function Substring(string, from, to){
  var n;
  var i, lengthx;

  lengthx = to - from;

  n = [];
  n.length = lengthx;

  for(i = from; i < to; i = i + 1){
    n[i - from] = string[i];
  }

  return n;
}
function AppendString(s1, s2){
  var newString;

  newString = ConcatenateString(s1, s2);

  delete(s1);

  return newString;
}
function ConcatenateString(s1, s2){
  var newString;
  var i;

  newString = [];
  newString.length = s1.length + s2.length;

  for(i = 0; i < s1.length; i = i + 1){
    newString[i] = s1[i];
  }

  for(i = 0; i < s2.length; i = i + 1){
    newString[s1.length + i] = s2[i];
  }

  return newString;
}
function AppendCharacter(string, c){
  var newString;

  newString = ConcatenateCharacter(string, c);

  delete(string);

  return newString;
}
function ConcatenateCharacter(string, c){
  var newString;
  var i;
  newString = [];
  newString.length = string.length + 1;

  for(i = 0; i < string.length; i = i + 1){
    newString[i] = string[i];
  }

  newString[string.length] = c;

  return newString;
}
function SplitByCharacter(toSplit, splitBy){
  var split;
  var stringToSplitBy;

  stringToSplitBy = [];
  stringToSplitBy.length = 1;
  stringToSplitBy[0] = splitBy;

  split = SplitByString(toSplit, stringToSplitBy);

  delete(stringToSplitBy);

  return split;
}
function IndexOfCharacter(string, character, indexReference){
  var i;
  var found;

  found = false;
  for(i = 0; i < string.length &&  !found ; i = i + 1){
    if(string[i] == character){
      found = true;
      indexReference.numberValue = i;
    }
  }

  return found;
}
function SubstringEqualsWithCheck(string, from, substring, equalsReference){
  var success;

  if(from < string.length){
    success = true;
    equalsReference.booleanValue = SubstringEquals(string, from, substring);
  }else{
    success = false;
  }

  return success;
}
function SubstringEquals(string, from, substring){
  var i;
  var equal;

  equal = true;
  for(i = 0; i < substring.length && equal; i = i + 1){
    if(string[from + i] != substring[i]){
      equal = false;
    }
  }

  return equal;
}
function IndexOfString(string, substring, indexReference){
  var i;
  var found;

  found = false;
  for(i = 0; i < string.length - substring.length + 1 &&  !found ; i = i + 1){
    if(SubstringEquals(string, i, substring)){
      found = true;
      indexReference.numberValue = i;
    }
  }

  return found;
}
function ContainsCharacter(string, character){
  var i;
  var found;

  found = false;
  for(i = 0; i < string.length &&  !found ; i = i + 1){
    if(string[i] == character){
      found = true;
    }
  }

  return found;
}
function ContainsString(string, substring){
  return IndexOfString(string, substring, {});
}
function ToUpperCase(string){
  var i;

  for(i = 0; i < string.length; i = i + 1){
    string[i] = charToUpperCase(string[i]);
  }
}
function ToLowerCase(string){
  var i;

  for(i = 0; i < string.length; i = i + 1){
    string[i] = charToLowerCase(string[i]);
  }
}
function EqualsIgnoreCase(a, b){
  var equal;
  var i;

  if(a.length == b.length){
    equal = true;
    for(i = 0; i < a.length && equal; i = i + 1){
      if(charToLowerCase(a[i]) != charToLowerCase(b[i])){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function ReplaceString(string, toReplace, replaceWith){
  var result;
  var i;
  var equalsReference;
  var success;

  equalsReference = {};
  result = [];
  result.length = 0;

  for(i = 0; i < string.length; ){
    success = SubstringEqualsWithCheck(string, i, toReplace, equalsReference);
    if(success){
      success = equalsReference.booleanValue;
    }

    if(success && toReplace.length > 0){
      result = ConcatenateString(result, replaceWith);
      i = i + toReplace.length;
    }else{
      result = ConcatenateCharacter(result, string[i]);
      i = i + 1;
    }
  }

  return result;
}
function ReplaceCharacter(string, toReplace, replaceWith){
  var result;
  var i;

  result = [];
  result.length = 0;

  for(i = 0; i < string.length; i = i + 1){
    if(string[i] == toReplace){
      result = ConcatenateCharacter(result, replaceWith);
    }else{
      result = ConcatenateCharacter(result, string[i]);
    }
  }

  return result;
}
function Trim(string){
  var result;
  var i, lastWhitespaceLocationStart, lastWhitespaceLocationEnd;
  var firstNonWhitespaceFound;

  /* Find whitepaces at the start. */
  lastWhitespaceLocationStart =  -1;
  firstNonWhitespaceFound = false;
  for(i = 0; i < string.length &&  !firstNonWhitespaceFound ; i = i + 1){
    if(charIsWhiteSpace(string[i])){
      lastWhitespaceLocationStart = i;
    }else{
      firstNonWhitespaceFound = true;
    }
  }

  /* Find whitepaces at the end. */
  lastWhitespaceLocationEnd = string.length;
  firstNonWhitespaceFound = false;
  for(i = string.length - 1; i >= 0 &&  !firstNonWhitespaceFound ; i = i - 1){
    if(charIsWhiteSpace(string[i])){
      lastWhitespaceLocationEnd = i;
    }else{
      firstNonWhitespaceFound = true;
    }
  }

  if(lastWhitespaceLocationStart < lastWhitespaceLocationEnd){
    result = Substring(string, lastWhitespaceLocationStart + 1, lastWhitespaceLocationEnd);
  }else{
    result = [];
    result.length = 0;
  }

  return result;
}
function StartsWith(string, start){
  var startsWithString;

  startsWithString = false;
  if(string.length >= start.length){
    startsWithString = SubstringEquals(string, 0, start);
  }

  return startsWithString;
}
function EndsWith(string, end){
  var endsWithString;

  endsWithString = false;
  if(string.length >= end.length){
    endsWithString = SubstringEquals(string, string.length - end.length, end);
  }

  return endsWithString;
}
function SplitByString(toSplit, splitBy){
  var split;
  var next;
  var i;
  var c;
  var n;

  split = [];
  split.length = 0;

  next = [];
  next.length = 0;
  for(i = 0; i < toSplit.length; ){
    c = toSplit[i];

    if(SubstringEquals(toSplit, i, splitBy)){
      if(split.length != 0 || i != 0){
        n = {};
        n.string = next;
        split = AddString(split, n);
        next = [];
        next.length = 0;
        i = i + splitBy.length;
      }
    }else{
      next = AppendCharacter(next, c);
      i = i + 1;
    }
  }

  if(next.length > 0){
    n = {};
    n.string = next;
    split = AddString(split, n);
  }

  return split;
}
function StringIsBefore(a, b){
  var before, equal, done;
  var i;

  before = false;
  equal = true;
  done = false;

  if(a.length == 0 && b.length > 0){
    before = true;
  }else{
    for(i = 0; i < a.length && i < b.length &&  !done ; i = i + 1){
      if(a[i] != b[i]){
        equal = false;
      }
      if(charCharacterIsBefore(a[i], b[i])){
        before = true;
      }
      if(charCharacterIsBefore(b[i], a[i])){
        done = true;
      }
    }

    if(equal){
      if(a.length < b.length){
        before = true;
      }
    }
  }

  return before;
}
function nCreateStringScientificNotationDecimalFromNumber(decimal){
  var mantissaReference, exponentReference;
  var multiplier, inc;
  var exponent;
  var done, isPositive;
  var result;

  mantissaReference = {};
  exponentReference = {};
  result = [];
  result.length = 0;
  done = false;
  exponent = 0;

  if(decimal < 0){
    isPositive = false;
    decimal =  -decimal;
  }else{
    isPositive = true;
  }

  if(decimal == 0){
    done = true;
  }

  if( !done ){
    multiplier = 0;
    inc = 0;

    if(decimal < 1){
      multiplier = 10;
      inc =  -1;
    }else if(decimal >= 10){
      multiplier = 0.1;
      inc = 1;
    }else{
      done = true;
    }

    if( !done ){
      for(; decimal >= 10 || decimal < 1; ){
        decimal = decimal*multiplier;
        exponent = exponent + inc;
      }
    }
  }

  nCreateStringFromNumberWithCheck(decimal, 10, mantissaReference);

  nCreateStringFromNumberWithCheck(exponent, 10, exponentReference);

  if( !isPositive ){
    result = AppendString(result, "-".split(''));
  }

  result = AppendString(result, mantissaReference.string);
  result = AppendString(result, "e".split(''));
  result = AppendString(result, exponentReference.string);

  return result;
}
function nCreateStringDecimalFromNumber(decimal){
  var stringReference;

  stringReference = {};

  /* This will succeed because base = 10. */
  nCreateStringFromNumberWithCheck(decimal, 10, stringReference);

  return stringReference.string;
}
function nCreateStringFromNumberWithCheck(decimal, base, stringReference){
  var string;
  var maximumDigits;
  var digitPosition;
  var hasPrintedPoint, isPositive;
  var i, d;
  var success;
  var characterReference;
  var c;

  isPositive = true;

  if(decimal < 0){
    isPositive = false;
    decimal =  -decimal;
  }

  if(decimal == 0){
    stringReference.string = "0".split('');
    success = true;
  }else{
    characterReference = {};

    if(mathIsInteger(base)){
      success = true;

      string = [];
      string.length = 0;

      maximumDigits = nGetMaximumDigitsForBase(base);

      digitPosition = nGetFirstDigitPosition(decimal, base);

      decimal = Math.round(decimal*Math.pow(base, maximumDigits - digitPosition - 1));

      hasPrintedPoint = false;

      if( !isPositive ){
        string = AppendCharacter(string, '-');
      }

      /* Print leading zeros. */
      if(digitPosition < 0){
        string = AppendCharacter(string, '0');
        string = AppendCharacter(string, '.');
        hasPrintedPoint = true;
        for(i = 0; i <  -digitPosition - 1; i = i + 1){
          string = AppendCharacter(string, '0');
        }
      }

      /* Print number. */
      for(i = 0; i < maximumDigits && success; i = i + 1){
        d = Math.floor(decimal/Math.pow(base, maximumDigits - i - 1));

        if(d >= base){
          d = base - 1;
        }

        if( !hasPrintedPoint  && digitPosition - i + 1 == 0){
          if(decimal != 0){
            string = AppendCharacter(string, '.');
          }
          hasPrintedPoint = true;
        }

        if(decimal == 0 && hasPrintedPoint){
        }else{
          success = nGetSingleDigitCharacterFromNumberWithCheck(d, base, characterReference);
          if(success){
            c = characterReference.characterValue;
            string = AppendCharacter(string, c);
          }
        }

        if(success){
          decimal = decimal - d*Math.pow(base, maximumDigits - i - 1);
        }
      }

      if(success){
        /* Print trailing zeros. */
        for(i = 0; i < digitPosition - maximumDigits + 1; i = i + 1){
          string = AppendCharacter(string, '0');
        }

        stringReference.string = string;
      }
    }else{
      success = false;
    }
  }

  /* Done */
  return success;
}
function nGetMaximumDigitsForBase(base){
  var t;

  t = Math.pow(10, 15);
  return Math.floor(Math.log10(t)/Math.log10(base));
}
function nGetFirstDigitPosition(decimal, base){
  var power;
  var t;

  power = Math.ceil(Math.log10(decimal)/Math.log10(base));

  t = decimal*Math.pow(base,  -power);
  if(t < base && t >= 1){
  }else if(t >= base){
    power = power + 1;
  }else if(t < 1){
    power = power - 1;
  }

  return power;
}
function nGetSingleDigitCharacterFromNumberWithCheck(c, base, characterReference){
  var numberTable;
  var success;

  numberTable = nGetDigitCharacterTable();

  if(c < base || c < numberTable.length){
    success = true;
    characterReference.characterValue = numberTable[c];
  }else{
    success = false;
  }

  return success;
}
function nGetDigitCharacterTable(){
  var numberTable;

  numberTable = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ".split('');

  return numberTable;
}
function nCreateNumberFromDecimalStringWithCheck(string, decimalReference, errorMessage){
  return nCreateNumberFromStringWithCheck(string, 10, decimalReference, errorMessage);
}
function nCreateNumberFromDecimalString(string){
  var doubleReference;
  var stringReference;
  var number;

  doubleReference = CreateNumberReference(0);
  stringReference = CreateStringReference("".split(''));
  nCreateNumberFromStringWithCheck(string, 10, doubleReference, stringReference);
  number = doubleReference.numberValue;

  delete(doubleReference);
  delete(stringReference);

  return number;
}
function nCreateNumberFromStringWithCheck(string, base, numberReference, errorMessage){
  var success;
  var numberIsPositive, exponentIsPositive;
  var beforePoint, afterPoint, exponent;

  numberIsPositive = CreateBooleanReference(true);
  exponentIsPositive = CreateBooleanReference(true);
  beforePoint = {};
  afterPoint = {};
  exponent = {};

  if(base >= 2 && base <= 36){
    success = nExtractPartsFromNumberString(string, base, numberIsPositive, beforePoint, afterPoint, exponentIsPositive, exponent, errorMessage);

    if(success){
      numberReference.numberValue = nCreateNumberFromParts(base, numberIsPositive.booleanValue, beforePoint.numberArray, afterPoint.numberArray, exponentIsPositive.booleanValue, exponent.numberArray);
    }
  }else{
    success = false;
    errorMessage.string = "Base must be from 2 to 36.".split('');
  }

  return success;
}
function nCreateNumberFromParts(base, numberIsPositive, beforePoint, afterPoint, exponentIsPositive, exponent){
  var n, i, p, e;

  n = 0;

  for(i = 0; i < beforePoint.length; i = i + 1){
    p = beforePoint[beforePoint.length - i - 1];

    n = n + p*Math.pow(base, i);
  }

  for(i = 0; i < afterPoint.length; i = i + 1){
    p = afterPoint[i];

    n = n + p*Math.pow(base,  -(i + 1));
  }

  if(exponent.length > 0){
    e = 0;
    for(i = 0; i < exponent.length; i = i + 1){
      p = exponent[exponent.length - i - 1];

      e = e + p*Math.pow(base, i);
    }

    if( !exponentIsPositive ){
      e =  -e;
    }

    n = n*Math.pow(base, e);
  }

  if( !numberIsPositive ){
    n =  -n;
  }

  return n;
}
function nExtractPartsFromNumberString(n, base, numberIsPositive, beforePoint, afterPoint, exponentIsPositive, exponent, errorMessages){
  var i;
  var success;

  i = 0;

  if(i < n.length){
    if(n[i] == '-'){
      numberIsPositive.booleanValue = false;
      i = i + 1;
    }else if(n[i] == '+'){
      numberIsPositive.booleanValue = true;
      i = i + 1;
    }

    success = nExtractPartsFromNumberStringFromSign(n, base, i, beforePoint, afterPoint, exponentIsPositive, exponent, errorMessages);
  }else{
    success = false;
    errorMessages.string = "Number cannot have length zero.".split('');
  }

  return success;
}
function nExtractPartsFromNumberStringFromSign(n, base, i, beforePoint, afterPoint, exponentIsPositive, exponent, errorMessages){
  var success, done;
  var count, j;

  done = false;
  count = 0;
  for(; i + count < n.length &&  !done ; ){
    if(nCharacterIsNumberCharacterInBase(n[i + count], base)){
      count = count + 1;
    }else{
      done = true;
    }
  }

  if(count >= 1){
    beforePoint.numberArray = [];
    beforePoint.numberArray.length = count;

    for(j = 0; j < count; j = j + 1){
      beforePoint.numberArray[j] = nGetNumberFromNumberCharacterForBase(n[i + j], base);
    }

    i = i + count;

    if(i < n.length){
      success = nExtractPartsFromNumberStringFromPointOrExponent(n, base, i, afterPoint, exponentIsPositive, exponent, errorMessages);
    }else{
      afterPoint.numberArray = [];
      afterPoint.numberArray.length = 0;
      exponent.numberArray = [];
      exponent.numberArray.length = 0;
      success = true;
    }
  }else{
    success = false;
    errorMessages.string = "Number must have at least one number after the optional sign.".split('');
  }

  return success;
}
function nExtractPartsFromNumberStringFromPointOrExponent(n, base, i, afterPoint, exponentIsPositive, exponent, errorMessages){
  var success, done;
  var count, j;

  if(n[i] == '.'){
    i = i + 1;

    if(i < n.length){
      done = false;
      count = 0;
      for(; i + count < n.length &&  !done ; ){
        if(nCharacterIsNumberCharacterInBase(n[i + count], base)){
          count = count + 1;
        }else{
          done = true;
        }
      }

      if(count >= 1){
        afterPoint.numberArray = [];
        afterPoint.numberArray.length = count;

        for(j = 0; j < count; j = j + 1){
          afterPoint.numberArray[j] = nGetNumberFromNumberCharacterForBase(n[i + j], base);
        }

        i = i + count;

        if(i < n.length){
          success = nExtractPartsFromNumberStringFromExponent(n, base, i, exponentIsPositive, exponent, errorMessages);
        }else{
          exponent.numberArray = [];
          exponent.numberArray.length = 0;
          success = true;
        }
      }else{
        success = false;
        errorMessages.string = "There must be at least one digit after the decimal point.".split('');
      }
    }else{
      success = false;
      errorMessages.string = "There must be at least one digit after the decimal point.".split('');
    }
  }else if(base <= 14 && (n[i] == 'e' || n[i] == 'E')){
    if(i < n.length){
      success = nExtractPartsFromNumberStringFromExponent(n, base, i, exponentIsPositive, exponent, errorMessages);
      afterPoint.numberArray = [];
      afterPoint.numberArray.length = 0;
    }else{
      success = false;
      errorMessages.string = "There must be at least one digit after the exponent.".split('');
    }
  }else{
    success = false;
    errorMessages.string = "Expected decimal point or exponent symbol.".split('');
  }

  return success;
}
function nExtractPartsFromNumberStringFromExponent(n, base, i, exponentIsPositive, exponent, errorMessages){
  var success, done;
  var count, j;

  if(base <= 14 && (n[i] == 'e' || n[i] == 'E')){
    i = i + 1;

    if(i < n.length){
      if(n[i] == '-'){
        exponentIsPositive.booleanValue = false;
        i = i + 1;
      }else if(n[i] == '+'){
        exponentIsPositive.booleanValue = true;
        i = i + 1;
      }

      if(i < n.length){
        done = false;
        count = 0;
        for(; i + count < n.length &&  !done ; ){
          if(nCharacterIsNumberCharacterInBase(n[i + count], base)){
            count = count + 1;
          }else{
            done = true;
          }
        }

        if(count >= 1){
          exponent.numberArray = [];
          exponent.numberArray.length = count;

          for(j = 0; j < count; j = j + 1){
            exponent.numberArray[j] = nGetNumberFromNumberCharacterForBase(n[i + j], base);
          }

          i = i + count;

          if(i == n.length){
            success = true;
          }else{
            success = false;
            errorMessages.string = "There cannot be any characters past the exponent of the number.".split('');
          }
        }else{
          success = false;
          errorMessages.string = "There must be at least one digit after the decimal point.".split('');
        }
      }else{
        success = false;
        errorMessages.string = "There must be at least one digit after the exponent symbol.".split('');
      }
    }else{
      success = false;
      errorMessages.string = "There must be at least one digit after the exponent symbol.".split('');
    }
  }else{
    success = false;
    errorMessages.string = "Expected exponent symbol.".split('');
  }

  return success;
}
function nGetNumberFromNumberCharacterForBase(c, base){
  var numberTable;
  var i;
  var position;

  numberTable = nGetDigitCharacterTable();
  position = 0;

  for(i = 0; i < base; i = i + 1){
    if(numberTable[i] == c){
      position = i;
    }
  }

  return position;
}
function nCharacterIsNumberCharacterInBase(c, base){
  var numberTable;
  var i;
  var found;

  numberTable = nGetDigitCharacterTable();
  found = false;

  for(i = 0; i < base; i = i + 1){
    if(numberTable[i] == c){
      found = true;
    }
  }

  return found;
}
function nStringToNumberArray(str){
  var numberArrayReference;
  var stringReference;
  var numbers;

  numberArrayReference = {};
  stringReference = {};

  nStringToNumberArrayWithCheck(str, numberArrayReference, stringReference);

  numbers = numberArrayReference.numberArray;

  delete(numberArrayReference);
  delete(stringReference);

  return numbers;
}
function nStringToNumberArrayWithCheck(str, numberArrayReference, errorMessage){
  var numberStrings;
  var numbers;
  var i;
  var numberString, trimmedNumberString;
  var success;
  var numberReference;

  numberStrings = SplitByString(str, ",".split(''));

  numbers = [];
  numbers.length = numberStrings.length;
  success = true;
  numberReference = {};

  for(i = 0; i < numberStrings.length; i = i + 1){
    numberString = numberStrings[i].string;
    trimmedNumberString = Trim(numberString);
    success = nCreateNumberFromDecimalStringWithCheck(trimmedNumberString, numberReference, errorMessage);
    numbers[i] = numberReference.numberValue;

    FreeStringReference(numberStrings[i]);
    delete(trimmedNumberString);
  }

  delete(numberStrings);
  delete(numberReference);

  numberArrayReference.numberArray = numbers;

  return success;
}
function AssertFalse(b, failures){
  if(b){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertTrue(b, failures){
  if( !b ){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertEquals(a, b, failures){
  if(a != b){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertBooleansEqual(a, b, failures){
  if(a != b){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertCharactersEqual(a, b, failures){
  if(a != b){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertStringEquals(a, b, failures){
  if( !StringsEqual(a, b) ){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertNumberArraysEqual(a, b, failures){
  var i;

  if(a.length == b.length){
    for(i = 0; i < a.length; i = i + 1){
      AssertEquals(a[i], b[i], failures);
    }
  }else{
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertBooleanArraysEqual(a, b, failures){
  var i;

  if(a.length == b.length){
    for(i = 0; i < a.length; i = i + 1){
      AssertBooleansEqual(a[i], b[i], failures);
    }
  }else{
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertStringArraysEqual(a, b, failures){
  var i;

  if(a.length == b.length){
    for(i = 0; i < a.length; i = i + 1){
      AssertStringEquals(a[i].string, b[i].string, failures);
    }
  }else{
    failures.numberValue = failures.numberValue + 1;
  }
}
function AddNumber(list, a){
  var newlist;
  var i;

  newlist = [];
  newlist.length = list.length + 1;
  for(i = 0; i < list.length; i = i + 1){
    newlist[i] = list[i];
  }
  newlist[list.length] = a;
		
  delete(list);
		
  return newlist;
}
function AddNumberRef(list, i){
  list.numberArray = AddNumber(list.numberArray, i);
}
function RemoveNumber(list, n){
  var newlist;
  var i;

  newlist = [];
  newlist.length = list.length - 1;

  if(n >= 0 && n < list.length){
    for(i = 0; i < list.length; i = i + 1){
      if(i < n){
        newlist[i] = list[i];
      }
      if(i > n){
        newlist[i - 1] = list[i];
      }
    }

    delete(list);
  }else{
    delete(newlist);
  }
		
  return newlist;
}
function GetNumberRef(list, i){
  return list.numberArray[i];
}
function RemoveNumberRef(list, i){
  list.numberArray = RemoveNumber(list.numberArray, i);
}
function AddString(list, a){
  var newlist;
  var i;

  newlist = [];
  newlist.length = list.length + 1;

  for(i = 0; i < list.length; i = i + 1){
    newlist[i] = list[i];
  }
  newlist[list.length] = a;
		
  delete(list);
		
  return newlist;
}
function AddStringRef(list, i){
  list.stringArray = AddString(list.stringArray, i);
}
function RemoveString(list, n){
  var newlist;
  var i;

  newlist = [];
  newlist.length = list.length - 1;

  if(n >= 0 && n < list.length){
    for(i = 0; i < list.length; i = i + 1){
      if(i < n){
        newlist[i] = list[i];
      }
      if(i > n){
        newlist[i - 1] = list[i];
      }
    }

    delete(list);
  }else{
    delete(newlist);
  }
		
  return newlist;
}
function GetStringRef(list, i){
  return list.stringArray[i];
}
function RemoveStringRef(list, i){
  list.stringArray = RemoveString(list.stringArray, i);
}
function AddBoolean(list, a){
  var newlist;
  var i;

  newlist = [];
  newlist.length = list.length + 1;
  for(i = 0; i < list.length; i = i + 1){
    newlist[i] = list[i];
  }
  newlist[list.length] = a;
		
  delete(list);
		
  return newlist;
}
function AddBooleanRef(list, i){
  list.booleanArray = AddBoolean(list.booleanArray, i);
}
function RemoveBoolean(list, n){
  var newlist;
  var i;

  newlist = [];
  newlist.length = list.length - 1;

  if(n >= 0 && n < list.length){
    for(i = 0; i < list.length; i = i + 1){
      if(i < n){
        newlist[i] = list[i];
      }
      if(i > n){
        newlist[i - 1] = list[i];
      }
    }

    delete(list);
  }else{
    delete(newlist);
  }
		
  return newlist;
}
function GetBooleanRef(list, i){
  return list.booleanArray[i];
}
function RemoveDecimalRef(list, i){
  list.booleanArray = RemoveBoolean(list.booleanArray, i);
}
function CreateLinkedListString(){
  var ll;

  ll = {};
  ll.first = {};
  ll.last = ll.first;
  ll.last.end = true;

  return ll;
}
function LinkedListAddString(ll, value){
  ll.last.end = false;
  ll.last.value = value;
  ll.last.next = {};
  ll.last.next.end = true;
  ll.last = ll.last.next;
}
function LinkedListStringsToArray(ll){
  var array;
  var lengthx, i;
  var node;

  node = ll.first;

  lengthx = LinkedListStringsLength(ll);

  array = [];
  array.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    array[i] = {};
    array[i].string = node.value;
    node = node.next;
  }

  return array;
}
function LinkedListStringsLength(ll){
  var l;
  var node;

  l = 0;
  node = ll.first;
  for(;  !node.end ; ){
    node = node.next;
    l = l + 1;
  }

  return l;
}
function FreeLinkedListString(ll){
  var node, prev;

  node = ll.first;

  for(;  !node.end ; ){
    prev = node;
    node = node.next;
    delete(prev);
  }

  delete(node);
}
function CreateLinkedListNumbers(){
  var ll;

  ll = {};
  ll.first = {};
  ll.last = ll.first;
  ll.last.end = true;

  return ll;
}
function CreateLinkedListNumbersArray(lengthx){
  var lls;
  var i;

  lls = [];
  lls.length = lengthx;
  for(i = 0; i < lls.length; i = i + 1){
    lls[i] = CreateLinkedListNumbers();
  }

  return lls;
}
function LinkedListAddNumber(ll, value){
  ll.last.end = false;
  ll.last.value = value;
  ll.last.next = {};
  ll.last.next.end = true;
  ll.last = ll.last.next;
}
function LinkedListNumbersLength(ll){
  var l;
  var node;

  l = 0;
  node = ll.first;
  for(;  !node.end ; ){
    node = node.next;
    l = l + 1;
  }

  return l;
}
function LinkedListNumbersIndex(ll, index){
  var i;
  var node;

  node = ll.first;
  for(i = 0; i < index; i = i + 1){
    node = node.next;
  }

  return node.value;
}
function LinkedListInsertNumber(ll, index, value){
  var i;
  var node, tmp;

  if(index == 0){
    tmp = ll.first;
    ll.first = {};
    ll.first.next = tmp;
    ll.first.value = value;
    ll.first.end = false;
  }else{
    node = ll.first;
    for(i = 0; i < index - 1; i = i + 1){
      node = node.next;
    }

    tmp = node.next;
    node.next = {};
    node.next.next = tmp;
    node.next.value = value;
    node.next.end = false;
  }
}
function LinkedListSet(ll, index, value){
  var i;
  var node;

  node = ll.first;
  for(i = 0; i < index; i = i + 1){
    node = node.next;
  }

  node.next.value = value;
}
function LinkedListRemoveNumber(ll, index){
  var i;
  var node, prev;

  node = ll.first;
  prev = ll.first;

  for(i = 0; i < index; i = i + 1){
    prev = node;
    node = node.next;
  }

  if(index == 0){
    ll.first = prev.next;
  }
  if( !prev.next.end ){
    prev.next = prev.next.next;
  }
}
function FreeLinkedListNumbers(ll){
  var node, prev;

  node = ll.first;

  for(;  !node.end ; ){
    prev = node;
    node = node.next;
    delete(prev);
  }

  delete(node);
}
function FreeLinkedListNumbersArray(lls){
  var i;

  for(i = 0; i < lls.length; i = i + 1){
    FreeLinkedListNumbers(lls[i]);
  }
  delete(lls);
}
function LinkedListNumbersToArray(ll){
  var array;
  var lengthx, i;
  var node;

  node = ll.first;

  lengthx = LinkedListNumbersLength(ll);

  array = [];
  array.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    array[i] = node.value;
    node = node.next;
  }

  return array;
}
function ArrayToLinkedListNumbers(array){
  var ll;
  var i;

  ll = CreateLinkedListNumbers();

  for(i = 0; i < array.length; i = i + 1){
    LinkedListAddNumber(ll, array[i]);
  }

  return ll;
}
function LinkedListNumbersEqual(a, b){
  var equal, done;
  var an, bn;

  an = a.first;
  bn = b.first;

  equal = true;
  done = false;
  for(; equal &&  !done ; ){
    if(an.end == bn.end){
      if(an.end){
        done = true;
      }else if(an.value == bn.value){
        an = an.next;
        bn = bn.next;
      }else{
        equal = false;
      }
    }else{
      equal = false;
    }
  }

  return equal;
}
function CreateLinkedListCharacter(){
  var ll;

  ll = {};
  ll.first = {};
  ll.last = ll.first;
  ll.last.end = true;

  return ll;
}
function LinkedListAddCharacter(ll, value){
  ll.last.end = false;
  ll.last.value = value;
  ll.last.next = {};
  ll.last.next.end = true;
  ll.last = ll.last.next;
}
function LinkedListCharactersToArray(ll){
  var array;
  var lengthx, i;
  var node;

  node = ll.first;

  lengthx = LinkedListCharactersLength(ll);

  array = [];
  array.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    array[i] = node.value;
    node = node.next;
  }

  return array;
}
function LinkedListCharactersLength(ll){
  var l;
  var node;

  l = 0;
  node = ll.first;
  for(;  !node.end ; ){
    node = node.next;
    l = l + 1;
  }

  return l;
}
function FreeLinkedListCharacter(ll){
  var node, prev;

  node = ll.first;

  for(;  !node.end ; ){
    prev = node;
    node = node.next;
    delete(prev);
  }

  delete(node);
}
function CreateDynamicArrayNumbers(){
  var da;

  da = {};
  da.array = [];
  da.array.length = 10;
  da.lengthx = 0;

  return da;
}
function CreateDynamicArrayNumbersWithInitialCapacity(capacity){
  var da;

  da = {};
  da.array = [];
  da.array.length = capacity;
  da.lengthx = 0;

  return da;
}
function DynamicArrayAddNumber(da, value){
  if(da.lengthx == da.array.length){
    DynamicArrayNumbersIncreaseSize(da);
  }

  da.array[da.lengthx] = value;
  da.lengthx = da.lengthx + 1;
}
function DynamicArrayNumbersIncreaseSize(da){
  var newLength, i;
  var newArray;

  newLength = Math.round(da.array.length*3/2);
  newArray = [];
  newArray.length = newLength;

  for(i = 0; i < da.array.length; i = i + 1){
    newArray[i] = da.array[i];
  }

  delete(da.array);

  da.array = newArray;
}
function DynamicArrayNumbersDecreaseSizeNecessary(da){
  var needsDecrease;

  needsDecrease = false;

  if(da.lengthx > 10){
    needsDecrease = da.lengthx <= Math.round(da.array.length*2/3);
  }

  return needsDecrease;
}
function DynamicArrayNumbersDecreaseSize(da){
  var newLength, i;
  var newArray;

  newLength = Math.round(da.array.length*2/3);
  newArray = [];
  newArray.length = newLength;

  for(i = 0; i < da.array.length; i = i + 1){
    newArray[i] = da.array[i];
  }

  delete(da.array);

  da.array = newArray;
}
function DynamicArrayNumbersIndex(da, index){
  return da.array[index];
}
function DynamicArrayNumbersLength(da){
  return da.lengthx;
}
function DynamicArrayInsertNumber(da, index, value){
  var i;

  if(da.lengthx == da.array.length){
    DynamicArrayNumbersIncreaseSize(da);
  }

  for(i = da.lengthx; i > index; i = i - 1){
    da.array[i] = da.array[i - 1];
  }

  da.array[index] = value;

  da.lengthx = da.lengthx + 1;
}
function DynamicArraySet(da, index, value){
  da.array[index] = value;
}
function DynamicArrayRemoveNumber(da, index){
  var i;

  for(i = index; i < da.lengthx - 1; i = i + 1){
    da.array[i] = da.array[i + 1];
  }

  da.lengthx = da.lengthx - 1;

  if(DynamicArrayNumbersDecreaseSizeNecessary(da)){
    DynamicArrayNumbersDecreaseSize(da);
  }
}
function FreeDynamicArrayNumbers(da){
  delete(da.array);
  delete(da);
}
function DynamicArrayNumbersToArray(da){
  var array;
  var i;

  array = [];
  array.length = da.lengthx;

  for(i = 0; i < da.lengthx; i = i + 1){
    array[i] = da.array[i];
  }

  return array;
}
function ArrayToDynamicArrayNumbersWithOptimalSize(array){
  var da;
  var i;
  var c, n, newCapacity;

  /*
         c = 10*(3/2)^n
         log(c) = log(10*(3/2)^n)
         log(c) = log(10) + log((3/2)^n)
         log(c) = 1 + log((3/2)^n)
         log(c) - 1 = log((3/2)^n)
         log(c) - 1 = n*log(3/2)
         n = (log(c) - 1)/log(3/2)
         */
  c = array.length;
  n = (Math.log(c) - 1)/Math.log(3/2);
  newCapacity = Math.floor(n) + 1;

  da = CreateDynamicArrayNumbersWithInitialCapacity(newCapacity);

  for(i = 0; i < array.length; i = i + 1){
    da.array[i] = array[i];
  }

  return da;
}
function ArrayToDynamicArrayNumbers(array){
  var da;

  da = {};
  da.array = CopyNumberArray(array);
  da.lengthx = array.length;

  return da;
}
function DynamicArrayNumbersEqual(a, b){
  var equal;
  var i;

  equal = true;
  if(a.lengthx == b.lengthx){
    for(i = 0; i < a.lengthx && equal; i = i + 1){
      if(a.array[i] != b.array[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function DynamicArrayNumbersToLinkedList(da){
  var ll;
  var i;

  ll = CreateLinkedListNumbers();

  for(i = 0; i < da.lengthx; i = i + 1){
    LinkedListAddNumber(ll, da.array[i]);
  }

  return ll;
}
function LinkedListToDynamicArrayNumbers(ll){
  var da;
  var i;
  var node;

  node = ll.first;

  da = {};
  da.lengthx = LinkedListNumbersLength(ll);

  da.array = [];
  da.array.length = da.lengthx;

  for(i = 0; i < da.lengthx; i = i + 1){
    da.array[i] = node.value;
    node = node.next;
  }

  return da;
}
function AddCharacter(list, a){
  var newlist;
  var i;

  newlist = [];
  newlist.length = list.length + 1;
  for(i = 0; i < list.length; i = i + 1){
    newlist[i] = list[i];
  }
  newlist[list.length] = a;
		
  delete(list);
		
  return newlist;
}
function AddCharacterRef(list, i){
  list.string = AddCharacter(list.string, i);
}
function RemoveCharacter(list, n){
  var newlist;
  var i;

  newlist = [];
  newlist.length = list.length - 1;

  if(n >= 0 && n < list.length){
    for(i = 0; i < list.length; i = i + 1){
      if(i < n){
        newlist[i] = list[i];
      }
      if(i > n){
        newlist[i - 1] = list[i];
      }
    }

    delete(list);
  }else{
    delete(newlist);
  }

  return newlist;
}
function GetCharacterRef(list, i){
  return list.string[i];
}
function RemoveCharacterRef(list, i){
  list.string = RemoveCharacter(list.string, i);
}
function charToLowerCase(character){
  var toReturn;

  toReturn = character;
  if(character == 'A'){
    toReturn = 'a';
  }else if(character == 'B'){
    toReturn = 'b';
  }else if(character == 'C'){
    toReturn = 'c';
  }else if(character == 'D'){
    toReturn = 'd';
  }else if(character == 'E'){
    toReturn = 'e';
  }else if(character == 'F'){
    toReturn = 'f';
  }else if(character == 'G'){
    toReturn = 'g';
  }else if(character == 'H'){
    toReturn = 'h';
  }else if(character == 'I'){
    toReturn = 'i';
  }else if(character == 'J'){
    toReturn = 'j';
  }else if(character == 'K'){
    toReturn = 'k';
  }else if(character == 'L'){
    toReturn = 'l';
  }else if(character == 'M'){
    toReturn = 'm';
  }else if(character == 'N'){
    toReturn = 'n';
  }else if(character == 'O'){
    toReturn = 'o';
  }else if(character == 'P'){
    toReturn = 'p';
  }else if(character == 'Q'){
    toReturn = 'q';
  }else if(character == 'R'){
    toReturn = 'r';
  }else if(character == 'S'){
    toReturn = 's';
  }else if(character == 'T'){
    toReturn = 't';
  }else if(character == 'U'){
    toReturn = 'u';
  }else if(character == 'V'){
    toReturn = 'v';
  }else if(character == 'W'){
    toReturn = 'w';
  }else if(character == 'X'){
    toReturn = 'x';
  }else if(character == 'Y'){
    toReturn = 'y';
  }else if(character == 'Z'){
    toReturn = 'z';
  }

  return toReturn;
}
function charToUpperCase(character){
  var toReturn;

  toReturn = character;
  if(character == 'a'){
    toReturn = 'A';
  }else if(character == 'b'){
    toReturn = 'B';
  }else if(character == 'c'){
    toReturn = 'C';
  }else if(character == 'd'){
    toReturn = 'D';
  }else if(character == 'e'){
    toReturn = 'E';
  }else if(character == 'f'){
    toReturn = 'F';
  }else if(character == 'g'){
    toReturn = 'G';
  }else if(character == 'h'){
    toReturn = 'H';
  }else if(character == 'i'){
    toReturn = 'I';
  }else if(character == 'j'){
    toReturn = 'J';
  }else if(character == 'k'){
    toReturn = 'K';
  }else if(character == 'l'){
    toReturn = 'L';
  }else if(character == 'm'){
    toReturn = 'M';
  }else if(character == 'n'){
    toReturn = 'N';
  }else if(character == 'o'){
    toReturn = 'O';
  }else if(character == 'p'){
    toReturn = 'P';
  }else if(character == 'q'){
    toReturn = 'Q';
  }else if(character == 'r'){
    toReturn = 'R';
  }else if(character == 's'){
    toReturn = 'S';
  }else if(character == 't'){
    toReturn = 'T';
  }else if(character == 'u'){
    toReturn = 'U';
  }else if(character == 'v'){
    toReturn = 'V';
  }else if(character == 'w'){
    toReturn = 'W';
  }else if(character == 'x'){
    toReturn = 'X';
  }else if(character == 'y'){
    toReturn = 'Y';
  }else if(character == 'z'){
    toReturn = 'Z';
  }

  return toReturn;
}
function charIsUpperCase(character){
  var isUpper;

  isUpper = false;
  if(character == 'A'){
    isUpper = true;
  }else if(character == 'B'){
    isUpper = true;
  }else if(character == 'C'){
    isUpper = true;
  }else if(character == 'D'){
    isUpper = true;
  }else if(character == 'E'){
    isUpper = true;
  }else if(character == 'F'){
    isUpper = true;
  }else if(character == 'G'){
    isUpper = true;
  }else if(character == 'H'){
    isUpper = true;
  }else if(character == 'I'){
    isUpper = true;
  }else if(character == 'J'){
    isUpper = true;
  }else if(character == 'K'){
    isUpper = true;
  }else if(character == 'L'){
    isUpper = true;
  }else if(character == 'M'){
    isUpper = true;
  }else if(character == 'N'){
    isUpper = true;
  }else if(character == 'O'){
    isUpper = true;
  }else if(character == 'P'){
    isUpper = true;
  }else if(character == 'Q'){
    isUpper = true;
  }else if(character == 'R'){
    isUpper = true;
  }else if(character == 'S'){
    isUpper = true;
  }else if(character == 'T'){
    isUpper = true;
  }else if(character == 'U'){
    isUpper = true;
  }else if(character == 'V'){
    isUpper = true;
  }else if(character == 'W'){
    isUpper = true;
  }else if(character == 'X'){
    isUpper = true;
  }else if(character == 'Y'){
    isUpper = true;
  }else if(character == 'Z'){
    isUpper = true;
  }

  return isUpper;
}
function charIsLowerCase(character){
  var isLower;

  isLower = false;
  if(character == 'a'){
    isLower = true;
  }else if(character == 'b'){
    isLower = true;
  }else if(character == 'c'){
    isLower = true;
  }else if(character == 'd'){
    isLower = true;
  }else if(character == 'e'){
    isLower = true;
  }else if(character == 'f'){
    isLower = true;
  }else if(character == 'g'){
    isLower = true;
  }else if(character == 'h'){
    isLower = true;
  }else if(character == 'i'){
    isLower = true;
  }else if(character == 'j'){
    isLower = true;
  }else if(character == 'k'){
    isLower = true;
  }else if(character == 'l'){
    isLower = true;
  }else if(character == 'm'){
    isLower = true;
  }else if(character == 'n'){
    isLower = true;
  }else if(character == 'o'){
    isLower = true;
  }else if(character == 'p'){
    isLower = true;
  }else if(character == 'q'){
    isLower = true;
  }else if(character == 'r'){
    isLower = true;
  }else if(character == 's'){
    isLower = true;
  }else if(character == 't'){
    isLower = true;
  }else if(character == 'u'){
    isLower = true;
  }else if(character == 'v'){
    isLower = true;
  }else if(character == 'w'){
    isLower = true;
  }else if(character == 'x'){
    isLower = true;
  }else if(character == 'y'){
    isLower = true;
  }else if(character == 'z'){
    isLower = true;
  }

  return isLower;
}
function charIsLetter(character){
  return charIsUpperCase(character) || charIsLowerCase(character);
}
function charIsNumber(character){
  var isNumberx;

  isNumberx = false;
  if(character == '0'){
    isNumberx = true;
  }else if(character == '1'){
    isNumberx = true;
  }else if(character == '2'){
    isNumberx = true;
  }else if(character == '3'){
    isNumberx = true;
  }else if(character == '4'){
    isNumberx = true;
  }else if(character == '5'){
    isNumberx = true;
  }else if(character == '6'){
    isNumberx = true;
  }else if(character == '7'){
    isNumberx = true;
  }else if(character == '8'){
    isNumberx = true;
  }else if(character == '9'){
    isNumberx = true;
  }

  return isNumberx;
}
function charIsWhiteSpace(character){
  var isWhiteSpacex;

  isWhiteSpacex = false;
  if(character == ' '){
    isWhiteSpacex = true;
  }else if(character == '\t'){
    isWhiteSpacex = true;
  }else if(character == '\n'){
    isWhiteSpacex = true;
  }else if(character == '\r'){
    isWhiteSpacex = true;
  }

  return isWhiteSpacex;
}
function charIsSymbol(character){
  var isSymbolx;

  isSymbolx = false;
  if(character == '!'){
    isSymbolx = true;
  }else if(character == '\"'){
    isSymbolx = true;
  }else if(character == '#'){
    isSymbolx = true;
  }else if(character == '$'){
    isSymbolx = true;
  }else if(character == '%'){
    isSymbolx = true;
  }else if(character == '&'){
    isSymbolx = true;
  }else if(character == '\''){
    isSymbolx = true;
  }else if(character == '('){
    isSymbolx = true;
  }else if(character == ')'){
    isSymbolx = true;
  }else if(character == '*'){
    isSymbolx = true;
  }else if(character == '+'){
    isSymbolx = true;
  }else if(character == ','){
    isSymbolx = true;
  }else if(character == '-'){
    isSymbolx = true;
  }else if(character == '.'){
    isSymbolx = true;
  }else if(character == '/'){
    isSymbolx = true;
  }else if(character == ':'){
    isSymbolx = true;
  }else if(character == ';'){
    isSymbolx = true;
  }else if(character == '<'){
    isSymbolx = true;
  }else if(character == '='){
    isSymbolx = true;
  }else if(character == '>'){
    isSymbolx = true;
  }else if(character == '?'){
    isSymbolx = true;
  }else if(character == '@'){
    isSymbolx = true;
  }else if(character == '['){
    isSymbolx = true;
  }else if(character == '\\'){
    isSymbolx = true;
  }else if(character == ']'){
    isSymbolx = true;
  }else if(character == '^'){
    isSymbolx = true;
  }else if(character == '_'){
    isSymbolx = true;
  }else if(character == '`'){
    isSymbolx = true;
  }else if(character == '{'){
    isSymbolx = true;
  }else if(character == '|'){
    isSymbolx = true;
  }else if(character == '}'){
    isSymbolx = true;
  }else if(character == '~'){
    isSymbolx = true;
  }

  return isSymbolx;
}
function charCharacterIsBefore(a, b){
  var ad, bd;

  ad = a.charCodeAt(0);
  bd = b.charCodeAt(0);

  return ad < bd;
}
function StringToNumberArray(string){
  var i;
  var array;

  array = [];
  array.length = string.length;

  for(i = 0; i < string.length; i = i + 1){
    array[i] = string[i].charCodeAt(0);
  }
  return array;
}
function NumberArrayToString(array){
  var i;
  var string;

  string = [];
  string.length = array.length;

  for(i = 0; i < array.length; i = i + 1){
    string[i] = String.fromCharCode(array[i]);
  }
  return string;
}
function NumberArraysEqual(a, b){
  var equal;
  var i;

  equal = true;
  if(a.length == b.length){
    for(i = 0; i < a.length && equal; i = i + 1){
      if(a[i] != b[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function BooleanArraysEqual(a, b){
  var equal;
  var i;

  equal = true;
  if(a.length == b.length){
    for(i = 0; i < a.length && equal; i = i + 1){
      if(a[i] != b[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function StringsEqual(a, b){
  var equal;
  var i;

  equal = true;
  if(a.length == b.length){
    for(i = 0; i < a.length && equal; i = i + 1){
      if(a[i] != b[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function FillNumberArray(a, value){
  var i;

  for(i = 0; i < a.length; i = i + 1){
    a[i] = value;
  }
}
function FillString(a, value){
  var i;

  for(i = 0; i < a.length; i = i + 1){
    a[i] = value;
  }
}
function FillBooleanArray(a, value){
  var i;

  for(i = 0; i < a.length; i = i + 1){
    a[i] = value;
  }
}
function FillNumberArrayRange(a, value, from, to){
  var i, lengthx;
  var success;

  if(from >= 0 && from <= a.length && to >= 0 && to <= a.length && from <= to){
    lengthx = to - from;
    for(i = 0; i < lengthx; i = i + 1){
      a[from + i] = value;
    }

    success = true;
  }else{
    success = false;
  }

  return success;
}
function FillBooleanArrayRange(a, value, from, to){
  var i, lengthx;
  var success;

  if(from >= 0 && from <= a.length && to >= 0 && to <= a.length && from <= to){
    lengthx = to - from;
    for(i = 0; i < lengthx; i = i + 1){
      a[from + i] = value;
    }

    success = true;
  }else{
    success = false;
  }

  return success;
}
function FillStringRange(a, value, from, to){
  var i, lengthx;
  var success;

  if(from >= 0 && from <= a.length && to >= 0 && to <= a.length && from <= to){
    lengthx = to - from;
    for(i = 0; i < lengthx; i = i + 1){
      a[from + i] = value;
    }

    success = true;
  }else{
    success = false;
  }

  return success;
}
function CopyNumberArray(a){
  var i;
  var n;

  n = [];
  n.length = a.length;

  for(i = 0; i < a.length; i = i + 1){
    n[i] = a[i];
  }

  return n;
}
function CopyBooleanArray(a){
  var i;
  var n;

  n = [];
  n.length = a.length;

  for(i = 0; i < a.length; i = i + 1){
    n[i] = a[i];
  }

  return n;
}
function CopyString(a){
  var i;
  var n;

  n = [];
  n.length = a.length;

  for(i = 0; i < a.length; i = i + 1){
    n[i] = a[i];
  }

  return n;
}
function CopyNumberArrayRange(a, from, to, copyReference){
  var i, lengthx;
  var n;
  var success;

  if(from >= 0 && from <= a.length && to >= 0 && to <= a.length && from <= to){
    lengthx = to - from;
    n = [];
    n.length = lengthx;

    for(i = 0; i < lengthx; i = i + 1){
      n[i] = a[from + i];
    }

    copyReference.numberArray = n;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function CopyBooleanArrayRange(a, from, to, copyReference){
  var i, lengthx;
  var n;
  var success;

  if(from >= 0 && from <= a.length && to >= 0 && to <= a.length && from <= to){
    lengthx = to - from;
    n = [];
    n.length = lengthx;

    for(i = 0; i < lengthx; i = i + 1){
      n[i] = a[from + i];
    }

    copyReference.booleanArray = n;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function CopyStringRange(a, from, to, copyReference){
  var i, lengthx;
  var n;
  var success;

  if(from >= 0 && from <= a.length && to >= 0 && to <= a.length && from <= to){
    lengthx = to - from;
    n = [];
    n.length = lengthx;

    for(i = 0; i < lengthx; i = i + 1){
      n[i] = a[from + i];
    }

    copyReference.string = n;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function IsLastElement(lengthx, index){
  return index + 1 == lengthx;
}
function CreateNumberArray(lengthx, value){
  var array;

  array = [];
  array.length = lengthx;
  FillNumberArray(array, value);

  return array;
}
function CreateBooleanArray(lengthx, value){
  var array;

  array = [];
  array.length = lengthx;
  FillBooleanArray(array, value);

  return array;
}
function CreateString(lengthx, value){
  var array;

  array = [];
  array.length = lengthx;
  FillString(array, value);

  return array;
}
function SwapElementsOfNumberArray(A, ai, bi){
  var tmp;

  tmp = A[ai];
  A[ai] = A[bi];
  A[bi] = tmp;
}
function SwapElementsOfStringArray(A, ai, bi){
  var tmp;

  tmp = A.stringArray[ai];
  A.stringArray[ai] = A.stringArray[bi];
  A.stringArray[bi] = tmp;
}
function ReverseNumberArray(array){
  var i;

  for(i = 0; i < array.length/2; i = i + 1){
    SwapElementsOfNumberArray(array, i, array.length - i - 1);
  }
}


module.exports = {
	Add: Add,
	Assign: Assign,
	Resize: Resize,
	Subtract: Subtract,
	SubtractToNew: SubtractToNew,
	ScalarMultiply: ScalarMultiply,
	ScalarDivide: ScalarDivide,
	ElementWisePower: ElementWisePower,
	ScalarMultiplyToNew: ScalarMultiplyToNew,
	MultiplyToNew: MultiplyToNew,
	Multiply: Multiply,
	CreateSquareMatrix: CreateSquareMatrix,
	CreateMatrix: CreateMatrix,
	CreateIdentityMatrix: CreateIdentityMatrix,
	Transpose: Transpose,
	TransposeAssign: TransposeAssign,
	TransposeToNew: TransposeToNew,
	CofactorOfMatrix: CofactorOfMatrix,
	DeterminantOfSubmatrix: DeterminantOfSubmatrix,
	Determinant: Determinant,
	Adjoint: Adjoint,
	Inverse: Inverse,
	InverseUsingAdjoint: InverseUsingAdjoint,
	InverseUsingLUDecomposition: InverseUsingLUDecomposition,
	LUDecomposition: LUDecomposition,
	IsSymmetric: IsSymmetric,
	IsSquare: IsSquare,
	Cholesky: Cholesky,
	Clear: Clear,
	Fill: Fill,
	Element: Element,
	Trace: Trace,
	ColumnCombineMatricesToNew: ColumnCombineMatricesToNew,
	NumberOfRows: NumberOfRows,
	NumberOfColumns: NumberOfColumns,
	CharacteristicPolynomial: CharacteristicPolynomial,
	CharacteristicPolynomialWithInverse: CharacteristicPolynomialWithInverse,
	FaddeevLeVerrierAlgorithm: FaddeevLeVerrierAlgorithm,
	InverseUsingCharacteristicPolynomial: InverseUsingCharacteristicPolynomial,
	Eigenvalues: Eigenvalues,
	EigenvaluesUsingQRAlgorithm: EigenvaluesUsingQRAlgorithm,
	EigenvaluesUsingLaguerreIterations: EigenvaluesUsingLaguerreIterations,
	GaussianElimination: GaussianElimination,
	GaussianEliminationToNew: GaussianEliminationToNew,
	CreateCopyOfMatrix: CreateCopyOfMatrix,
	SwapRows: SwapRows,
	UnnormalizeVector: UnnormalizeVector,
	InversePowerMethod: InversePowerMethod,
	Eigenvectors: Eigenvectors,
	Eigenpairs: Eigenpairs,
	EigenpairsUsingQRAlgorithmAndInversePowerMethod: EigenpairsUsingQRAlgorithmAndInversePowerMethod,
	CheckEigenpairPrecision: CheckEigenpairPrecision,
	EigenvectorsLaguerreIterationsAndGaussianEliminations: EigenvectorsLaguerreIterationsAndGaussianEliminations,
	RowIsZero: RowIsZero,
	FreeMatrix: FreeMatrix,
	FreeMatrixRows: FreeMatrixRows,
	CreateDiagonalMatrixFromArray: CreateDiagonalMatrixFromArray,
	CreateMatrixFromRowCopies: CreateMatrixFromRowCopies,
	ExtractDiagonal: ExtractDiagonal,
	ExtractDiagonalToNew: ExtractDiagonalToNew,
	MatrixEqualsEpsilon: MatrixEqualsEpsilon,
	Minor: Minor,
	QRDecomposition: QRDecomposition,
	HouseholderTriangularizationAlgorithm: HouseholderTriangularizationAlgorithm,
	HouseholderMethod: HouseholderMethod,
	Hypothenuse: Hypothenuse,
	Norm: Norm,
	ExtractSubMatrix: ExtractSubMatrix,
	QRAlgorithm: QRAlgorithm,
	InvertUpperTriangularMatrix: InvertUpperTriangularMatrix,
	InvertLowerTriangularMatrix: InvertLowerTriangularMatrix,
	ParseMatrixFromString: ParseMatrixFromString,
	FreeStringReferenceArray: FreeStringReferenceArray,
	MatrixToString: MatrixToString,
	RoundToDigits: RoundToDigits,
	MatrixArrayToString: MatrixArrayToString,
	RoundMatrixElementsToDigits: RoundMatrixElementsToDigits,
	SingularValueDecomposition: SingularValueDecomposition,
	CreateComplexMatrix: CreateComplexMatrix,
	CreateComplexMatrixFromMatrix: CreateComplexMatrixFromMatrix,
	CreateReMatrixFromComplexMatrix: CreateReMatrixFromComplexMatrix,
	CreateImMatrixFromComplexMatrix: CreateImMatrixFromComplexMatrix,
	NumberOfRowsComplex: NumberOfRowsComplex,
	NumberOfColumnsComplex: NumberOfColumnsComplex,
	IndexComplex: IndexComplex,
	AddComplex: AddComplex,
	SubtractComplex: SubtractComplex,
	SubtractComplexToNew: SubtractComplexToNew,
	MultiplyComplex: MultiplyComplex,
	MultiplyComplexToNew: MultiplyComplexToNew,
	Conjugate: Conjugate,
	AssignComplexMatrix: AssignComplexMatrix,
	ScalarMultiplyComplex: ScalarMultiplyComplex,
	ScalarMultiplyComplexToNew: ScalarMultiplyComplexToNew,
	ScalarDivideComplex: ScalarDivideComplex,
	ElementWisePowerComplex: ElementWisePowerComplex,
	CreateComplexIdentityMatrix: CreateComplexIdentityMatrix,
	CreateSquareComplexMatrix: CreateSquareComplexMatrix,
	ClearComplex: ClearComplex,
	FillComplex: FillComplex,
	TraceComplex: TraceComplex,
	CofactorOfComplexMatrix: CofactorOfComplexMatrix,
	DeterminantOfComplexSubmatrix: DeterminantOfComplexSubmatrix,
	DeleteComplexMatrix: DeleteComplexMatrix,
	DeterminantComplex: DeterminantComplex,
	AdjointComplex: AdjointComplex,
	InverseComplex: InverseComplex,
	ComplexMatrixEqualsEpsilon: ComplexMatrixEqualsEpsilon,
	MinorComplex: MinorComplex,
	AssignComplex: AssignComplex,
	CreateCopyOfComplexMatrix: CreateCopyOfComplexMatrix,
	TransposeComplex: TransposeComplex,
	ConjugateTransposeComplex: ConjugateTransposeComplex,
	IsSquareComplexMatrix: IsSquareComplexMatrix,
	TransposeComplexAssign: TransposeComplexAssign,
	TransposeComplexToNew: TransposeComplexToNew,
	ExtractComplexSubMatrix: ExtractComplexSubMatrix,
	NormComplex: NormComplex,
	ComplexCharacteristicPolynomial: ComplexCharacteristicPolynomial,
	ComplexCharacteristicPolynomialWithInverse: ComplexCharacteristicPolynomialWithInverse,
	FaddeevLeVerrierAlgorithmComplex: FaddeevLeVerrierAlgorithmComplex,
	EigenvaluesComplex: EigenvaluesComplex,
	EigenvectorsComplex: EigenvectorsComplex,
	InversePowerMethodComplex: InversePowerMethodComplex,
	EigenpairsComplex: EigenpairsComplex,
	ComplexEigenpairsUsingDurandKernerAndInversePowerMethod: ComplexEigenpairsUsingDurandKernerAndInversePowerMethod,
	CheckComplexEigenpairPrecision: CheckComplexEigenpairPrecision,
	FindRoots: FindRoots,
	LaguerresMethodWithRepeatedDivision: LaguerresMethodWithRepeatedDivision,
	LaguerresMethod: LaguerresMethod,
	DurandKernerMethod: DurandKernerMethod,
	FindRootsComplex: FindRootsComplex,
	DurandKernerMethodComplex: DurandKernerMethodComplex,
	CreateBooleanReference: CreateBooleanReference,
	CreateBooleanArrayReference: CreateBooleanArrayReference,
	CreateBooleanArrayReferenceLengthValue: CreateBooleanArrayReferenceLengthValue,
	FreeBooleanArrayReference: FreeBooleanArrayReference,
	CreateCharacterReference: CreateCharacterReference,
	CreateNumberReference: CreateNumberReference,
	CreateNumberArrayReference: CreateNumberArrayReference,
	CreateNumberArrayReferenceLengthValue: CreateNumberArrayReferenceLengthValue,
	FreeNumberArrayReference: FreeNumberArrayReference,
	CreateStringReference: CreateStringReference,
	CreateStringReferenceLengthValue: CreateStringReferenceLengthValue,
	FreeStringReference: FreeStringReference,
	CreateStringArrayReference: CreateStringArrayReference,
	CreateStringArrayReferenceLengthValue: CreateStringArrayReferenceLengthValue,
	FreeStringArrayReference: FreeStringArrayReference,
	mathNegate: mathNegate,
	mathPositive: mathPositive,
	mathFactorial: mathFactorial,
	mathRound: mathRound,
	mathBankersRound: mathBankersRound,
	mathCeil: mathCeil,
	mathFloor: mathFloor,
	mathTruncate: mathTruncate,
	mathAbsolute: mathAbsolute,
	mathLogarithm: mathLogarithm,
	mathNaturalLogarithm: mathNaturalLogarithm,
	mathSin: mathSin,
	mathCos: mathCos,
	mathTan: mathTan,
	mathAsin: mathAsin,
	mathAcos: mathAcos,
	mathAtan: mathAtan,
	mathAtan2: mathAtan2,
	mathSquareroot: mathSquareroot,
	mathExp: mathExp,
	mathDivisibleBy: mathDivisibleBy,
	mathCombinations: mathCombinations,
	mathPermutations: mathPermutations,
	mathEpsilonCompare: mathEpsilonCompare,
	mathGreatestCommonDivisor: mathGreatestCommonDivisor,
	mathGCDWithSubtraction: mathGCDWithSubtraction,
	mathIsInteger: mathIsInteger,
	mathGreatestCommonDivisorWithCheck: mathGreatestCommonDivisorWithCheck,
	mathLeastCommonMultiple: mathLeastCommonMultiple,
	mathSign: mathSign,
	mathMax: mathMax,
	mathMin: mathMin,
	mathPower: mathPower,
	mathGamma: mathGamma,
	mathLogGamma: mathLogGamma,
	mathLanczosApproximation: mathLanczosApproximation,
	mathBeta: mathBeta,
	mathSinh: mathSinh,
	mathCosh: mathCosh,
	mathTanh: mathTanh,
	mathCot: mathCot,
	mathSec: mathSec,
	mathCsc: mathCsc,
	mathCoth: mathCoth,
	mathSech: mathSech,
	mathCsch: mathCsch,
	mathError: mathError,
	mathErrorInverse: mathErrorInverse,
	mathFallingFactorial: mathFallingFactorial,
	mathRisingFactorial: mathRisingFactorial,
	mathHypergeometric: mathHypergeometric,
	mathHypergeometricDirect: mathHypergeometricDirect,
	mathBernouilliNumber: mathBernouilliNumber,
	mathAkiyamaTanigawaAlgorithm: mathAkiyamaTanigawaAlgorithm,
	cCreateComplexNumber: cCreateComplexNumber,
	cCreatePolarComplexNumber: cCreatePolarComplexNumber,
	cAdd: cAdd,
	cAddToNew: cAddToNew,
	cSub: cSub,
	cSubToNew: cSubToNew,
	cMul: cMul,
	cMulToNew: cMulToNew,
	cDiv: cDiv,
	cDivToNew: cDivToNew,
	cConjugate: cConjugate,
	cConjugateToNew: cConjugateToNew,
	cAbs: cAbs,
	cArg: cArg,
	cCreatePolarFromComplexNumber: cCreatePolarFromComplexNumber,
	cCreateComplexFromPolar: cCreateComplexFromPolar,
	cRe: cRe,
	cIm: cIm,
	cAddPolar: cAddPolar,
	cAddPolarToNew: cAddPolarToNew,
	cSubPolar: cSubPolar,
	cSubPolarToNew: cSubPolarToNew,
	cMulPolar: cMulPolar,
	cMulPolarToNew: cMulPolarToNew,
	cDivPolar: cDivPolar,
	cDivPolarToNew: cDivPolarToNew,
	cSquareRoot: cSquareRoot,
	cPowerPolar: cPowerPolar,
	cPowerToNew: cPowerToNew,
	cPower: cPower,
	cNegate: cNegate,
	cAssignComplexByValues: cAssignComplexByValues,
	cAssignComplex: cAssignComplex,
	cEpsilonCompareComplex: cEpsilonCompareComplex,
	cExpComplex: cExpComplex,
	cSineComplex: cSineComplex,
	cCosineComplex: cCosineComplex,
	cComplexToString: cComplexToString,
	pPolynomialToTextDirect: pPolynomialToTextDirect,
	pLinkedListCharactersAddString: pLinkedListCharactersAddString,
	pGenerateCommonRenderSpecification: pGenerateCommonRenderSpecification,
	pPolynomialToText: pPolynomialToText,
	pComplexPolynomialToTextDirect: pComplexPolynomialToTextDirect,
	pAdd: pAdd,
	pSubtract: pSubtract,
	pMultiply: pMultiply,
	pDivide: pDivide,
	pIsZero: pIsZero,
	pAssign: pAssign,
	pCreatePolynomial: pCreatePolynomial,
	pFill: pFill,
	pDegree: pDegree,
	pLead: pLead,
	pEvaluate: pEvaluate,
	pEvaluateWithHornersMethod: pEvaluateWithHornersMethod,
	pEvaluateWithPowers: pEvaluateWithPowers,
	pEvaluateDerivative: pEvaluateDerivative,
	pDerivative: pDerivative,
	pAddComplex: pAddComplex,
	pSubtractComplex: pSubtractComplex,
	pIsZeroComplex: pIsZeroComplex,
	pAssignComplex: pAssignComplex,
	pCreateComplexPolynomial: pCreateComplexPolynomial,
	pFillComplex: pFillComplex,
	pDegreeComplex: pDegreeComplex,
	pLeadComplex: pLeadComplex,
	pEvaluateComplex: pEvaluateComplex,
	pTotalNumberOfRoots: pTotalNumberOfRoots,
	WriteStringToStingStream: WriteStringToStingStream,
	WriteCharacterToStingStream: WriteCharacterToStingStream,
	WriteBooleanToStingStream: WriteBooleanToStingStream,
	SubstringWithCheck: SubstringWithCheck,
	Substring: Substring,
	AppendString: AppendString,
	ConcatenateString: ConcatenateString,
	AppendCharacter: AppendCharacter,
	ConcatenateCharacter: ConcatenateCharacter,
	SplitByCharacter: SplitByCharacter,
	IndexOfCharacter: IndexOfCharacter,
	SubstringEqualsWithCheck: SubstringEqualsWithCheck,
	SubstringEquals: SubstringEquals,
	IndexOfString: IndexOfString,
	ContainsCharacter: ContainsCharacter,
	ContainsString: ContainsString,
	ToUpperCase: ToUpperCase,
	ToLowerCase: ToLowerCase,
	EqualsIgnoreCase: EqualsIgnoreCase,
	ReplaceString: ReplaceString,
	ReplaceCharacter: ReplaceCharacter,
	Trim: Trim,
	StartsWith: StartsWith,
	EndsWith: EndsWith,
	SplitByString: SplitByString,
	StringIsBefore: StringIsBefore,
	nCreateStringScientificNotationDecimalFromNumber: nCreateStringScientificNotationDecimalFromNumber,
	nCreateStringDecimalFromNumber: nCreateStringDecimalFromNumber,
	nCreateStringFromNumberWithCheck: nCreateStringFromNumberWithCheck,
	nGetMaximumDigitsForBase: nGetMaximumDigitsForBase,
	nGetFirstDigitPosition: nGetFirstDigitPosition,
	nGetSingleDigitCharacterFromNumberWithCheck: nGetSingleDigitCharacterFromNumberWithCheck,
	nGetDigitCharacterTable: nGetDigitCharacterTable,
	nCreateNumberFromDecimalStringWithCheck: nCreateNumberFromDecimalStringWithCheck,
	nCreateNumberFromDecimalString: nCreateNumberFromDecimalString,
	nCreateNumberFromStringWithCheck: nCreateNumberFromStringWithCheck,
	nCreateNumberFromParts: nCreateNumberFromParts,
	nExtractPartsFromNumberString: nExtractPartsFromNumberString,
	nExtractPartsFromNumberStringFromSign: nExtractPartsFromNumberStringFromSign,
	nExtractPartsFromNumberStringFromPointOrExponent: nExtractPartsFromNumberStringFromPointOrExponent,
	nExtractPartsFromNumberStringFromExponent: nExtractPartsFromNumberStringFromExponent,
	nGetNumberFromNumberCharacterForBase: nGetNumberFromNumberCharacterForBase,
	nCharacterIsNumberCharacterInBase: nCharacterIsNumberCharacterInBase,
	nStringToNumberArray: nStringToNumberArray,
	nStringToNumberArrayWithCheck: nStringToNumberArrayWithCheck,
	AssertFalse: AssertFalse,
	AssertTrue: AssertTrue,
	AssertEquals: AssertEquals,
	AssertBooleansEqual: AssertBooleansEqual,
	AssertCharactersEqual: AssertCharactersEqual,
	AssertStringEquals: AssertStringEquals,
	AssertNumberArraysEqual: AssertNumberArraysEqual,
	AssertBooleanArraysEqual: AssertBooleanArraysEqual,
	AssertStringArraysEqual: AssertStringArraysEqual,
	AddNumber: AddNumber,
	AddNumberRef: AddNumberRef,
	RemoveNumber: RemoveNumber,
	GetNumberRef: GetNumberRef,
	RemoveNumberRef: RemoveNumberRef,
	AddString: AddString,
	AddStringRef: AddStringRef,
	RemoveString: RemoveString,
	GetStringRef: GetStringRef,
	RemoveStringRef: RemoveStringRef,
	AddBoolean: AddBoolean,
	AddBooleanRef: AddBooleanRef,
	RemoveBoolean: RemoveBoolean,
	GetBooleanRef: GetBooleanRef,
	RemoveDecimalRef: RemoveDecimalRef,
	CreateLinkedListString: CreateLinkedListString,
	LinkedListAddString: LinkedListAddString,
	LinkedListStringsToArray: LinkedListStringsToArray,
	LinkedListStringsLength: LinkedListStringsLength,
	FreeLinkedListString: FreeLinkedListString,
	CreateLinkedListNumbers: CreateLinkedListNumbers,
	CreateLinkedListNumbersArray: CreateLinkedListNumbersArray,
	LinkedListAddNumber: LinkedListAddNumber,
	LinkedListNumbersLength: LinkedListNumbersLength,
	LinkedListNumbersIndex: LinkedListNumbersIndex,
	LinkedListInsertNumber: LinkedListInsertNumber,
	LinkedListSet: LinkedListSet,
	LinkedListRemoveNumber: LinkedListRemoveNumber,
	FreeLinkedListNumbers: FreeLinkedListNumbers,
	FreeLinkedListNumbersArray: FreeLinkedListNumbersArray,
	LinkedListNumbersToArray: LinkedListNumbersToArray,
	ArrayToLinkedListNumbers: ArrayToLinkedListNumbers,
	LinkedListNumbersEqual: LinkedListNumbersEqual,
	CreateLinkedListCharacter: CreateLinkedListCharacter,
	LinkedListAddCharacter: LinkedListAddCharacter,
	LinkedListCharactersToArray: LinkedListCharactersToArray,
	LinkedListCharactersLength: LinkedListCharactersLength,
	FreeLinkedListCharacter: FreeLinkedListCharacter,
	CreateDynamicArrayNumbers: CreateDynamicArrayNumbers,
	CreateDynamicArrayNumbersWithInitialCapacity: CreateDynamicArrayNumbersWithInitialCapacity,
	DynamicArrayAddNumber: DynamicArrayAddNumber,
	DynamicArrayNumbersIncreaseSize: DynamicArrayNumbersIncreaseSize,
	DynamicArrayNumbersDecreaseSizeNecessary: DynamicArrayNumbersDecreaseSizeNecessary,
	DynamicArrayNumbersDecreaseSize: DynamicArrayNumbersDecreaseSize,
	DynamicArrayNumbersIndex: DynamicArrayNumbersIndex,
	DynamicArrayNumbersLength: DynamicArrayNumbersLength,
	DynamicArrayInsertNumber: DynamicArrayInsertNumber,
	DynamicArraySet: DynamicArraySet,
	DynamicArrayRemoveNumber: DynamicArrayRemoveNumber,
	FreeDynamicArrayNumbers: FreeDynamicArrayNumbers,
	DynamicArrayNumbersToArray: DynamicArrayNumbersToArray,
	ArrayToDynamicArrayNumbersWithOptimalSize: ArrayToDynamicArrayNumbersWithOptimalSize,
	ArrayToDynamicArrayNumbers: ArrayToDynamicArrayNumbers,
	DynamicArrayNumbersEqual: DynamicArrayNumbersEqual,
	DynamicArrayNumbersToLinkedList: DynamicArrayNumbersToLinkedList,
	LinkedListToDynamicArrayNumbers: LinkedListToDynamicArrayNumbers,
	AddCharacter: AddCharacter,
	AddCharacterRef: AddCharacterRef,
	RemoveCharacter: RemoveCharacter,
	GetCharacterRef: GetCharacterRef,
	RemoveCharacterRef: RemoveCharacterRef,
	charToLowerCase: charToLowerCase,
	charToUpperCase: charToUpperCase,
	charIsUpperCase: charIsUpperCase,
	charIsLowerCase: charIsLowerCase,
	charIsLetter: charIsLetter,
	charIsNumber: charIsNumber,
	charIsWhiteSpace: charIsWhiteSpace,
	charIsSymbol: charIsSymbol,
	charCharacterIsBefore: charCharacterIsBefore,
	StringToNumberArray: StringToNumberArray,
	NumberArrayToString: NumberArrayToString,
	NumberArraysEqual: NumberArraysEqual,
	BooleanArraysEqual: BooleanArraysEqual,
	StringsEqual: StringsEqual,
	FillNumberArray: FillNumberArray,
	FillString: FillString,
	FillBooleanArray: FillBooleanArray,
	FillNumberArrayRange: FillNumberArrayRange,
	FillBooleanArrayRange: FillBooleanArrayRange,
	FillStringRange: FillStringRange,
	CopyNumberArray: CopyNumberArray,
	CopyBooleanArray: CopyBooleanArray,
	CopyString: CopyString,
	CopyNumberArrayRange: CopyNumberArrayRange,
	CopyBooleanArrayRange: CopyBooleanArrayRange,
	CopyStringRange: CopyStringRange,
	IsLastElement: IsLastElement,
	CreateNumberArray: CreateNumberArray,
	CreateBooleanArray: CreateBooleanArray,
	CreateString: CreateString,
	SwapElementsOfNumberArray: SwapElementsOfNumberArray,
	SwapElementsOfStringArray: SwapElementsOfStringArray,
	ReverseNumberArray: ReverseNumberArray,
	test: test
};